import datetime
import typing
from typing import Any, Dict, Union

import bqplot
import ipywidgets
import traitlets
from numpy import ndarray

import react_ipywidgets as react
from react_ipywidgets.core import Element
import numpy as np
from .ipywidgets import Layout

# def wrap(mod, globals):
#     for cls_name in dir(mod):
#         cls = getattr(mod, cls_name)
#         if isclass(cls) and issubclass(cls, widgets.Widget):
#             globals[cls_name] = react.component(cls)


# # del g

# wrap(bqplot, globals())
# del bqplot

if __name__ == "__main__":

    from .generate import generate

    ignore_traits = "domain_class".split()
    generate(__file__, [bqplot], ignore_traits=ignore_traits)

# generated code:


def Albers(
    allow_padding: bool = True,
    center: tuple = (0, 60),
    parallels: tuple = (29.5, 45.5),
    precision: float = 0.1,
    reverse: bool = False,
    rotate: tuple = (96, 0),
    scale_factor: float = 250,
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_center: typing.Callable[[tuple], Any] = None,
    on_parallels: typing.Callable[[tuple], Any] = None,
    on_precision: typing.Callable[[float], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
    on_rotate: typing.Callable[[tuple], Any] = None,
    on_scale_factor: typing.Callable[[float], Any] = None,
) -> Element[bqplot.scales.Albers]:
    """A geographical scale which is an alias for a conic equal area projection.

    The Albers projection is a conic equal area map. It does not preserve scale
    or shape, though it is recommended for chloropleths since it preserves the
    relative areas of geographic features. Default values are US-centric.

    Attributes
    ----------
    scale_factor: float (default: 250)
        Specifies the scale value for the projection
    rotate: tuple (default: (96, 0))
        Degree of rotation in each axis.
    parallels: tuple (default: (29.5, 45.5))
        Sets the two parallels for the conic projection.
    center: tuple (default: (0, 60))
        Specifies the longitude and latitude where the map is centered.
    precision: float (default: 0.1)
        Specifies the threshold for the projections adaptive resampling to the
        specified value in pixels.
    rtype: (Number, Number) (class-level attribute)
        This attribute should not be modified. The range type of a geo
        scale is a tuple.
    dtype: type (class-level attribute)
        the associated data type / domain type


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        center=center,
        parallels=parallels,
        precision=precision,
        reverse=reverse,
        rotate=rotate,
        scale_factor=scale_factor,
        on_allow_padding=on_allow_padding,
        on_center=on_center,
        on_parallels=on_parallels,
        on_precision=on_precision,
        on_reverse=on_reverse,
        on_rotate=on_rotate,
        on_scale_factor=on_scale_factor,
    )
    widget_cls = bqplot.scales.Albers
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def AlbersUSA(
    allow_padding: bool = True,
    reverse: bool = False,
    scale_factor: float = 1200,
    translate: tuple = (600, 490),
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
    on_scale_factor: typing.Callable[[float], Any] = None,
    on_translate: typing.Callable[[tuple], Any] = None,
) -> Element[bqplot.scales.AlbersUSA]:
    """A composite projection of four Albers projections meant specifically for
    the United States.

    Attributes
    ----------
    scale_factor: float (default: 1200)
        Specifies the scale value for the projection
    translate: tuple (default: (600, 490))
    rtype: (Number, Number) (class-level attribute)
        This attribute should not be modified. The range type of a geo
        scale is a tuple.
    dtype: type (class-level attribute)
        the associated data type / domain type


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        reverse=reverse,
        scale_factor=scale_factor,
        translate=translate,
        on_allow_padding=on_allow_padding,
        on_reverse=on_reverse,
        on_scale_factor=on_scale_factor,
        on_translate=on_translate,
    )
    widget_cls = bqplot.scales.AlbersUSA
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Axis(
    color: str = None,
    grid_color: str = None,
    grid_lines: str = "solid",
    label: str = "",
    label_color: str = None,
    label_location: str = "middle",
    label_offset: str = None,
    num_ticks: int = None,
    offset: dict = {},
    orientation: str = "horizontal",
    scale: Element[bqplot.scales.Scale] = None,
    side: str = None,
    tick_format: str = None,
    tick_rotate: int = 0,
    tick_style: dict = {},
    tick_values: ndarray = None,
    visible: bool = True,
    on_color: typing.Callable[[str], Any] = None,
    on_grid_color: typing.Callable[[str], Any] = None,
    on_grid_lines: typing.Callable[[str], Any] = None,
    on_label: typing.Callable[[str], Any] = None,
    on_label_color: typing.Callable[[str], Any] = None,
    on_label_location: typing.Callable[[str], Any] = None,
    on_label_offset: typing.Callable[[str], Any] = None,
    on_num_ticks: typing.Callable[[int], Any] = None,
    on_offset: typing.Callable[[dict], Any] = None,
    on_orientation: typing.Callable[[str], Any] = None,
    on_scale: typing.Callable[[Element[bqplot.scales.Scale]], Any] = None,
    on_side: typing.Callable[[str], Any] = None,
    on_tick_format: typing.Callable[[str], Any] = None,
    on_tick_rotate: typing.Callable[[int], Any] = None,
    on_tick_style: typing.Callable[[dict], Any] = None,
    on_tick_values: typing.Callable[[ndarray], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.axes.Axis]:
    """A line axis.

    A line axis is the visual representation of a numerical or date scale.

    Attributes
    ----------
    icon: string (class-level attribute)
        The font-awesome icon name for this object.
    axis_types: dict (class-level attribute)
        A registry of existing axis types.
    orientation: {'horizontal', 'vertical'}
        The orientation of the axis, either vertical or horizontal
    side: {'bottom', 'top', 'left', 'right'} or None (default: None)
        The side of the axis, either bottom, top, left or right.
    label: string (default: '')
        The axis label
    tick_format: string or None (default: '')
        The tick format for the axis, for dates use d3 string formatting.
    scale: Scale
        The scale represented by the axis
    num_ticks: int or None (default: None)
        If tick_values is None, number of ticks
    tick_values: numpy.ndarray or None (default: None)
        Tick values for the axis
    offset: dict (default: {})
        Contains a scale and a value {'scale': scale or None,
        'value': value of the offset}
        If offset['scale'] is None, the corresponding figure scale is used
        instead.
    label_location: {'middle', 'start', 'end'}
        The location of the label along the axis, one of 'start', 'end' or
        'middle'
    label_color: Color or None (default: None)
        The color of the axis label
    grid_lines: {'none', 'solid', 'dashed'}
        The display of the grid lines
    grid_color: Color or None (default: None)
        The color of the grid lines
    color: Color or None (default: None)
        The color of the line
    label_offset: string or None (default: None)
        Label displacement from the axis line. Units allowed are 'em', 'px'
        and 'ex'. Positive values are away from the figure and negative
        values are towards the figure with respect to the axis line.
    visible: bool (default: True)
        A visibility toggle for the axis
    tick_style: Dict (default: {})
        Dictionary containing the CSS-style of the text for the ticks.
        For example: font-size of the text can be changed by passing
        `{'font-size': 14}`
    tick_rotate: int (default: 0)
        Degrees to rotate tick labels by.


    """

    kwargs: Dict[Any, Any] = dict(
        color=color,
        grid_color=grid_color,
        grid_lines=grid_lines,
        label=label,
        label_color=label_color,
        label_location=label_location,
        label_offset=label_offset,
        num_ticks=num_ticks,
        offset=offset,
        orientation=orientation,
        scale=scale,
        side=side,
        tick_format=tick_format,
        tick_rotate=tick_rotate,
        tick_style=tick_style,
        tick_values=tick_values,
        visible=visible,
        on_color=on_color,
        on_grid_color=on_grid_color,
        on_grid_lines=on_grid_lines,
        on_label=on_label,
        on_label_color=on_label_color,
        on_label_location=on_label_location,
        on_label_offset=on_label_offset,
        on_num_ticks=on_num_ticks,
        on_offset=on_offset,
        on_orientation=on_orientation,
        on_scale=on_scale,
        on_side=on_side,
        on_tick_format=on_tick_format,
        on_tick_rotate=on_tick_rotate,
        on_tick_style=on_tick_style,
        on_tick_values=on_tick_values,
        on_visible=on_visible,
    )
    widget_cls = bqplot.axes.Axis
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Bars(
    align: str = "center",
    apply_clip: bool = True,
    base: float = 0.0,
    color: ndarray = None,
    color_mode: str = "auto",
    colors: list = ["steelblue"],
    display_legend: bool = False,
    enable_hover: bool = True,
    fill: bool = True,
    interactions: dict = {"hover": "tooltip"},
    label_display: bool = False,
    label_display_format: str = ".2f",
    label_display_horizontal_offset: float = 0.0,
    label_display_vertical_offset: float = 0.0,
    label_font_style: dict = {},
    labels: list = [],
    opacities: list = [],
    opacity_mode: str = "auto",
    orientation: str = "vertical",
    padding: float = 0.05,
    preserve_domain: dict = {},
    scales: dict = {},
    scales_metadata: dict = {
        "x": {"orientation": "horizontal", "dimension": "x"},
        "y": {"orientation": "vertical", "dimension": "y"},
        "color": {"dimension": "color"},
    },
    selected: ndarray = None,
    selected_style: dict = {},
    stroke: str = None,
    stroke_width: float = 1.0,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    type: str = "stacked",
    unselected_style: dict = {},
    visible: bool = True,
    x: ndarray = np.array([]),
    y: ndarray = np.array([]),
    on_align: typing.Callable[[str], Any] = None,
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_base: typing.Callable[[float], Any] = None,
    on_color: typing.Callable[[ndarray], Any] = None,
    on_color_mode: typing.Callable[[str], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_fill: typing.Callable[[bool], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_label_display: typing.Callable[[bool], Any] = None,
    on_label_display_format: typing.Callable[[str], Any] = None,
    on_label_display_horizontal_offset: typing.Callable[[float], Any] = None,
    on_label_display_vertical_offset: typing.Callable[[float], Any] = None,
    on_label_font_style: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_opacities: typing.Callable[[list], Any] = None,
    on_opacity_mode: typing.Callable[[str], Any] = None,
    on_orientation: typing.Callable[[str], Any] = None,
    on_padding: typing.Callable[[float], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_stroke: typing.Callable[[str], Any] = None,
    on_stroke_width: typing.Callable[[float], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_type: typing.Callable[[str], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_x: typing.Callable[[ndarray], Any] = None,
    on_y: typing.Callable[[ndarray], Any] = None,
) -> Element[bqplot.marks.Bars]:
    """Bar mark.

    In the case of the Bars mark, scales for 'x' and 'y'  MUST be provided.
    The scales of other data attributes are optional. In the case where another
    data attribute than 'x' or 'y' is provided but the corresponding scale is
    missing, the data attribute is ignored.

    Attributes
    ----------
    icon: string (class-level attribute)
        font-awesome icon for that mark
    name: string (class-level attribute)
        user-friendly name of the mark
    color_mode: {'auto', 'group', 'element', 'no_group'}
        Specify how default colors are applied to bars.
        The 'group' mode means colors are assigned per group. If the list
        of colors is shorter than the number of groups, colors are reused.
        The 'element' mode means colors are assigned per group element. If the list
        of colors is shorter than the number of bars in a group, colors are reused.
        The 'no_group' mode means colors are assigned per bar, discarding the fact
        that there are groups or stacks. If the list of colors is shorter than the
        total number of bars, colors are reused.
    opacity_mode: {'auto', 'group', 'element', 'no_group'}
        Same as the `color_mode` attribute, but for the opacity.
    type: {'stacked', 'grouped'}
        whether 2-dimensional bar charts should appear grouped or stacked.
    colors: list of colors (default: ['steelblue'])
        list of colors for the bars.
    orientation: {'horizontal', 'vertical'}
        Specifies whether the bar chart is drawn horizontally or vertically.
        If a horizontal bar chart is drawn, the x data is drawn vertically.
    padding: float (default: 0.05)
        Attribute to control the spacing between the bars value is specified
        as a percentage of the width of the bar
    fill: Bool (default: True)
        Whether to fill the bars or not
    stroke: Color or None (default: None)
        Stroke color for the bars
    stroke_width: Float (default: 0.)
        Stroke width of the bars
    opacities: list of floats (default: [])
        Opacities for the bars. Defaults to 1 when the list is too
        short, or the element of the list is set to None.
    base: float (default: 0.0)
        reference value from which the bars are drawn. defaults to 0.0
    align: {'center', 'left', 'right'}
        alignment of bars with respect to the tick value
    label_display: bool (default: False)
        whether or not to display bar data labels
    label_display_format: string (default: .2f)
        format for displaying values.
    label_font_style: dict
        CSS style for the text of each cell
    label_display_vertical_offset: float
        vertical offset value for the label display
    label_display_horizontal_offset: float
        horizontal offset value for the label display

    Data Attributes

    x: numpy.ndarray (default: [])
        abscissas of the data points (1d array)
    y: numpy.ndarray (default: [])
        ordinates of the values for the data points
    color: numpy.ndarray or None (default: None)
        color of the data points (1d array). Defaults to default_color when not
        provided or when a value is NaN

    Notes
    -----
    The fields which can be passed to the default tooltip are:
        All the data attributes
        index: index of the bar being hovered on
        sub_index: if data is two dimensional, this is the minor index


    """

    kwargs: Dict[Any, Any] = dict(
        align=align,
        apply_clip=apply_clip,
        base=base,
        color=color,
        color_mode=color_mode,
        colors=colors,
        display_legend=display_legend,
        enable_hover=enable_hover,
        fill=fill,
        interactions=interactions,
        label_display=label_display,
        label_display_format=label_display_format,
        label_display_horizontal_offset=label_display_horizontal_offset,
        label_display_vertical_offset=label_display_vertical_offset,
        label_font_style=label_font_style,
        labels=labels,
        opacities=opacities,
        opacity_mode=opacity_mode,
        orientation=orientation,
        padding=padding,
        preserve_domain=preserve_domain,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        stroke=stroke,
        stroke_width=stroke_width,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        type=type,
        unselected_style=unselected_style,
        visible=visible,
        x=x,
        y=y,
        on_align=on_align,
        on_apply_clip=on_apply_clip,
        on_base=on_base,
        on_color=on_color,
        on_color_mode=on_color_mode,
        on_colors=on_colors,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_fill=on_fill,
        on_interactions=on_interactions,
        on_label_display=on_label_display,
        on_label_display_format=on_label_display_format,
        on_label_display_horizontal_offset=on_label_display_horizontal_offset,
        on_label_display_vertical_offset=on_label_display_vertical_offset,
        on_label_font_style=on_label_font_style,
        on_labels=on_labels,
        on_opacities=on_opacities,
        on_opacity_mode=on_opacity_mode,
        on_orientation=on_orientation,
        on_padding=on_padding,
        on_preserve_domain=on_preserve_domain,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_stroke=on_stroke,
        on_stroke_width=on_stroke_width,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_type=on_type,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
        on_x=on_x,
        on_y=on_y,
    )
    widget_cls = bqplot.marks.Bars
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def BaseAxis() -> Element[bqplot.axes.BaseAxis]:
    """ """

    kwargs: Dict[Any, Any] = dict()
    widget_cls = bqplot.axes.BaseAxis
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Bins(
    align: str = "center",
    apply_clip: bool = True,
    base: float = 0.0,
    bins: typing.Union[int, list, str] = traitlets.Undefined,
    color: ndarray = None,
    color_mode: str = "auto",
    colors: list = ["steelblue"],
    density: bool = False,
    display_legend: bool = False,
    enable_hover: bool = True,
    fill: bool = True,
    interactions: dict = {"hover": "tooltip"},
    label_display: bool = False,
    label_display_format: str = ".2f",
    label_display_horizontal_offset: float = 0.0,
    label_display_vertical_offset: float = 0.0,
    label_font_style: dict = {},
    labels: list = [],
    max: float = None,
    min: float = None,
    opacities: list = [],
    opacity_mode: str = "auto",
    orientation: str = "vertical",
    padding: float = 0.05,
    preserve_domain: dict = {},
    sample: ndarray = np.array([]),
    scales: dict = {},
    scales_metadata: dict = {
        "x": {"orientation": "horizontal", "dimension": "x"},
        "y": {"orientation": "vertical", "dimension": "y"},
        "color": {"dimension": "color"},
    },
    selected: ndarray = None,
    selected_style: dict = {},
    stroke: str = None,
    stroke_width: float = 1.0,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    type: str = "stacked",
    unselected_style: dict = {},
    visible: bool = True,
    x: ndarray = np.array([]),
    y: ndarray = np.array([]),
    on_align: typing.Callable[[str], Any] = None,
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_base: typing.Callable[[float], Any] = None,
    on_bins: typing.Callable[[typing.Union[int, list, str]], Any] = None,
    on_color: typing.Callable[[ndarray], Any] = None,
    on_color_mode: typing.Callable[[str], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_density: typing.Callable[[bool], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_fill: typing.Callable[[bool], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_label_display: typing.Callable[[bool], Any] = None,
    on_label_display_format: typing.Callable[[str], Any] = None,
    on_label_display_horizontal_offset: typing.Callable[[float], Any] = None,
    on_label_display_vertical_offset: typing.Callable[[float], Any] = None,
    on_label_font_style: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_max: typing.Callable[[float], Any] = None,
    on_min: typing.Callable[[float], Any] = None,
    on_opacities: typing.Callable[[list], Any] = None,
    on_opacity_mode: typing.Callable[[str], Any] = None,
    on_orientation: typing.Callable[[str], Any] = None,
    on_padding: typing.Callable[[float], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_sample: typing.Callable[[ndarray], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_stroke: typing.Callable[[str], Any] = None,
    on_stroke_width: typing.Callable[[float], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_type: typing.Callable[[str], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_x: typing.Callable[[ndarray], Any] = None,
    on_y: typing.Callable[[ndarray], Any] = None,
) -> Element[bqplot.marks.Bins]:
    """Backend histogram mark.

    A `Bars` instance that bins sample data.

    It is very similar in purpose to the `Hist` mark, the difference being that
    the binning is done in the backend (python), which avoids large amounts of
    data being shipped back and forth to the frontend. It should therefore be
    preferred for large data.
    The binning method is the numpy `histogram` method.

    The following  documentation is in part taken from the numpy documentation.

    Attributes
    ----------
    icon: string (class-level attribute)
        font-awesome icon for that mark
    name: string (class-level attribute)
        user-friendly name of the mark
    bins: nonnegative int (default: 10)
          or {'auto', 'fd', 'doane', 'scott', 'rice', 'sturges', 'sqrt'}
        If `bins` is an int, it defines the number of equal-width
        bins in the given range (10, by default).
        If `bins` is a string (method name), `histogram` will use
        the method chosen to calculate the optimal bin width and
        consequently the number of bins (see `Notes` for more detail on
        the estimators) from the data that falls within the requested
        range.
    density : bool (default: `False`)
        If `False`, the height of each bin is the number of samples in it.
        If `True`, the height of each bin is the value of the
        probability *density* function at the bin, normalized such that
        the *integral* over the range is 1. Note that the sum of the
        histogram values will not be equal to 1 unless bins of unity
        width are chosen; it is not a probability *mass* function.
    min : float (default: None)
        The lower range of the bins.  If not provided, lower range
        is simply `x.min()`.
    max : float (default: None)
        The upper range of the bins.  If not provided, lower range
        is simply `x.max()`.
    Data Attributes
    sample: numpy.ndarray (default: [])
        sample of which the histogram must be computed.
    Notes
    -----
    The fields which can be passed to the default tooltip are:
        All the `Bars` data attributes (`x`, `y`, `color`)
        index: index of the bin


    """

    kwargs: Dict[Any, Any] = dict(
        align=align,
        apply_clip=apply_clip,
        base=base,
        bins=bins,
        color=color,
        color_mode=color_mode,
        colors=colors,
        density=density,
        display_legend=display_legend,
        enable_hover=enable_hover,
        fill=fill,
        interactions=interactions,
        label_display=label_display,
        label_display_format=label_display_format,
        label_display_horizontal_offset=label_display_horizontal_offset,
        label_display_vertical_offset=label_display_vertical_offset,
        label_font_style=label_font_style,
        labels=labels,
        max=max,
        min=min,
        opacities=opacities,
        opacity_mode=opacity_mode,
        orientation=orientation,
        padding=padding,
        preserve_domain=preserve_domain,
        sample=sample,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        stroke=stroke,
        stroke_width=stroke_width,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        type=type,
        unselected_style=unselected_style,
        visible=visible,
        x=x,
        y=y,
        on_align=on_align,
        on_apply_clip=on_apply_clip,
        on_base=on_base,
        on_bins=on_bins,
        on_color=on_color,
        on_color_mode=on_color_mode,
        on_colors=on_colors,
        on_density=on_density,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_fill=on_fill,
        on_interactions=on_interactions,
        on_label_display=on_label_display,
        on_label_display_format=on_label_display_format,
        on_label_display_horizontal_offset=on_label_display_horizontal_offset,
        on_label_display_vertical_offset=on_label_display_vertical_offset,
        on_label_font_style=on_label_font_style,
        on_labels=on_labels,
        on_max=on_max,
        on_min=on_min,
        on_opacities=on_opacities,
        on_opacity_mode=on_opacity_mode,
        on_orientation=on_orientation,
        on_padding=on_padding,
        on_preserve_domain=on_preserve_domain,
        on_sample=on_sample,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_stroke=on_stroke,
        on_stroke_width=on_stroke_width,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_type=on_type,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
        on_x=on_x,
        on_y=on_y,
    )
    widget_cls = bqplot.marks.Bins
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Boxplot(
    apply_clip: bool = True,
    auto_detect_outliers: bool = True,
    box_fill_color: str = "steelblue",
    box_width: int = None,
    display_legend: bool = False,
    enable_hover: bool = True,
    interactions: dict = {"hover": "tooltip"},
    labels: list = [],
    opacities: list = [],
    outlier_fill_color: str = "gray",
    preserve_domain: dict = {},
    scales: dict = {},
    scales_metadata: dict = {"x": {"orientation": "horizontal", "dimension": "x"}, "y": {"orientation": "vertical", "dimension": "y"}},
    selected: ndarray = None,
    selected_style: dict = {},
    stroke: str = None,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unselected_style: dict = {},
    visible: bool = True,
    x: ndarray = np.array([]),
    y: ndarray = np.array([[]]),
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_auto_detect_outliers: typing.Callable[[bool], Any] = None,
    on_box_fill_color: typing.Callable[[str], Any] = None,
    on_box_width: typing.Callable[[int], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_opacities: typing.Callable[[list], Any] = None,
    on_outlier_fill_color: typing.Callable[[str], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_stroke: typing.Callable[[str], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_x: typing.Callable[[ndarray], Any] = None,
    on_y: typing.Callable[[ndarray], Any] = None,
) -> Element[bqplot.marks.Boxplot]:
    """Boxplot marks.

    Attributes
    ----------
    stroke: Color or None
        stroke color of the marker
    color: Color
        fill color of the box
    opacities: list of floats (default: [])
        Opacities for the markers of the boxplot. Defaults to 1 when the
        list is too short, or the element of the list is set to None.
    outlier-color: color
        color for the outlier
    box_width: int (default: None)
        width of the box in pixels. The minimum value is 5.
        If set to None, box_with is auto calculated
    auto_detect_outliers: bool (default: True)
        Flag to toggle outlier auto-detection

    Data Attributes

    x: numpy.ndarray (default: [])
        abscissas of the data points (1d array)
    y: numpy.ndarray (default: [[]])
        Sample data points (2d array)


    """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        auto_detect_outliers=auto_detect_outliers,
        box_fill_color=box_fill_color,
        box_width=box_width,
        display_legend=display_legend,
        enable_hover=enable_hover,
        interactions=interactions,
        labels=labels,
        opacities=opacities,
        outlier_fill_color=outlier_fill_color,
        preserve_domain=preserve_domain,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        stroke=stroke,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unselected_style=unselected_style,
        visible=visible,
        x=x,
        y=y,
        on_apply_clip=on_apply_clip,
        on_auto_detect_outliers=on_auto_detect_outliers,
        on_box_fill_color=on_box_fill_color,
        on_box_width=on_box_width,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_interactions=on_interactions,
        on_labels=on_labels,
        on_opacities=on_opacities,
        on_outlier_fill_color=on_outlier_fill_color,
        on_preserve_domain=on_preserve_domain,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_stroke=on_stroke,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
        on_x=on_x,
        on_y=on_y,
    )
    widget_cls = bqplot.marks.Boxplot
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def ColorAxis(
    color: str = None,
    grid_color: str = None,
    grid_lines: str = "solid",
    label: str = "",
    label_color: str = None,
    label_location: str = "middle",
    label_offset: str = None,
    num_ticks: int = None,
    offset: dict = {},
    orientation: str = "horizontal",
    scale: Element[bqplot.scales.ColorScale] = None,
    side: str = "bottom",
    tick_format: str = None,
    tick_rotate: int = 0,
    tick_style: dict = {},
    tick_values: ndarray = None,
    visible: bool = True,
    on_color: typing.Callable[[str], Any] = None,
    on_grid_color: typing.Callable[[str], Any] = None,
    on_grid_lines: typing.Callable[[str], Any] = None,
    on_label: typing.Callable[[str], Any] = None,
    on_label_color: typing.Callable[[str], Any] = None,
    on_label_location: typing.Callable[[str], Any] = None,
    on_label_offset: typing.Callable[[str], Any] = None,
    on_num_ticks: typing.Callable[[int], Any] = None,
    on_offset: typing.Callable[[dict], Any] = None,
    on_orientation: typing.Callable[[str], Any] = None,
    on_scale: typing.Callable[[Element[bqplot.scales.ColorScale]], Any] = None,
    on_side: typing.Callable[[str], Any] = None,
    on_tick_format: typing.Callable[[str], Any] = None,
    on_tick_rotate: typing.Callable[[int], Any] = None,
    on_tick_style: typing.Callable[[dict], Any] = None,
    on_tick_values: typing.Callable[[ndarray], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.axes.ColorAxis]:
    """A colorbar axis.

    A color axis is the visual representation of a color scale.

    Attributes
    ----------
    scale: ColorScale
        The scale represented by the axis


    """

    kwargs: Dict[Any, Any] = dict(
        color=color,
        grid_color=grid_color,
        grid_lines=grid_lines,
        label=label,
        label_color=label_color,
        label_location=label_location,
        label_offset=label_offset,
        num_ticks=num_ticks,
        offset=offset,
        orientation=orientation,
        scale=scale,
        side=side,
        tick_format=tick_format,
        tick_rotate=tick_rotate,
        tick_style=tick_style,
        tick_values=tick_values,
        visible=visible,
        on_color=on_color,
        on_grid_color=on_grid_color,
        on_grid_lines=on_grid_lines,
        on_label=on_label,
        on_label_color=on_label_color,
        on_label_location=on_label_location,
        on_label_offset=on_label_offset,
        on_num_ticks=on_num_ticks,
        on_offset=on_offset,
        on_orientation=on_orientation,
        on_scale=on_scale,
        on_side=on_side,
        on_tick_format=on_tick_format,
        on_tick_rotate=on_tick_rotate,
        on_tick_style=on_tick_style,
        on_tick_values=on_tick_values,
        on_visible=on_visible,
    )
    widget_cls = bqplot.axes.ColorAxis
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def ColorScale(
    allow_padding: bool = True,
    colors: list = [],
    extrapolation: str = "constant",
    max: float = None,
    mid: float = None,
    min: float = None,
    reverse: bool = False,
    scale_type: str = "linear",
    scheme: str = "RdYlGn",
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_extrapolation: typing.Callable[[str], Any] = None,
    on_max: typing.Callable[[float], Any] = None,
    on_mid: typing.Callable[[float], Any] = None,
    on_min: typing.Callable[[float], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
    on_scale_type: typing.Callable[[str], Any] = None,
    on_scheme: typing.Callable[[str], Any] = None,
) -> Element[bqplot.scales.ColorScale]:
    """A color scale.

    A mapping from numbers to colors. The relation is affine by part.

    Attributes
    ----------
    scale_type: {'linear'}
        scale type
    colors: list of colors (default: [])
        list of colors
    min: float or None (default: None)
        if not None, min is the minimal value of the domain
    max: float or None (default: None)
        if not None, max is the maximal value of the domain
    mid: float or None (default: None)
        if not None, mid is the value corresponding to the mid color.
    scheme: string (default: 'RdYlGn')
        Colorbrewer color scheme of the color scale.
    extrapolation: {'constant', 'linear'} (default: 'constant')
        How to extrapolate values outside the [min, max] domain.
    rtype: string (class-level attribute)
        The range type of a color scale is 'Color'. This should not be modified.
    dtype: type (class-level attribute)
        the associated data type / domain type


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        colors=colors,
        extrapolation=extrapolation,
        max=max,
        mid=mid,
        min=min,
        reverse=reverse,
        scale_type=scale_type,
        scheme=scheme,
        on_allow_padding=on_allow_padding,
        on_colors=on_colors,
        on_extrapolation=on_extrapolation,
        on_max=on_max,
        on_mid=on_mid,
        on_min=on_min,
        on_reverse=on_reverse,
        on_scale_type=on_scale_type,
        on_scheme=on_scheme,
    )
    widget_cls = bqplot.scales.ColorScale
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def DOMWidget(
    layout: Union[Dict[str, Any], Element[ipywidgets.widgets.widget_layout.Layout]] = {},
    on_layout: typing.Callable[[Union[Dict[str, Any], Element[ipywidgets.widgets.widget_layout.Layout]]], Any] = None,
) -> Element[ipywidgets.widgets.domwidget.DOMWidget]:
    """Widget that can be inserted into the DOM"""
    if isinstance(layout, dict):
        layout = Layout(**layout)
    kwargs: Dict[Any, Any] = dict(layout=layout, on_layout=on_layout)
    widget_cls = ipywidgets.widgets.domwidget.DOMWidget
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def DateColorScale(
    allow_padding: bool = True,
    colors: list = [],
    extrapolation: str = "constant",
    max: datetime.datetime = None,
    mid: datetime.datetime = None,
    min: datetime.datetime = None,
    reverse: bool = False,
    scale_type: str = "linear",
    scheme: str = "RdYlGn",
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_extrapolation: typing.Callable[[str], Any] = None,
    on_max: typing.Callable[[datetime.datetime], Any] = None,
    on_mid: typing.Callable[[datetime.datetime], Any] = None,
    on_min: typing.Callable[[datetime.datetime], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
    on_scale_type: typing.Callable[[str], Any] = None,
    on_scheme: typing.Callable[[str], Any] = None,
) -> Element[bqplot.scales.DateColorScale]:
    """A date color scale.

    A mapping from dates to a numerical domain.

    Attributes
    ----------
    min: Date or None (default: None)
        if not None, min is the minimal value of the domain
    max: Date or None (default: None)
        if not None, max is the maximal value of the domain
    mid: Date or None (default: None)
        if not None, mid is the value corresponding to the mid color.
    rtype: string (class-level attribute)
        This attribute should not be modified by the user.
        The range type of a color scale is 'Color'.
    dtype: type (class-level attribute)
        the associated data type / domain type


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        colors=colors,
        extrapolation=extrapolation,
        max=max,
        mid=mid,
        min=min,
        reverse=reverse,
        scale_type=scale_type,
        scheme=scheme,
        on_allow_padding=on_allow_padding,
        on_colors=on_colors,
        on_extrapolation=on_extrapolation,
        on_max=on_max,
        on_mid=on_mid,
        on_min=on_min,
        on_reverse=on_reverse,
        on_scale_type=on_scale_type,
        on_scheme=on_scheme,
    )
    widget_cls = bqplot.scales.DateColorScale
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def DateScale(
    allow_padding: bool = True,
    max: datetime.datetime = None,
    min: datetime.datetime = None,
    reverse: bool = False,
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_max: typing.Callable[[datetime.datetime], Any] = None,
    on_min: typing.Callable[[datetime.datetime], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.scales.DateScale]:
    """A date scale, with customizable formatting.

    An affine mapping from dates to a numerical range.

    Attributes
    ----------
    min: Date or None (default: None)
        if not None, min is the minimal value of the domain
    max: Date (default: None)
        if not None, max is the maximal value of the domain
    domain_class: type (default: Date)
         traitlet type used to validate values in of the domain of the scale.
    rtype: string (class-level attribute)
        This attribute should not be modified by the user.
        The range type of a linear scale is numerical.
    dtype: type (class-level attribute)
        the associated data type / domain type


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        max=max,
        min=min,
        reverse=reverse,
        on_allow_padding=on_allow_padding,
        on_max=on_max,
        on_min=on_min,
        on_reverse=on_reverse,
    )
    widget_cls = bqplot.scales.DateScale
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def EquiRectangular(
    allow_padding: bool = True,
    center: tuple = (0, 60),
    reverse: bool = False,
    scale_factor: float = 145.0,
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_center: typing.Callable[[tuple], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
    on_scale_factor: typing.Callable[[float], Any] = None,
) -> Element[bqplot.scales.EquiRectangular]:
    """An elementary projection that uses the identity function.

    The projection is neither equal-area nor conformal.

    Attributes
    ----------
    scale_factor: float (default: 145)
       Specifies the scale value for the projection
    center: tuple (default: (0, 60))
        Specifies the longitude and latitude where the map is centered.


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        center=center,
        reverse=reverse,
        scale_factor=scale_factor,
        on_allow_padding=on_allow_padding,
        on_center=on_center,
        on_reverse=on_reverse,
        on_scale_factor=on_scale_factor,
    )
    widget_cls = bqplot.scales.EquiRectangular
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Figure(
    animation_duration: int = 0,
    axes: list = [],
    background_style: dict = {},
    fig_margin: dict = {"top": 60, "bottom": 60, "left": 60, "right": 60},
    interaction: Element[bqplot.interacts.Interaction] = None,
    layout: Union[Dict[str, Any], Element[ipywidgets.widgets.widget_layout.Layout]] = {},
    legend_location: str = "top-right",
    legend_style: dict = {},
    legend_text: dict = {},
    marks: list = [],
    max_aspect_ratio: float = 100,
    min_aspect_ratio: float = 0.01,
    padding_x: float = 0.0,
    padding_y: float = 0.025,
    pixel_ratio: float = None,
    scale_x: Element[bqplot.scales.Scale] = None,
    scale_y: Element[bqplot.scales.Scale] = None,
    theme: str = "classic",
    title: str = "",
    title_style: dict = {},
    on_animation_duration: typing.Callable[[int], Any] = None,
    on_axes: typing.Callable[[list], Any] = None,
    on_background_style: typing.Callable[[dict], Any] = None,
    on_fig_margin: typing.Callable[[dict], Any] = None,
    on_interaction: typing.Callable[[Element[bqplot.interacts.Interaction]], Any] = None,
    on_layout: typing.Callable[[Union[Dict[str, Any], Element[ipywidgets.widgets.widget_layout.Layout]]], Any] = None,
    on_legend_location: typing.Callable[[str], Any] = None,
    on_legend_style: typing.Callable[[dict], Any] = None,
    on_legend_text: typing.Callable[[dict], Any] = None,
    on_marks: typing.Callable[[list], Any] = None,
    on_max_aspect_ratio: typing.Callable[[float], Any] = None,
    on_min_aspect_ratio: typing.Callable[[float], Any] = None,
    on_padding_x: typing.Callable[[float], Any] = None,
    on_padding_y: typing.Callable[[float], Any] = None,
    on_pixel_ratio: typing.Callable[[float], Any] = None,
    on_scale_x: typing.Callable[[Element[bqplot.scales.Scale]], Any] = None,
    on_scale_y: typing.Callable[[Element[bqplot.scales.Scale]], Any] = None,
    on_theme: typing.Callable[[str], Any] = None,
    on_title: typing.Callable[[str], Any] = None,
    on_title_style: typing.Callable[[dict], Any] = None,
) -> Element[bqplot.figure.Figure]:
    """Main canvas for drawing a chart.

    The Figure object holds the list of Marks and Axes. It also holds an
    optional Interaction object that is responsible for figure-level mouse
    interactions, the "interaction layer".

    Besides, the Figure object has two reference scales, for positioning items
    in an absolute fashion in the figure canvas.

    Attributes
    ----------
    title: string (default: '')
        title of the figure
    axes: List of Axes (default: [])
        list containing the instances of the axes for the figure
    marks: List of Marks (default: [])
        list containing the marks which are to be appended to the figure
    interaction: Interaction or None (default: None)
        optional interaction layer for the figure
    scale_x: Scale
        Scale representing the x values of the figure
    scale_y: Scale
        Scale representing the y values of the figure
    padding_x: Float (default: 0.0)
        Padding to be applied in the horizontal direction of the figure
        around the data points, proportion of the horizontal length
    padding_y: Float (default: 0.025)
        Padding to be applied in the vertical direction of the figure
        around the data points, proportion of the vertical length
    legend_location: {'top-right', 'top', 'top-left', 'left',
        'bottom-left', 'bottom', 'bottom-right', 'right'}
        location of the legend relative to the center of the figure
    background_style: Dict (default: {})
        CSS style to be applied to the background of the figure
    legend_style: Dict (default: {})
        CSS style to be applied to the SVG legend e.g, {'fill': 'white'}
    legend_text: Dict (default: {})
        CSS style to be applied to the legend text e.g., {'font-size': 20}
    title_style: Dict (default: {})
        CSS style to be applied to the title of the figure
    animation_duration: nonnegative int (default: 0)
        Duration of transition on change of data attributes, in milliseconds.
    pixel_ratio:
        Pixel ratio of the WebGL canvas (2 on retina screens). Set to 1 for better performance,
        but less crisp edges. If set to None it will use the browser's window.devicePixelRatio.

    Layout Attributes

    fig_margin: dict (default: {top=60, bottom=60, left=60, right=60})
        Dictionary containing the top, bottom, left and right margins. The user
        is responsible for making sure that the width and height are greater
        than the sum of the margins.
    min_aspect_ratio: float
         minimum width / height ratio of the figure
    max_aspect_ratio: float
         maximum width / height ratio of the figure

    Methods
    -------

    save_png:
       Saves the figure as a PNG file
    save_svg:
       Saves the figure as an SVG file

    Note
    ----

    The aspect ratios stand for width / height ratios.

     - If the available space is within bounds in terms of min and max aspect
       ratio, we use the entire available space.
     - If the available space is too oblong horizontally, we use the client
       height and the width that corresponds max_aspect_ratio (maximize width
       under the constraints).
     - If the available space is too oblong vertically, we use the client width
       and the height that corresponds to min_aspect_ratio (maximize height
       under the constraint).
       This corresponds to maximizing the area under the constraints.

    Default min and max aspect ratio are both equal to 16 / 9.


    """
    if isinstance(layout, dict):
        layout = Layout(**layout)
    kwargs: Dict[Any, Any] = dict(
        animation_duration=animation_duration,
        axes=axes,
        background_style=background_style,
        fig_margin=fig_margin,
        interaction=interaction,
        layout=layout,
        legend_location=legend_location,
        legend_style=legend_style,
        legend_text=legend_text,
        marks=marks,
        max_aspect_ratio=max_aspect_ratio,
        min_aspect_ratio=min_aspect_ratio,
        padding_x=padding_x,
        padding_y=padding_y,
        pixel_ratio=pixel_ratio,
        scale_x=scale_x,
        scale_y=scale_y,
        theme=theme,
        title=title,
        title_style=title_style,
        on_animation_duration=on_animation_duration,
        on_axes=on_axes,
        on_background_style=on_background_style,
        on_fig_margin=on_fig_margin,
        on_interaction=on_interaction,
        on_layout=on_layout,
        on_legend_location=on_legend_location,
        on_legend_style=on_legend_style,
        on_legend_text=on_legend_text,
        on_marks=on_marks,
        on_max_aspect_ratio=on_max_aspect_ratio,
        on_min_aspect_ratio=on_min_aspect_ratio,
        on_padding_x=on_padding_x,
        on_padding_y=on_padding_y,
        on_pixel_ratio=on_pixel_ratio,
        on_scale_x=on_scale_x,
        on_scale_y=on_scale_y,
        on_theme=on_theme,
        on_title=on_title,
        on_title_style=on_title_style,
    )
    widget_cls = bqplot.figure.Figure
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def FlexLine(
    apply_clip: bool = True,
    color: ndarray = None,
    colors: list = ["#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd", "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"],
    display_legend: bool = False,
    enable_hover: bool = True,
    interactions: dict = {"hover": "tooltip"},
    labels: list = [],
    preserve_domain: dict = {},
    scales: dict = {},
    scales_metadata: dict = {
        "x": {"orientation": "horizontal", "dimension": "x"},
        "y": {"orientation": "vertical", "dimension": "y"},
        "color": {"dimension": "color"},
    },
    selected: ndarray = None,
    selected_style: dict = {},
    stroke_width: float = 1.5,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unselected_style: dict = {},
    visible: bool = True,
    width: ndarray = None,
    x: ndarray = np.array([]),
    y: ndarray = np.array([]),
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_color: typing.Callable[[ndarray], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_stroke_width: typing.Callable[[float], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_width: typing.Callable[[ndarray], Any] = None,
    on_x: typing.Callable[[ndarray], Any] = None,
    on_y: typing.Callable[[ndarray], Any] = None,
) -> Element[bqplot.marks.FlexLine]:
    """Flexible Lines mark.

    In the case of the FlexLines mark, scales for 'x' and 'y' MUST be provided.
    Scales for the color and width data attributes are optional. In the case
    where another data attribute than 'x' or 'y' is provided but the
    corresponding scale is missing, the data attribute is ignored.

    Attributes
    ----------
    name: string (class-level attributes)
        user-friendly name of the mark
    colors: list of colors (default: CATEGORY10)
        List of colors for the Lines
    stroke_width: float (default: 1.5)
        Default stroke width of the Lines

    Data Attributes

    x: numpy.ndarray (default: [])
        abscissas of the data points (1d array)
    y: numpy.ndarray (default: [])
        ordinates of the data points (1d array)
    color: numpy.ndarray or None (default: None)
        Array controlling the color of the data points
    width: numpy.ndarray or None (default: None)
        Array controlling the widths of the Lines.


    """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        color=color,
        colors=colors,
        display_legend=display_legend,
        enable_hover=enable_hover,
        interactions=interactions,
        labels=labels,
        preserve_domain=preserve_domain,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        stroke_width=stroke_width,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unselected_style=unselected_style,
        visible=visible,
        width=width,
        x=x,
        y=y,
        on_apply_clip=on_apply_clip,
        on_color=on_color,
        on_colors=on_colors,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_interactions=on_interactions,
        on_labels=on_labels,
        on_preserve_domain=on_preserve_domain,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_stroke_width=on_stroke_width,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
        on_width=on_width,
        on_x=on_x,
        on_y=on_y,
    )
    widget_cls = bqplot.marks.FlexLine
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def GeoScale(
    allow_padding: bool = True,
    reverse: bool = False,
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.scales.GeoScale]:
    """The base projection scale class for Map marks.

    The GeoScale represents a mapping between topographic data and a
    2d visual representation.


    """

    kwargs: Dict[Any, Any] = dict(allow_padding=allow_padding, reverse=reverse, on_allow_padding=on_allow_padding, on_reverse=on_reverse)
    widget_cls = bqplot.scales.GeoScale
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Gnomonic(
    allow_padding: bool = True,
    center: tuple = (0, 60),
    clip_angle: float = 89.999,
    precision: float = 0.1,
    reverse: bool = False,
    scale_factor: float = 145.0,
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_center: typing.Callable[[tuple], Any] = None,
    on_clip_angle: typing.Callable[[float], Any] = None,
    on_precision: typing.Callable[[float], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
    on_scale_factor: typing.Callable[[float], Any] = None,
) -> Element[bqplot.scales.Gnomonic]:
    """A perspective projection which displays great circles as straight lines.

    The projection is neither equal-area nor conformal.

    Attributes
    ----------
    scale_factor: float (default: 145)
       Specifies the scale value for the projection
    center: tuple (default: (0, 60))
        Specifies the longitude and latitude where the map is centered.
    precision: float (default: 0.1)
        Specifies the threshold for the projections adaptive resampling to the
        specified value in pixels.
    clip_angle: float (default: 89.999)
        Specifies the clipping circle radius to the specified angle in degrees.


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        center=center,
        clip_angle=clip_angle,
        precision=precision,
        reverse=reverse,
        scale_factor=scale_factor,
        on_allow_padding=on_allow_padding,
        on_center=on_center,
        on_clip_angle=on_clip_angle,
        on_precision=on_precision,
        on_reverse=on_reverse,
        on_scale_factor=on_scale_factor,
    )
    widget_cls = bqplot.scales.Gnomonic
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Graph(
    apply_clip: bool = True,
    charge: int = -600,
    color: ndarray = None,
    colors: list = ["#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd", "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"],
    directed: bool = True,
    display_legend: bool = False,
    enable_hover: bool = True,
    highlight_links: bool = True,
    hovered_point: int = None,
    hovered_style: dict = {},
    interactions: dict = {"hover": "tooltip", "click": "select"},
    labels: list = [],
    link_color: ndarray = np.array([]),
    link_data: list = [],
    link_distance: float = 100,
    link_matrix: ndarray = np.array([]),
    link_type: str = "arc",
    node_data: list = [],
    preserve_domain: dict = {},
    scales: dict = {},
    scales_metadata: dict = {
        "x": {"orientation": "horizontal", "dimension": "x"},
        "y": {"orientation": "vertical", "dimension": "y"},
        "color": {"dimension": "color"},
        "link_color": {"dimension": "link_color"},
    },
    selected: ndarray = None,
    selected_style: dict = {},
    static: bool = False,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unhovered_style: dict = {},
    unselected_style: dict = {},
    visible: bool = True,
    x: ndarray = np.array([]),
    y: ndarray = np.array([]),
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_charge: typing.Callable[[int], Any] = None,
    on_color: typing.Callable[[ndarray], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_directed: typing.Callable[[bool], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_highlight_links: typing.Callable[[bool], Any] = None,
    on_hovered_point: typing.Callable[[int], Any] = None,
    on_hovered_style: typing.Callable[[dict], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_link_color: typing.Callable[[ndarray], Any] = None,
    on_link_data: typing.Callable[[list], Any] = None,
    on_link_distance: typing.Callable[[float], Any] = None,
    on_link_matrix: typing.Callable[[ndarray], Any] = None,
    on_link_type: typing.Callable[[str], Any] = None,
    on_node_data: typing.Callable[[list], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_static: typing.Callable[[bool], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unhovered_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_x: typing.Callable[[ndarray], Any] = None,
    on_y: typing.Callable[[ndarray], Any] = None,
) -> Element[bqplot.marks.Graph]:
    """Graph with nodes and links.

    Attributes
    ----------
    node_data: List
        list of node attributes for the graph
    link_matrix: numpy.ndarray of shape(len(nodes), len(nodes))
        link data passed as 2d matrix
    link_data: List
        list of link attributes for the graph
    charge: int (default: -600)
        charge of force layout. Will be ignored when x and y data attributes
        are set
    static: bool (default: False)
        whether the graph is static or not
    link_distance: float (default: 100)
        link distance in pixels between nodes. Will be ignored when x and y
        data attributes are set
    link_type: {'arc', 'line', 'slant_line'} (default: 'arc')
        Enum representing link type
    directed: bool (default: True)
        directed or undirected graph
    highlight_links: bool (default: True)
        highlights incoming and outgoing links when hovered on a node
    colors: list (default: CATEGORY10)
        list of node colors

    Data Attributes

    x: numpy.ndarray (default: [])
        abscissas of the node data points (1d array)
    y: numpy.ndarray (default: [])
        ordinates of the node data points (1d array)
    color: numpy.ndarray or None (default: None)
        color of the node data points (1d array).
    link_color: numpy.ndarray of shape(len(nodes), len(nodes))
        link data passed as 2d matrix


    """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        charge=charge,
        color=color,
        colors=colors,
        directed=directed,
        display_legend=display_legend,
        enable_hover=enable_hover,
        highlight_links=highlight_links,
        hovered_point=hovered_point,
        hovered_style=hovered_style,
        interactions=interactions,
        labels=labels,
        link_color=link_color,
        link_data=link_data,
        link_distance=link_distance,
        link_matrix=link_matrix,
        link_type=link_type,
        node_data=node_data,
        preserve_domain=preserve_domain,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        static=static,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unhovered_style=unhovered_style,
        unselected_style=unselected_style,
        visible=visible,
        x=x,
        y=y,
        on_apply_clip=on_apply_clip,
        on_charge=on_charge,
        on_color=on_color,
        on_colors=on_colors,
        on_directed=on_directed,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_highlight_links=on_highlight_links,
        on_hovered_point=on_hovered_point,
        on_hovered_style=on_hovered_style,
        on_interactions=on_interactions,
        on_labels=on_labels,
        on_link_color=on_link_color,
        on_link_data=on_link_data,
        on_link_distance=on_link_distance,
        on_link_matrix=on_link_matrix,
        on_link_type=on_link_type,
        on_node_data=on_node_data,
        on_preserve_domain=on_preserve_domain,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_static=on_static,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unhovered_style=on_unhovered_style,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
        on_x=on_x,
        on_y=on_y,
    )
    widget_cls = bqplot.marks.Graph
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def GridHeatMap(
    anchor_style: dict = {},
    apply_clip: bool = True,
    color: ndarray = None,
    column: ndarray = None,
    column_align: str = "start",
    display_format: str = None,
    display_legend: bool = False,
    enable_hover: bool = True,
    font_style: dict = {},
    interactions: dict = {"hover": "tooltip"},
    labels: list = [],
    null_color: str = "black",
    opacity: float = 1.0,
    preserve_domain: dict = {},
    row: ndarray = None,
    row_align: str = "start",
    scales: dict = {},
    scales_metadata: dict = {
        "row": {"orientation": "vertical", "dimension": "y"},
        "column": {"orientation": "horizontal", "dimension": "x"},
        "color": {"dimension": "color"},
    },
    selected: ndarray = None,
    selected_style: dict = {},
    stroke: str = "black",
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unselected_style: dict = {},
    visible: bool = True,
    on_anchor_style: typing.Callable[[dict], Any] = None,
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_color: typing.Callable[[ndarray], Any] = None,
    on_column: typing.Callable[[ndarray], Any] = None,
    on_column_align: typing.Callable[[str], Any] = None,
    on_display_format: typing.Callable[[str], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_font_style: typing.Callable[[dict], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_null_color: typing.Callable[[str], Any] = None,
    on_opacity: typing.Callable[[float], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_row: typing.Callable[[ndarray], Any] = None,
    on_row_align: typing.Callable[[str], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_stroke: typing.Callable[[str], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.marks.GridHeatMap]:
    """GridHeatMap mark.

    Alignment: The tiles can be aligned so that the data matches either the
    start, the end or the midpoints of the tiles. This is controlled by the
    align attribute.

    Suppose the data passed is a m-by-n matrix. If the scale for the rows is
    Ordinal, then alignment is by default the mid points. For a non-ordinal
    scale, the data cannot be aligned to the mid points of the rectangles.

    If it is not ordinal, then two cases arise. If the number of rows passed
    is m, then align attribute can be used. If the number of rows passed
    is m+1, then the data are the boundaries of the m rectangles.

    If rows and columns are not passed, and scales for them are also
    not passed, then ordinal scales are generated for the rows and columns.

    Attributes
    ----------
    row_align: Enum(['start', 'end'])
        This is only valid if the number of entries in `row` exactly match the
        number of rows in `color` and the `row_scale` is not `OrdinalScale`.
        `start` aligns the row values passed to be aligned with the start
        of the tiles and `end` aligns the row values to the end of the tiles.
    column_align: Enum(['start', end'])
        This is only valid if the number of entries in `column` exactly
        match the number of columns in `color` and the `column_scale` is
        not `OrdinalScale`. `start` aligns the column values passed to
        be aligned with the start of the tiles and `end` aligns the
        column values to the end of the tiles.
    anchor_style: dict (default: {})
        Controls the style for the element which serves as the anchor during
        selection.
    display_format: string (default: None)
        format for displaying values. If None, then values are not displayed
    font_style: dict
        CSS style for the text of each cell

    Data Attributes

    color: numpy.ndarray or None (default: None)
        color of the data points (2d array). The number of elements in
        this array correspond to the number of cells created in the heatmap.
    row: numpy.ndarray or None (default: None)
        labels for the rows of the `color` array passed. The length of
        this can be no more than 1 away from the number of rows in `color`.
        This is a scaled attribute and can be used to affect the height of the
        cells as the entries of `row` can indicate the start or the end points
        of the cells. Refer to the property `row_align`.
        If this property is None, then a uniformly spaced grid is generated in
        the row direction.
    column: numpy.ndarray or None (default: None)
        labels for the columns of the `color` array passed. The length of
        this can be no more than 1 away from the number of columns in `color`
        This is a scaled attribute and can be used to affect the width of the
        cells as the entries of `column` can indicate the start or the
        end points of the cells. Refer to the property `column_align`.
        If this property is None, then a uniformly spaced grid is generated in
        the column direction.


    """

    kwargs: Dict[Any, Any] = dict(
        anchor_style=anchor_style,
        apply_clip=apply_clip,
        color=color,
        column=column,
        column_align=column_align,
        display_format=display_format,
        display_legend=display_legend,
        enable_hover=enable_hover,
        font_style=font_style,
        interactions=interactions,
        labels=labels,
        null_color=null_color,
        opacity=opacity,
        preserve_domain=preserve_domain,
        row=row,
        row_align=row_align,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        stroke=stroke,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unselected_style=unselected_style,
        visible=visible,
        on_anchor_style=on_anchor_style,
        on_apply_clip=on_apply_clip,
        on_color=on_color,
        on_column=on_column,
        on_column_align=on_column_align,
        on_display_format=on_display_format,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_font_style=on_font_style,
        on_interactions=on_interactions,
        on_labels=on_labels,
        on_null_color=on_null_color,
        on_opacity=on_opacity,
        on_preserve_domain=on_preserve_domain,
        on_row=on_row,
        on_row_align=on_row_align,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_stroke=on_stroke,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
    )
    widget_cls = bqplot.marks.GridHeatMap
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def HeatMap(
    apply_clip: bool = True,
    color: ndarray = None,
    display_legend: bool = False,
    enable_hover: bool = True,
    interactions: dict = {"hover": "tooltip"},
    labels: list = [],
    null_color: str = "black",
    preserve_domain: dict = {},
    scales: dict = {},
    scales_metadata: dict = {
        "x": {"orientation": "horizontal", "dimension": "x"},
        "y": {"orientation": "vertical", "dimension": "y"},
        "color": {"dimension": "color"},
    },
    selected: ndarray = None,
    selected_style: dict = {},
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unselected_style: dict = {},
    visible: bool = True,
    x: ndarray = None,
    y: ndarray = None,
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_color: typing.Callable[[ndarray], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_null_color: typing.Callable[[str], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_x: typing.Callable[[ndarray], Any] = None,
    on_y: typing.Callable[[ndarray], Any] = None,
) -> Element[bqplot.marks.HeatMap]:
    """HeatMap mark.


    Attributes
    ----------

    Data Attributes

    color: numpy.ndarray or None (default: None)
        color of the data points (2d array).
    x: numpy.ndarray or None (default: None)
        labels for the columns of the `color` array passed. The length of
        this has to be the number of columns in `color`.
        This is a scaled attribute.
    y: numpy.ndarray or None (default: None)
        labels for the rows of the `color` array passed. The length of this has
        to be the number of rows in `color`.
        This is a scaled attribute.


    """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        color=color,
        display_legend=display_legend,
        enable_hover=enable_hover,
        interactions=interactions,
        labels=labels,
        null_color=null_color,
        preserve_domain=preserve_domain,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unselected_style=unselected_style,
        visible=visible,
        x=x,
        y=y,
        on_apply_clip=on_apply_clip,
        on_color=on_color,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_interactions=on_interactions,
        on_labels=on_labels,
        on_null_color=on_null_color,
        on_preserve_domain=on_preserve_domain,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
        on_x=on_x,
        on_y=on_y,
    )
    widget_cls = bqplot.marks.HeatMap
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Hist(
    apply_clip: bool = True,
    bins: int = 10,
    colors: list = ["steelblue"],
    count: ndarray = np.array([]),
    display_legend: bool = False,
    enable_hover: bool = True,
    interactions: dict = {"hover": "tooltip"},
    labels: list = [],
    midpoints: list = [],
    normalized: bool = False,
    opacities: list = [],
    preserve_domain: dict = {},
    sample: ndarray = np.array([]),
    scales: dict = {},
    scales_metadata: dict = {"sample": {"orientation": "horizontal", "dimension": "x"}, "count": {"orientation": "vertical", "dimension": "y"}},
    selected: ndarray = None,
    selected_style: dict = {},
    stroke: str = None,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unselected_style: dict = {},
    visible: bool = True,
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_bins: typing.Callable[[int], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_count: typing.Callable[[ndarray], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_midpoints: typing.Callable[[list], Any] = None,
    on_normalized: typing.Callable[[bool], Any] = None,
    on_opacities: typing.Callable[[list], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_sample: typing.Callable[[ndarray], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_stroke: typing.Callable[[str], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.marks.Hist]:
    """Histogram mark.

    In the case of the Hist mark, scales for 'sample' and 'count' MUST be
    provided.

    Attributes
    ----------
    icon: string (class-level attribute)
        font-awesome icon for that mark
    name: string (class-level attribute)
        user-friendly name of the mark
    bins: nonnegative int (default: 10)
        number of bins in the histogram
    normalized: bool (default: False)
        Boolean attribute to return normalized values which
        sum to 1 or direct counts for the `count` attribute. The scale of
        `count` attribute is determined by the value of this flag.
    colors: list of colors (default: ['steelblue'])
        List of colors of the Histogram. If the list is shorter than the number
        of bins, the colors are reused.
    stroke: Color or None (default: None)
        Stroke color of the histogram
    opacities: list of floats (default: [])
        Opacity for the bins of the histogram. Defaults to 1 when the list
        is too short, or the element of the list is set to None.
    midpoints: list (default: [])
        midpoints of the bins of the histogram. It is a read-only attribute.

    Data Attributes

    sample: numpy.ndarray (default: [])
        sample of which the histogram must be computed.
    count: numpy.ndarray (read-only)
        number of sample points per bin. It is a read-only attribute.

    Notes
    -----
    The fields which can be passed to the default tooltip are:
        midpoint: mid-point of the bin related to the rectangle hovered on
        count: number of elements in the bin hovered on
        bin_start: start point of the bin
        bin-end: end point of the bin
        index: index of the bin


    """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        bins=bins,
        colors=colors,
        count=count,
        display_legend=display_legend,
        enable_hover=enable_hover,
        interactions=interactions,
        labels=labels,
        midpoints=midpoints,
        normalized=normalized,
        opacities=opacities,
        preserve_domain=preserve_domain,
        sample=sample,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        stroke=stroke,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unselected_style=unselected_style,
        visible=visible,
        on_apply_clip=on_apply_clip,
        on_bins=on_bins,
        on_colors=on_colors,
        on_count=on_count,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_interactions=on_interactions,
        on_labels=on_labels,
        on_midpoints=on_midpoints,
        on_normalized=on_normalized,
        on_opacities=on_opacities,
        on_preserve_domain=on_preserve_domain,
        on_sample=on_sample,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_stroke=on_stroke,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
    )
    widget_cls = bqplot.marks.Hist
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Image(
    apply_clip: bool = True,
    display_legend: bool = False,
    enable_hover: bool = True,
    image: Element[ipywidgets.widgets.widget_media.Image] = None,
    interactions: dict = {"hover": "tooltip"},
    labels: list = [],
    pixelated: bool = True,
    preserve_domain: dict = {},
    scales: dict = {},
    scales_metadata: dict = {"x": {"orientation": "horizontal", "dimension": "x"}, "y": {"orientation": "vertical", "dimension": "y"}},
    selected: ndarray = None,
    selected_style: dict = {},
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unselected_style: dict = {},
    visible: bool = True,
    x: ndarray = np.array([0, 1]),
    y: ndarray = np.array([0, 1]),
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_image: typing.Callable[[Element[ipywidgets.widgets.widget_media.Image]], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_pixelated: typing.Callable[[bool], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_x: typing.Callable[[ndarray], Any] = None,
    on_y: typing.Callable[[ndarray], Any] = None,
) -> Element[bqplot.marks.Image]:
    """Image mark, based on the ipywidgets image

    If no scales are passed, uses the parent Figure scales.

    Attributes
    ----------
    image: Instance of ipywidgets.Image
        Image to be displayed

    Data Attributes

    x: tuple (default: (0, 1))
        abscissas of the left and right-hand side of the image
        in the format (x0, x1)
    y: tuple (default: (0, 1))
        ordinates of the bottom and top side of the image
        in the format (y0, y1)


    """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        display_legend=display_legend,
        enable_hover=enable_hover,
        image=image,
        interactions=interactions,
        labels=labels,
        pixelated=pixelated,
        preserve_domain=preserve_domain,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unselected_style=unselected_style,
        visible=visible,
        x=x,
        y=y,
        on_apply_clip=on_apply_clip,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_image=on_image,
        on_interactions=on_interactions,
        on_labels=on_labels,
        on_pixelated=on_pixelated,
        on_preserve_domain=on_preserve_domain,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
        on_x=on_x,
        on_y=on_y,
    )
    widget_cls = bqplot.marks.Image
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Interaction() -> Element[bqplot.interacts.Interaction]:
    """The base interaction class.

    An interaction is a mouse interaction layer for a figure that requires the
    capture of all mouse events on the plot area. A consequence is that one can
    allow only one interaction at any time on a figure.

    An interaction can be associated with features such as selection or
    manual change of specific mark. Although, they differ from the so called
    'mark interactions' in that they do not rely on knowing whether a specific
    element of the mark are hovered by the mouse.

    Attributes
    ----------
    types: dict (class-level attribute) representing interaction types
        A registry of existing interaction types.


    """

    kwargs: Dict[Any, Any] = dict()
    widget_cls = bqplot.interacts.Interaction
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Label(
    align: str = "start",
    apply_clip: bool = True,
    color: ndarray = None,
    colors: list = ["#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd", "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"],
    default_size: float = 16.0,
    display_legend: bool = False,
    drag_size: float = 1.0,
    enable_delete: bool = False,
    enable_hover: bool = True,
    enable_move: bool = False,
    font_unit: str = "px",
    font_weight: str = "bold",
    hovered_point: int = None,
    hovered_style: dict = {},
    interactions: dict = {"hover": "tooltip"},
    labels: list = [],
    opacities: ndarray = np.array([1.0]),
    opacity: ndarray = None,
    preserve_domain: dict = {},
    restrict_x: bool = False,
    restrict_y: bool = False,
    rotate_angle: float = 0.0,
    rotation: ndarray = None,
    scales: dict = {},
    scales_metadata: dict = {
        "x": {"orientation": "horizontal", "dimension": "x"},
        "y": {"orientation": "vertical", "dimension": "y"},
        "color": {"dimension": "color"},
        "size": {"dimension": "size"},
        "opacity": {"dimension": "opacity"},
        "rotation": {"dimension": "rotation"},
    },
    selected: ndarray = None,
    selected_style: dict = {},
    size: ndarray = None,
    text: ndarray = None,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unhovered_style: dict = {},
    unselected_style: dict = {},
    update_on_move: bool = False,
    visible: bool = True,
    x: ndarray = np.array([]),
    x_offset: int = 0,
    y: ndarray = np.array([]),
    y_offset: int = 0,
    on_align: typing.Callable[[str], Any] = None,
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_color: typing.Callable[[ndarray], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_default_size: typing.Callable[[float], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_drag_size: typing.Callable[[float], Any] = None,
    on_enable_delete: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_enable_move: typing.Callable[[bool], Any] = None,
    on_font_unit: typing.Callable[[str], Any] = None,
    on_font_weight: typing.Callable[[str], Any] = None,
    on_hovered_point: typing.Callable[[int], Any] = None,
    on_hovered_style: typing.Callable[[dict], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_opacities: typing.Callable[[ndarray], Any] = None,
    on_opacity: typing.Callable[[ndarray], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_restrict_x: typing.Callable[[bool], Any] = None,
    on_restrict_y: typing.Callable[[bool], Any] = None,
    on_rotate_angle: typing.Callable[[float], Any] = None,
    on_rotation: typing.Callable[[ndarray], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_size: typing.Callable[[ndarray], Any] = None,
    on_text: typing.Callable[[ndarray], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unhovered_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_update_on_move: typing.Callable[[bool], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_x: typing.Callable[[ndarray], Any] = None,
    on_x_offset: typing.Callable[[int], Any] = None,
    on_y: typing.Callable[[ndarray], Any] = None,
    on_y_offset: typing.Callable[[int], Any] = None,
) -> Element[bqplot.marks.Label]:
    """Label mark.

    Attributes
    ----------
    x_offset: int (default: 0)
        horizontal offset in pixels from the stated x location
    y_offset: int (default: 0)
        vertical offset in pixels from the stated y location
    text: string (default: '')
        text to be displayed
    default_size: string (default: '14px')
        font size in px, em or ex
    font_weight: {'bold', 'normal', 'bolder'}
        font weight of the caption
    drag_size: nonnegative float (default: 1.)
        Ratio of the size of the dragged label font size to the default
        label font size.
    align: {'start', 'middle', 'end'}
        alignment of the text with respect to the provided location
        enable_move: Bool (default: False)
        Enable the label to be moved by dragging. Refer to restrict_x,
        restrict_y for more options.
    restrict_x: bool (default: False)
        Restricts movement of the label to only along the x axis. This is valid
        only when enable_move is set to True. If both restrict_x and restrict_y
        are set to True, the label cannot be moved.
    restrict_y: bool (default: False)
        Restricts movement of the label to only along the y axis. This is valid
        only when enable_move is set to True. If both restrict_x and restrict_y
        are set to True, the label cannot be moved.

    Data Attributes

    x: numpy.ndarray (default: [])
        horizontal position of the labels, in data coordinates or in
        figure coordinates
    y: numpy.ndarray (default: [])
        vertical position of the labels, in data coordinates or in
        figure coordinates
    color: numpy.ndarray or None (default: None)
        label colors
    size: numpy.ndarray or None (default: None)
        label sizes
    rotation: numpy.ndarray or None (default: None)
        label rotations
    opacity: numpy.ndarray or None (default: None)
        label opacities



    """

    kwargs: Dict[Any, Any] = dict(
        align=align,
        apply_clip=apply_clip,
        color=color,
        colors=colors,
        default_size=default_size,
        display_legend=display_legend,
        drag_size=drag_size,
        enable_delete=enable_delete,
        enable_hover=enable_hover,
        enable_move=enable_move,
        font_unit=font_unit,
        font_weight=font_weight,
        hovered_point=hovered_point,
        hovered_style=hovered_style,
        interactions=interactions,
        labels=labels,
        opacities=opacities,
        opacity=opacity,
        preserve_domain=preserve_domain,
        restrict_x=restrict_x,
        restrict_y=restrict_y,
        rotate_angle=rotate_angle,
        rotation=rotation,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        size=size,
        text=text,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unhovered_style=unhovered_style,
        unselected_style=unselected_style,
        update_on_move=update_on_move,
        visible=visible,
        x=x,
        x_offset=x_offset,
        y=y,
        y_offset=y_offset,
        on_align=on_align,
        on_apply_clip=on_apply_clip,
        on_color=on_color,
        on_colors=on_colors,
        on_default_size=on_default_size,
        on_display_legend=on_display_legend,
        on_drag_size=on_drag_size,
        on_enable_delete=on_enable_delete,
        on_enable_hover=on_enable_hover,
        on_enable_move=on_enable_move,
        on_font_unit=on_font_unit,
        on_font_weight=on_font_weight,
        on_hovered_point=on_hovered_point,
        on_hovered_style=on_hovered_style,
        on_interactions=on_interactions,
        on_labels=on_labels,
        on_opacities=on_opacities,
        on_opacity=on_opacity,
        on_preserve_domain=on_preserve_domain,
        on_restrict_x=on_restrict_x,
        on_restrict_y=on_restrict_y,
        on_rotate_angle=on_rotate_angle,
        on_rotation=on_rotation,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_size=on_size,
        on_text=on_text,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unhovered_style=on_unhovered_style,
        on_unselected_style=on_unselected_style,
        on_update_on_move=on_update_on_move,
        on_visible=on_visible,
        on_x=on_x,
        on_x_offset=on_x_offset,
        on_y=on_y,
        on_y_offset=on_y_offset,
    )
    widget_cls = bqplot.marks.Label
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def LinearScale(
    allow_padding: bool = True,
    max: float = None,
    mid_range: float = 0.8,
    min: float = None,
    min_range: float = 0.6,
    reverse: bool = False,
    stabilized: bool = False,
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_max: typing.Callable[[float], Any] = None,
    on_mid_range: typing.Callable[[float], Any] = None,
    on_min: typing.Callable[[float], Any] = None,
    on_min_range: typing.Callable[[float], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
    on_stabilized: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.scales.LinearScale]:
    """A linear scale.

    An affine mapping from a numerical domain to a numerical range.

    Attributes
    ----------
    min: float or None (default: None)
        if not None, min is the minimal value of the domain
    max: float or None (default: None)
        if not None, max is the maximal value of the domain
    rtype: string (class-level attribute)
        This attribute should not be modified. The range type of a linear
        scale is numerical.
    dtype: type (class-level attribute)
        the associated data type / domain type
    precedence: int (class-level attribute, default_value=2)
        attribute used to determine which scale takes precedence in cases when
        two or more scales have the same rtype and dtype.
        default_value is 2 because for the same range and domain types,
        LinearScale should take precedence.
    stabilized: bool (default: False)
        if set to False, the domain of the scale is tied to the data range
        if set to True, the domain of the scale is updated only when
        the data range is beyond certain thresholds, given by the attributes
        mid_range and min_range.
    mid_range: float (default: 0.8)
        Proportion of the range that is spanned initially.
        Used only if stabilized is True.
    min_range: float (default: 0.6)
        Minimum proportion of the range that should be spanned by the data.
        If the data span falls beneath that level, the scale is reset.
        min_range must be <= mid_range.
        Used only if stabilized is True.


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        max=max,
        mid_range=mid_range,
        min=min,
        min_range=min_range,
        reverse=reverse,
        stabilized=stabilized,
        on_allow_padding=on_allow_padding,
        on_max=on_max,
        on_mid_range=on_mid_range,
        on_min=on_min,
        on_min_range=on_min_range,
        on_reverse=on_reverse,
        on_stabilized=on_stabilized,
    )
    widget_cls = bqplot.scales.LinearScale
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Lines(
    apply_clip: bool = True,
    close_path: bool = False,
    color: ndarray = None,
    colors: list = ["#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd", "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"],
    curves_subset: list = [],
    display_legend: bool = False,
    enable_hover: bool = True,
    fill: str = "none",
    fill_colors: list = [],
    fill_opacities: list = [],
    interactions: dict = {"hover": "tooltip"},
    interpolation: str = "linear",
    labels: list = [],
    labels_visibility: str = "none",
    line_style: str = "solid",
    marker: str = None,
    marker_size: int = 64,
    opacities: list = [],
    preserve_domain: dict = {},
    scales: dict = {},
    scales_metadata: dict = {
        "x": {"orientation": "horizontal", "dimension": "x"},
        "y": {"orientation": "vertical", "dimension": "y"},
        "color": {"dimension": "color"},
    },
    selected: ndarray = None,
    selected_style: dict = {},
    stroke_width: float = 2.0,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unselected_style: dict = {},
    visible: bool = True,
    x: ndarray = np.array([]),
    y: ndarray = np.array([]),
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_close_path: typing.Callable[[bool], Any] = None,
    on_color: typing.Callable[[ndarray], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_curves_subset: typing.Callable[[list], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_fill: typing.Callable[[str], Any] = None,
    on_fill_colors: typing.Callable[[list], Any] = None,
    on_fill_opacities: typing.Callable[[list], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_interpolation: typing.Callable[[str], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_labels_visibility: typing.Callable[[str], Any] = None,
    on_line_style: typing.Callable[[str], Any] = None,
    on_marker: typing.Callable[[str], Any] = None,
    on_marker_size: typing.Callable[[int], Any] = None,
    on_opacities: typing.Callable[[list], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_stroke_width: typing.Callable[[float], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_x: typing.Callable[[ndarray], Any] = None,
    on_y: typing.Callable[[ndarray], Any] = None,
) -> Element[bqplot.marks.Lines]:
    """Lines mark.

    In the case of the Lines mark, scales for 'x' and 'y' MUST be provided.

    Attributes
    ----------
    icon: string (class-level attribute)
        Font-awesome icon for the respective mark
    name: string (class-level attribute)
        User-friendly name of the mark
    colors: list of colors (default: CATEGORY10)
        List of colors of the Lines. If the list is shorter than the number
        of lines, the colors are reused.
    close_path: bool (default: False)
        Whether to close the paths or not.
    fill: {'none', 'bottom', 'top', 'inside', 'between'}
        Fill in the area defined by the curves
    fill_colors: list of colors (default: [])
        Fill colors for the areas. Defaults to stroke-colors when no
        color provided
    opacities: list of floats (default: [])
        Opacity for the  lines and patches. Defaults to 1 when the list is too
        short, or the element of the list is set to None.
    fill_opacities: list of floats (default: [])
        Opacity for the areas. Defaults to 1 when the list is too
        short, or the element of the list is set to None.
    stroke_width: float (default: 2)
        Stroke width of the Lines
    labels_visibility: {'none', 'label'}
        Visibility of the curve labels
    curves_subset: list of integers or None (default: [])
        If set to None, all the lines are displayed. Otherwise, only the items
        in the list will have full opacity, while others will be faded.
    line_style: {'solid', 'dashed', 'dotted', 'dash_dotted'}
        Line style.
    interpolation: {'linear', 'basis', 'cardinal', 'monotone'}
        Interpolation scheme used for interpolation between the data points
        provided. Please refer to the svg interpolate documentation for details
        about the different interpolation schemes.
    marker: {'circle', 'cross', 'diamond', 'square', 'triangle-down', 'triangle-up', 'arrow', 'rectangle', 'ellipse'}
        Marker shape
    marker_size: nonnegative int (default: 64)
        Default marker size in pixels

    Data Attributes

    x: numpy.ndarray (default: [])
        abscissas of the data points (1d or 2d array)
    y: numpy.ndarray (default: [])
        ordinates of the data points (1d or 2d array)
    color: numpy.ndarray (default: None)
        colors of the different lines based on data. If it is [], then the
        colors from the colors attribute are used. Each line has a single color
        and if the size of colors is less than the number of lines, the
        remaining lines are given the default colors.

    Notes
    -----
    The fields which can be passed to the default tooltip are:
        name: label of the line
        index: index of the line being hovered on
        color: data attribute for the color of the line
    The following are the events which can trigger interactions:
        click: left click of the mouse
        hover: mouse-over an element
    The following are the interactions which can be linked to the above events:
        tooltip: display tooltip


    """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        close_path=close_path,
        color=color,
        colors=colors,
        curves_subset=curves_subset,
        display_legend=display_legend,
        enable_hover=enable_hover,
        fill=fill,
        fill_colors=fill_colors,
        fill_opacities=fill_opacities,
        interactions=interactions,
        interpolation=interpolation,
        labels=labels,
        labels_visibility=labels_visibility,
        line_style=line_style,
        marker=marker,
        marker_size=marker_size,
        opacities=opacities,
        preserve_domain=preserve_domain,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        stroke_width=stroke_width,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unselected_style=unselected_style,
        visible=visible,
        x=x,
        y=y,
        on_apply_clip=on_apply_clip,
        on_close_path=on_close_path,
        on_color=on_color,
        on_colors=on_colors,
        on_curves_subset=on_curves_subset,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_fill=on_fill,
        on_fill_colors=on_fill_colors,
        on_fill_opacities=on_fill_opacities,
        on_interactions=on_interactions,
        on_interpolation=on_interpolation,
        on_labels=on_labels,
        on_labels_visibility=on_labels_visibility,
        on_line_style=on_line_style,
        on_marker=on_marker,
        on_marker_size=on_marker_size,
        on_opacities=on_opacities,
        on_preserve_domain=on_preserve_domain,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_stroke_width=on_stroke_width,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
        on_x=on_x,
        on_y=on_y,
    )
    widget_cls = bqplot.marks.Lines
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def LogScale(
    allow_padding: bool = True,
    max: float = None,
    min: float = None,
    reverse: bool = False,
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_max: typing.Callable[[float], Any] = None,
    on_min: typing.Callable[[float], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.scales.LogScale]:
    """A log scale.

    A logarithmic mapping from a numerical domain to a numerical range.

    Attributes
    ----------
    min: float or None (default: None)
        if not None, min is the minimal value of the domain
    max: float or None (default: None)
        if not None, max is the maximal value of the domain
    rtype: string (class-level attribute)
        This attribute should not be modified by the user.
        The range type of a linear scale is numerical.
    dtype: type (class-level attribute)
        the associated data type / domain type


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        max=max,
        min=min,
        reverse=reverse,
        on_allow_padding=on_allow_padding,
        on_max=on_max,
        on_min=on_min,
        on_reverse=on_reverse,
    )
    widget_cls = bqplot.scales.LogScale
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Map(
    apply_clip: bool = True,
    color: dict = {},
    colors: dict = {},
    display_legend: bool = False,
    enable_hover: bool = True,
    hover_highlight: bool = True,
    hovered_styles: dict = {"hovered_fill": "Orange", "hovered_stroke": None, "hovered_stroke_width": 2.0},
    interactions: dict = {"hover": "tooltip"},
    labels: list = [],
    map_data: dict = {
        "type": "Topology",
        "transform": {"translate": [-180, -90], "scale": [0.03600360036003601, 0.017366249624962495]},
        "objects": {
            "land": {
                "type": "MultiPolygon",
                "arcs": [
                    [[0]],
                    [[1]],
                    [[2]],
                    [[3]],
                    [[4]],
                    [[5]],
                    [[6]],
                    [[7, 8]],
                    [[9, 10]],
                    [[11]],
                    [[12]],
                    [[13]],
                    [[14]],
                    [[15]],
                    [[16]],
                    [[17]],
                    [[18]],
                    [[19]],
                    [[20]],
                    [[21]],
                    [[22]],
                    [[23]],
                    [[24]],
                    [[25]],
                    [[26]],
                    [[27]],
                    [[28, 29]],
                    [[30]],
                    [[31]],
                    [[32]],
                    [[33]],
                    [[34]],
                    [[35]],
                    [[36]],
                    [[37]],
                    [[38]],
                    [[39]],
                    [[40]],
                    [[41, 42]],
                    [[43]],
                    [[44]],
                    [[45]],
                    [[46, 47, 48, 49]],
                    [[50]],
                    [[51]],
                    [[52]],
                    [[53]],
                    [[54]],
                    [[55]],
                    [[56]],
                    [[57]],
                    [[58]],
                    [[59]],
                    [[60]],
                    [[61, 62]],
                    [[63]],
                    [[64]],
                    [[65]],
                    [[66]],
                    [[67]],
                    [[68]],
                    [[69]],
                    [[70]],
                    [[71]],
                    [[72]],
                    [[73]],
                    [[74]],
                    [[75, 76]],
                    [[77]],
                    [[78]],
                    [[79]],
                    [[80]],
                    [[81]],
                    [[82]],
                    [[83]],
                    [[84]],
                    [[85]],
                    [[86]],
                    [[87]],
                    [[88]],
                    [[89, 90]],
                    [[91]],
                    [[92]],
                    [[93]],
                    [[94]],
                    [[95]],
                    [[96]],
                    [[97]],
                    [[98]],
                    [[99]],
                    [[100]],
                    [[101]],
                    [[102]],
                    [[103]],
                    [[104]],
                    [
                        [
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            137,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                        ]
                    ],
                    [[153, 154]],
                    [[155]],
                    [[156]],
                    [[157]],
                    [[158]],
                    [[159]],
                    [[160]],
                    [[161, 162, 163, 164]],
                    [[165]],
                    [[166]],
                    [[167]],
                    [[168]],
                    [[169]],
                    [[170]],
                    [[171]],
                    [[172]],
                    [
                        [
                            173,
                            174,
                            175,
                            176,
                            177,
                            178,
                            179,
                            180,
                            181,
                            182,
                            183,
                            184,
                            185,
                            186,
                            187,
                            188,
                            189,
                            190,
                            191,
                            192,
                            193,
                            194,
                            195,
                            196,
                            197,
                            198,
                            199,
                            200,
                            201,
                            202,
                            203,
                            204,
                            205,
                            206,
                            207,
                            208,
                            209,
                            210,
                            211,
                            212,
                            213,
                            214,
                            215,
                            216,
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            241,
                            242,
                            243,
                            244,
                            245,
                            246,
                            247,
                            248,
                            249,
                            250,
                            251,
                            252,
                            253,
                            254,
                            255,
                            256,
                            257,
                            258,
                            259,
                            260,
                            261,
                            262,
                            263,
                            264,
                            265,
                            266,
                            267,
                            268,
                            269,
                            270,
                            271,
                            272,
                            273,
                            274,
                            275,
                            276,
                            277,
                        ],
                        [278, 279, 280, 281, 282],
                    ],
                    [[283]],
                    [[284]],
                    [[285]],
                    [[286]],
                    [[287]],
                    [[288]],
                    [[289]],
                    [[290]],
                    [[291]],
                    [[292]],
                    [[293]],
                    [[294]],
                    [[295]],
                    [[296]],
                ],
            },
            "subunits": {
                "type": "GeometryCollection",
                "geometries": [
                    {"properties": {"name": "Afghanistan"}, "type": "Polygon", "id": 4, "arcs": [[297, 298, 299, 300, 301, 302]]},
                    {"properties": {"name": "Angola"}, "type": "MultiPolygon", "id": 24, "arcs": [[[303, 304, 211, 305]], [[213, 306, 307]]]},
                    {"properties": {"name": "Albania"}, "type": "Polygon", "id": 8, "arcs": [[308, 248, 309, 310, 311]]},
                    {"properties": {"name": "United Arab Emirates"}, "type": "Polygon", "id": 784, "arcs": [[312, 195, 313, 314, 193]]},
                    {"properties": {"name": "Argentina"}, "type": "MultiPolygon", "id": 32, "arcs": [[[315, 10]], [[316, 317, 318, 131, 319, 320]]]},
                    {"properties": {"name": "Armenia"}, "type": "Polygon", "id": 51, "arcs": [[321, 322, 323, 324, 325]]},
                    {
                        "properties": {"name": "Antarctica"},
                        "type": "MultiPolygon",
                        "id": 10,
                        "arcs": [[[0]], [[1]], [[2]], [[3]], [[4]], [[5]], [[6]], [[326]]],
                    },
                    {"properties": {"name": "French Southern Territories"}, "type": "Polygon", "id": 260, "arcs": [[327]]},
                    {"properties": {"name": "Australia"}, "type": "MultiPolygon", "id": 36, "arcs": [[[13]], [[23]]]},
                    {"properties": {"name": "Austria"}, "type": "Polygon", "id": 40, "arcs": [[328, 329, 330, 331, 332, 333, 334]]},
                    {"properties": {"name": "Azerbaijan"}, "type": "MultiPolygon", "id": 31, "arcs": [[[335, -323]], [[282, 336, -326, 337, 338]]]},
                    {"properties": {"name": "Burundi"}, "type": "Polygon", "id": 108, "arcs": [[339, 340, 341]]},
                    {"properties": {"name": "Belgium"}, "type": "Polygon", "id": 56, "arcs": [[342, 343, 344, 258, 345]]},
                    {"properties": {"name": "Benin"}, "type": "Polygon", "id": 204, "arcs": [[346, 347, 348, 219, 349]]},
                    {"properties": {"name": "Burkina Faso"}, "type": "Polygon", "id": 854, "arcs": [[350, 351, -347, 352, 353, 354]]},
                    {"properties": {"name": "Bangladesh"}, "type": "Polygon", "id": 50, "arcs": [[184, 355, 356]]},
                    {"properties": {"name": "Bulgaria"}, "type": "Polygon", "id": 100, "arcs": [[245, 357, 358, 359, 360, 361]]},
                    {"properties": {"name": "Bahamas"}, "type": "MultiPolygon", "id": 44, "arcs": [[[70]], [[72]], [[73]]]},
                    {"properties": {"name": "Bosnia and Herzegovina"}, "type": "Polygon", "id": 70, "arcs": [[362, 363, 364]]},
                    {"properties": {"name": "Belarus"}, "type": "Polygon", "id": 112, "arcs": [[365, 366, 367, 368, 369]]},
                    {"properties": {"name": "Belize"}, "type": "Polygon", "id": 84, "arcs": [[118, 370, 371]]},
                    {"properties": {"name": "Plurinational State of Bolivia"}, "type": "Polygon", "id": 68, "arcs": [[372, 373, 374, 375, -321]]},
                    {
                        "properties": {"name": "Brazil"},
                        "type": "Polygon",
                        "id": 76,
                        "arcs": [[376, -375, 377, 378, 379, 380, 381, 382, 129, 383, -318]],
                    },
                    {"properties": {"name": "Brunei Darussalam"}, "type": "Polygon", "id": 96, "arcs": [[384, 47]]},
                    {"properties": {"name": "Bhutan"}, "type": "Polygon", "id": 64, "arcs": [[385, 386]]},
                    {"properties": {"name": "Botswana"}, "type": "Polygon", "id": 72, "arcs": [[387, 388, 389, 390]]},
                    {"properties": {"name": "Central African Republic"}, "type": "Polygon", "id": 140, "arcs": [[391, 392, 393, 394, 395, 396, 397]]},
                    {
                        "properties": {"name": "Canada"},
                        "type": "MultiPolygon",
                        "id": 124,
                        "arcs": [
                            [[83]],
                            [[84]],
                            [[85]],
                            [[86]],
                            [[398]],
                            [[95]],
                            [[96]],
                            [[98]],
                            [[100]],
                            [[102]],
                            [[399, 146, 400, 148, 401, 150, 402, 152]],
                            [[153, 403]],
                            [[155]],
                            [[156]],
                            [[157]],
                            [[158]],
                            [[160]],
                            [[161, 404, 163, 405]],
                            [[166]],
                            [[168]],
                            [[169]],
                            [[171]],
                            [[172]],
                            [[283]],
                            [[284]],
                            [[286]],
                            [[287]],
                            [[288]],
                            [[294]],
                            [[295]],
                        ],
                    },
                    {"properties": {"name": "Switzerland"}, "type": "Polygon", "id": 756, "arcs": [[406, 407, 408, -331]]},
                    {"properties": {"name": "Chile"}, "type": "MultiPolygon", "id": 152, "arcs": [[[409, 410, 411, -316]], [[-320, 132, 412, -373]]]},
                    {
                        "properties": {"name": "China"},
                        "type": "MultiPolygon",
                        "id": 156,
                        "arcs": [[[63]], [[413, 177, 414, 415, 416, 417, -386, 418, 419, 420, 421, -300, 422, 423, 424, 425, 426, 427]]],
                    },
                    {"properties": {"name": "Cote d'Ivoire"}, "type": "Polygon", "id": 384, "arcs": [[428, 429, 430, -355, 431, 222]]},
                    {"properties": {"name": "Cameroon"}, "type": "Polygon", "id": 120, "arcs": [[432, 217, 433, 434, 435, -397, 436, 437]]},
                    {
                        "properties": {"name": "The Democratic Republic of the Congo"},
                        "type": "Polygon",
                        "id": 180,
                        "arcs": [[438, -342, 439, 440, -306, 212, -308, 441, -395, 442, 443]],
                    },
                    {"properties": {"name": "Congo"}, "type": "Polygon", "id": 178, "arcs": [[214, 444, -437, -396, -442, -307]]},
                    {"properties": {"name": "Colombia"}, "type": "Polygon", "id": 170, "arcs": [[135, 445, 124, 446, -379, 447, 448]]},
                    {"properties": {"name": "Costa Rica"}, "type": "Polygon", "id": 188, "arcs": [[449, 122, 450, 137]]},
                    {"properties": {"name": "Cuba"}, "type": "Polygon", "id": 192, "arcs": [[451]]},
                    {"properties": {"name": "Northern Cyprus"}, "type": "Polygon", "id": -1, "arcs": [[452, 76]]},
                    {"properties": {"name": "Cyprus"}, "type": "Polygon", "id": 196, "arcs": [[-453, 75]]},
                    {"properties": {"name": "Czech Republic"}, "type": "Polygon", "id": 203, "arcs": [[453, 454, 455, -333]]},
                    {
                        "properties": {"name": "Germany"},
                        "type": "Polygon",
                        "id": 276,
                        "arcs": [[456, -454, -332, -409, 457, 458, -343, 459, 260, 460, 262]],
                    },
                    {"properties": {"name": "Djibouti"}, "type": "Polygon", "id": 262, "arcs": [[461, 462, 463, 203]]},
                    {"properties": {"name": "Denmark"}, "type": "MultiPolygon", "id": 208, "arcs": [[[91]], [[-461, 261]]]},
                    {"properties": {"name": "Dominican Republic"}, "type": "Polygon", "id": 214, "arcs": [[464, 61]]},
                    {"properties": {"name": "Algeria"}, "type": "Polygon", "id": 12, "arcs": [[465, 466, 467, 468, 233, 469, 470, 471]]},
                    {"properties": {"name": "Ecuador"}, "type": "Polygon", "id": 218, "arcs": [[-449, 472, 134]]},
                    {"properties": {"name": "Egypt"}, "type": "Polygon", "id": 818, "arcs": [[473, 474, 236, 475, 200]]},
                    {"properties": {"name": "Eritrea"}, "type": "Polygon", "id": 232, "arcs": [[476, 202, -464, 477]]},
                    {"properties": {"name": "Spain"}, "type": "Polygon", "id": 724, "arcs": [[478, 254, 479, 256]]},
                    {"properties": {"name": "Estonia"}, "type": "Polygon", "id": 233, "arcs": [[480, 481, 267]]},
                    {"properties": {"name": "Ethiopia"}, "type": "Polygon", "id": 231, "arcs": [[-463, 482, 483, 484, 485, 486, 487, -478]]},
                    {"properties": {"name": "Finland"}, "type": "Polygon", "id": 246, "arcs": [[269, 488, 489, 490]]},
                    {"properties": {"name": "Fiji"}, "type": "MultiPolygon", "id": 242, "arcs": [[[17]], [[18]], [[19]]]},
                    {"properties": {"name": "Falkland Islands (Malvinas)"}, "type": "Polygon", "id": 238, "arcs": [[491]]},
                    {
                        "properties": {"name": "France"},
                        "type": "MultiPolygon",
                        "id": 250,
                        "arcs": [[[492, 493, 494, 128, -383]], [[81]], [[495, -458, -408, 496, 253, -479, 257, -345]]],
                    },
                    {"properties": {"name": "Gabon"}, "type": "Polygon", "id": 266, "arcs": [[497, -438, -445, 215]]},
                    {"properties": {"name": "Britain (UK)"}, "type": "MultiPolygon", "id": 826, "arcs": [[[498, 89]], [[499, 500, 501, 502]]]},
                    {"properties": {"name": "Georgia"}, "type": "Polygon", "id": 268, "arcs": [[503, -338, -325, 504, 241]]},
                    {"properties": {"name": "Ghana"}, "type": "Polygon", "id": 288, "arcs": [[-432, -354, 505, 221]]},
                    {"properties": {"name": "Guinea"}, "type": "Polygon", "id": 324, "arcs": [[506, 225, 507, 508, 509, -430, 510]]},
                    {"properties": {"name": "Gambia"}, "type": "Polygon", "id": 270, "arcs": [[228, 511]]},
                    {"properties": {"name": "Guinea-Bissau"}, "type": "Polygon", "id": 624, "arcs": [[512, -508, 226]]},
                    {"properties": {"name": "Equatorial Guinea"}, "type": "Polygon", "id": 226, "arcs": [[-433, -498, 216]]},
                    {"properties": {"name": "Greece"}, "type": "MultiPolygon", "id": 300, "arcs": [[[77]], [[247, -309, 513, -359, 514]]]},
                    {"properties": {"name": "Greenland"}, "type": "Polygon", "id": 304, "arcs": [[515]]},
                    {"properties": {"name": "Guatemala"}, "type": "Polygon", "id": 320, "arcs": [[516, -371, 119, 517, 518, 143]]},
                    {"properties": {"name": "Guyana"}, "type": "Polygon", "id": 328, "arcs": [[519, -381, 520, 126]]},
                    {"properties": {"name": "Honduras"}, "type": "Polygon", "id": 340, "arcs": [[521, -518, 120, 522, 139]]},
                    {"properties": {"name": "Croatia"}, "type": "Polygon", "id": 191, "arcs": [[-364, 523, 250, 524, 525, 526]]},
                    {"properties": {"name": "Haiti"}, "type": "Polygon", "id": 332, "arcs": [[-465, 62]]},
                    {"properties": {"name": "Hungary"}, "type": "Polygon", "id": 348, "arcs": [[527, 528, 529, 530, -526, 531, -335]]},
                    {
                        "properties": {"name": "Indonesia"},
                        "type": "MultiPolygon",
                        "id": 360,
                        "arcs": [[[25]], [[532, 29]], [[30]], [[31]], [[34]], [[35]], [[38]], [[39]], [[533, 42]], [[43]], [[44]], [[534, 49]], [[45]]],
                    },
                    {"properties": {"name": "India"}, "type": "Polygon", "id": 356, "arcs": [[535, -419, -387, -418, 536, -356, 185, 537, -421]]},
                    {"properties": {"name": "Ireland"}, "type": "Polygon", "id": 372, "arcs": [[-499, 90]]},
                    {
                        "properties": {"name": "Islamic Republic of Iran"},
                        "type": "Polygon",
                        "id": 364,
                        "arcs": [[-302, 538, 187, 539, 540, -336, -322, -337, 278, 541]],
                    },
                    {"properties": {"name": "Iraq"}, "type": "Polygon", "id": 368, "arcs": [[188, 542, 543, 544, 545, 546, -540]]},
                    {"properties": {"name": "Iceland"}, "type": "Polygon", "id": 352, "arcs": [[99]]},
                    {"properties": {"name": "Israel"}, "type": "Polygon", "id": 376, "arcs": [[547, -476, 237, 548, 549, 550, 551]]},
                    {"properties": {"name": "Italy"}, "type": "MultiPolygon", "id": 380, "arcs": [[[78]], [[79]], [[552, 252, -497, -407, -330]]]},
                    {"properties": {"name": "Jamaica"}, "type": "Polygon", "id": 388, "arcs": [[553]]},
                    {"properties": {"name": "Jordan"}, "type": "Polygon", "id": 400, "arcs": [[-545, 554, 199, -548, 555, -551, 556]]},
                    {"properties": {"name": "Japan"}, "type": "MultiPolygon", "id": 392, "arcs": [[[74]], [[80]], [[82]]]},
                    {"properties": {"name": "Kazakhstan"}, "type": "Polygon", "id": 398, "arcs": [[557, 280, 558, -425, 559, 560]]},
                    {"properties": {"name": "Kenya"}, "type": "Polygon", "id": 404, "arcs": [[206, 561, 562, 563, -485, 564]]},
                    {"properties": {"name": "Kyrgyzstan"}, "type": "Polygon", "id": 417, "arcs": [[-424, 565, 566, -560]]},
                    {"properties": {"name": "Cambodia"}, "type": "Polygon", "id": 116, "arcs": [[567, 568, 569, 179]]},
                    {"properties": {"name": "Republic of Korea"}, "type": "Polygon", "id": 410, "arcs": [[570, 175]]},
                    {"properties": {"name": "Kosovo"}, "type": "Polygon", "id": -2, "arcs": [[-311, 571, 572, 573]]},
                    {"properties": {"name": "Kuwait"}, "type": "Polygon", "id": 414, "arcs": [[574, -543, 189]]},
                    {"properties": {"name": "Lao People's Democratic Republic"}, "type": "Polygon", "id": 418, "arcs": [[575, -416, 576, -569, 577]]},
                    {"properties": {"name": "Lebanon"}, "type": "Polygon", "id": 422, "arcs": [[238, 578, -549]]},
                    {"properties": {"name": "Liberia"}, "type": "Polygon", "id": 430, "arcs": [[579, -511, -429, 223]]},
                    {"properties": {"name": "Libya"}, "type": "Polygon", "id": 434, "arcs": [[-471, 580, 235, -475, 581, 582, 583]]},
                    {"properties": {"name": "Sri Lanka"}, "type": "Polygon", "id": 144, "arcs": [[584]]},
                    {"properties": {"name": "Lesotho"}, "type": "Polygon", "id": 426, "arcs": [[585]]},
                    {"properties": {"name": "Lithuania"}, "type": "Polygon", "id": 440, "arcs": [[265, 586, -370, 587, 588]]},
                    {"properties": {"name": "Luxembourg"}, "type": "Polygon", "id": 442, "arcs": [[-496, -344, -459]]},
                    {"properties": {"name": "Latvia"}, "type": "Polygon", "id": 428, "arcs": [[-482, 589, -366, -587, 266]]},
                    {"properties": {"name": "Morocco"}, "type": "Polygon", "id": 504, "arcs": [[-469, 590, 591, 592, 593]]},
                    {"properties": {"name": "Republic of Moldova"}, "type": "Polygon", "id": 498, "arcs": [[594, 595]]},
                    {"properties": {"name": "Madagascar"}, "type": "Polygon", "id": 450, "arcs": [[596]]},
                    {"properties": {"name": "Mexico"}, "type": "Polygon", "id": 484, "arcs": [[-372, -517, 144, 597, 598]]},
                    {
                        "properties": {"name": "The former Yugoslav Republic of Macedonia"},
                        "type": "Polygon",
                        "id": 807,
                        "arcs": [[599, -360, -514, -312, -574]],
                    },
                    {"properties": {"name": "Mali"}, "type": "Polygon", "id": 466, "arcs": [[-466, 600, -351, -431, -510, 601, 602]]},
                    {"properties": {"name": "Myanmar"}, "type": "Polygon", "id": 104, "arcs": [[183, -357, -537, -417, -576, 603]]},
                    {"properties": {"name": "Montenegro"}, "type": "Polygon", "id": 499, "arcs": [[249, -524, -363, 604, -572, -310]]},
                    {"properties": {"name": "Mongolia"}, "type": "Polygon", "id": 496, "arcs": [[-427, 605]]},
                    {"properties": {"name": "Mozambique"}, "type": "Polygon", "id": 508, "arcs": [[208, 606, 607, 608, 609, 610, 611, 612]]},
                    {"properties": {"name": "Mauritania"}, "type": "Polygon", "id": 478, "arcs": [[230, 613, -467, -603, 614]]},
                    {"properties": {"name": "Malawi"}, "type": "Polygon", "id": 454, "arcs": [[615, 616, -612]]},
                    {"properties": {"name": "Malaysia"}, "type": "MultiPolygon", "id": 458, "arcs": [[[181, 617]], [[-535, 46, -385, 48]]]},
                    {"properties": {"name": "Namibia"}, "type": "Polygon", "id": 516, "arcs": [[-305, 618, -389, 619, 210]]},
                    {"properties": {"name": "New Caledonia"}, "type": "Polygon", "id": 540, "arcs": [[620]]},
                    {"properties": {"name": "Niger"}, "type": "Polygon", "id": 562, "arcs": [[-601, -472, -584, 621, -435, 622, -348, -352]]},
                    {"properties": {"name": "Nigeria"}, "type": "Polygon", "id": 566, "arcs": [[-349, -623, -434, 218]]},
                    {"properties": {"name": "Nicaragua"}, "type": "Polygon", "id": 558, "arcs": [[-523, 121, -450, 138]]},
                    {"properties": {"name": "Netherlands"}, "type": "Polygon", "id": 528, "arcs": [[-460, -346, 259]]},
                    {"properties": {"name": "Norway"}, "type": "MultiPolygon", "id": 578, "arcs": [[[623, -490, 624, 271]], [[285]], [[290]], [[291]]]},
                    {"properties": {"name": "Nepal"}, "type": "Polygon", "id": 524, "arcs": [[-420, -536]]},
                    {"properties": {"name": "New Zealand"}, "type": "MultiPolygon", "id": 554, "arcs": [[[14]], [[15]]]},
                    {"properties": {"name": "Oman"}, "type": "MultiPolygon", "id": 512, "arcs": [[[625, 626, -314, 196]], [[-313, 194]]]},
                    {"properties": {"name": "Pakistan"}, "type": "Polygon", "id": 586, "arcs": [[-538, 186, -539, -301, -422]]},
                    {"properties": {"name": "Panama"}, "type": "Polygon", "id": 591, "arcs": [[-451, 123, -446, 136]]},
                    {"properties": {"name": "Peru"}, "type": "Polygon", "id": 604, "arcs": [[133, -473, -448, -378, -374, -413]]},
                    {
                        "properties": {"name": "Philippines"},
                        "type": "MultiPolygon",
                        "id": 608,
                        "arcs": [[[50]], [[53]], [[54]], [[55]], [[56]], [[57]], [[58]]],
                    },
                    {"properties": {"name": "Papua New Guinea"}, "type": "MultiPolygon", "id": 598, "arcs": [[[36]], [[37]], [[-534, 41]], [[40]]]},
                    {"properties": {"name": "Poland"}, "type": "Polygon", "id": 616, "arcs": [[263, 627, -588, -369, 628, 629, -455, -457]]},
                    {"properties": {"name": "Puerto Rico"}, "type": "Polygon", "id": 630, "arcs": [[630]]},
                    {
                        "properties": {"name": "Democratic People's Republic of  Korea"},
                        "type": "Polygon",
                        "id": 408,
                        "arcs": [[-571, 176, -414, 631, 174]],
                    },
                    {"properties": {"name": "Portugal"}, "type": "Polygon", "id": 620, "arcs": [[255, -480]]},
                    {"properties": {"name": "Paraguay"}, "type": "Polygon", "id": 600, "arcs": [[-377, -317, -376]]},
                    {"properties": {"name": "Occupied Palestinian Territory"}, "type": "Polygon", "id": 275, "arcs": [[-552, -556]]},
                    {"properties": {"name": "Qatar"}, "type": "Polygon", "id": 634, "arcs": [[632, 191]]},
                    {"properties": {"name": "Romania"}, "type": "Polygon", "id": 642, "arcs": [[-595, 633, 244, -362, 634, -530, 635]]},
                    {
                        "properties": {"name": "Russian Federation"},
                        "type": "MultiPolygon",
                        "id": 643,
                        "arcs": [
                            [[88]],
                            [[264, -589, -628]],
                            [[101]],
                            [[103]],
                            [[104]],
                            [[159]],
                            [[165]],
                            [[167]],
                            [[170]],
                            [
                                [
                                    173,
                                    -632,
                                    -428,
                                    -606,
                                    -426,
                                    -559,
                                    281,
                                    -339,
                                    -504,
                                    242,
                                    636,
                                    -367,
                                    -590,
                                    -481,
                                    268,
                                    -491,
                                    -624,
                                    637,
                                    638,
                                    639,
                                    640,
                                    274,
                                    641,
                                    276,
                                    642,
                                ]
                            ],
                            [[289]],
                            [[292]],
                            [[293]],
                        ],
                    },
                    {"properties": {"name": "Rwanda"}, "type": "Polygon", "id": 646, "arcs": [[-340, -439, 643, 644]]},
                    {"properties": {"name": "Western Sahara"}, "type": "Polygon", "id": 732, "arcs": [[-468, -614, 231, -591]]},
                    {
                        "properties": {"name": "Saudi Arabia"},
                        "type": "Polygon",
                        "id": 682,
                        "arcs": [[-555, -544, -575, 190, -633, 192, -315, -627, 645, 198]],
                    },
                    {"properties": {"name": "Sudan"}, "type": "Polygon", "id": 729, "arcs": [[646, -582, -474, 201, -477, -488, 647, 648, 649, -392]]},
                    {"properties": {"name": "South Sudan"}, "type": "Polygon", "id": 728, "arcs": [[-564, 650, -443, -394, 651, -649, 652, -486]]},
                    {"properties": {"name": "Senegal"}, "type": "Polygon", "id": 686, "arcs": [[-615, -602, -509, -513, 227, -512, 229]]},
                    {"properties": {"name": "Solomon Islands"}, "type": "MultiPolygon", "id": 90, "arcs": [[[24]], [[26]], [[27]], [[32]], [[33]]]},
                    {"properties": {"name": "Sierra Leone"}, "type": "Polygon", "id": 694, "arcs": [[-507, -580, 224]]},
                    {"properties": {"name": "El Salvador"}, "type": "Polygon", "id": 222, "arcs": [[142, -519, -522, 140, 653]]},
                    {"properties": {"name": "Somaliland"}, "type": "Polygon", "id": -3, "arcs": [[-483, -462, 204, 654]]},
                    {"properties": {"name": "Somalia"}, "type": "Polygon", "id": 706, "arcs": [[-565, -484, -655, 205]]},
                    {"properties": {"name": "Serbia"}, "type": "Polygon", "id": 688, "arcs": [[-361, -600, -573, -605, -365, -527, -531, -635]]},
                    {"properties": {"name": "Suriname"}, "type": "Polygon", "id": 740, "arcs": [[-495, 655, 656, -382, -520, 127]]},
                    {"properties": {"name": "Slovakia"}, "type": "Polygon", "id": 703, "arcs": [[657, -528, -334, -456, -630]]},
                    {"properties": {"name": "Slovenia"}, "type": "Polygon", "id": 705, "arcs": [[-532, -525, 251, -553, -329]]},
                    {"properties": {"name": "Sweden"}, "type": "Polygon", "id": 752, "arcs": [[-625, -489, 270]]},
                    {"properties": {"name": "Swaziland"}, "type": "Polygon", "id": 748, "arcs": [[-608, 658]]},
                    {"properties": {"name": "Syrian Arab Republic"}, "type": "Polygon", "id": 760, "arcs": [[-550, -579, 239, 659, -546, -557]]},
                    {"properties": {"name": "Chad"}, "type": "Polygon", "id": 148, "arcs": [[-583, -647, -398, -436, -622]]},
                    {"properties": {"name": "Togo"}, "type": "Polygon", "id": 768, "arcs": [[-353, -350, 220, -506]]},
                    {"properties": {"name": "Thailand"}, "type": "Polygon", "id": 764, "arcs": [[-618, 182, -604, -578, -568, 180]]},
                    {"properties": {"name": "Tajikistan"}, "type": "Polygon", "id": 762, "arcs": [[-423, -299, 660, -566]]},
                    {"properties": {"name": "Turkmenistan"}, "type": "Polygon", "id": 795, "arcs": [[279, -558, 661, -303, -542]]},
                    {"properties": {"name": "Timor-Leste"}, "type": "Polygon", "id": 626, "arcs": [[-533, 28]]},
                    {"properties": {"name": "Trinidad and Tobago"}, "type": "Polygon", "id": 780, "arcs": [[662]]},
                    {"properties": {"name": "Tunisia"}, "type": "Polygon", "id": 788, "arcs": [[234, -581, -470]]},
                    {
                        "properties": {"name": "Turkey"},
                        "type": "MultiPolygon",
                        "id": 792,
                        "arcs": [[[-505, -324, -541, -547, -660, 240]], [[-515, -358, 246]]],
                    },
                    {"properties": {"name": "Taiwan, Province of China"}, "type": "Polygon", "id": 158, "arcs": [[663]]},
                    {
                        "properties": {"name": "United Republic of Tanzania"},
                        "type": "Polygon",
                        "id": 834,
                        "arcs": [[207, -613, -617, 664, -440, -341, -645, 665, -562]],
                    },
                    {"properties": {"name": "Uganda"}, "type": "Polygon", "id": 800, "arcs": [[-644, -444, -651, -563, -666]]},
                    {"properties": {"name": "Ukraine"}, "type": "Polygon", "id": 804, "arcs": [[243, -634, -596, -636, -529, -658, -629, -368, -637]]},
                    {"properties": {"name": "Uruguay"}, "type": "Polygon", "id": 858, "arcs": [[130, -319, -384]]},
                    {
                        "properties": {"name": "United States"},
                        "type": "MultiPolygon",
                        "id": 840,
                        "arcs": [
                            [[64]],
                            [[65]],
                            [[66]],
                            [[67]],
                            [[68]],
                            [[105, 666, 107, 667, 109, 668, 111, 669, 113, 670, 115, -598, 671, 672, 673, -400]],
                            [[92]],
                            [[94]],
                            [[97]],
                            [[-401, 147]],
                        ],
                    },
                    {"properties": {"name": "Uzbekistan"}, "type": "Polygon", "id": 860, "arcs": [[-561, -567, -661, -298, -662]]},
                    {"properties": {"name": "Bolivarian Republic of Venezuela"}, "type": "Polygon", "id": 862, "arcs": [[-521, -380, -447, 125]]},
                    {"properties": {"name": "Viet Nam"}, "type": "Polygon", "id": 704, "arcs": [[-570, -577, -415, 178]]},
                    {"properties": {"name": "Vanuatu"}, "type": "MultiPolygon", "id": 548, "arcs": [[[20]], [[21]]]},
                    {"properties": {"name": "Yemen"}, "type": "Polygon", "id": 887, "arcs": [[-646, -626, 197]]},
                    {"properties": {"name": "South Africa"}, "type": "Polygon", "id": 710, "arcs": [[-620, -388, 674, -609, -659, -607, 209], [-586]]},
                    {"properties": {"name": "Zambia"}, "type": "Polygon", "id": 894, "arcs": [[-611, 675, -390, -619, -304, -441, -665, -616]]},
                    {"properties": {"name": "Zimbabwe"}, "type": "Polygon", "id": 716, "arcs": [[-391, -676, -610, -675]]},
                ],
            },
        },
        "arcs": [
            [[3344, 573], [-8, -29], [-8, -26], [-58, 8], [-62, -4], [-35, 19], [0, 3], [-15, 17], [62, -3], [60, -5], [21, 23], [15, 21], [28, -24]],
            [[577, 604], [-53, -8], [-37, 21], [-16, 20], [-1, 3], [-18, 16], [17, 22], [51, -9], [28, -18], [21, -21], [8, -26]],
            [
                [3745, 688],
                [34, -25],
                [12, -35],
                [3, -25],
                [1, -29],
                [-43, -18],
                [-45, -15],
                [-52, -13],
                [-58, -12],
                [-66, 4],
                [-36, 19],
                [4, 24],
                [60, 15],
                [24, 20],
                [17, 24],
                [13, 22],
                [16, 20],
                [18, 24],
                [15, 0],
                [41, 12],
                [42, -12],
            ],
            [[1632, 950], [36, -9], [33, 10], [-15, -21], [-26, -14], [-39, 4], [-28, 21], [6, 19], [33, -10]],
            [[1512, 951], [42, -23], [-16, 2], [-36, 6], [-38, 16], [20, 12], [28, -13]],
            [[2250, 1040], [30, -8], [31, 7], [16, -33], [-22, 4], [-33, -2], [-35, 2], [-37, -3], [-29, 11], [-14, 24], [17, 10], [36, -8], [40, -4]],
            [
                [3098, 1096],
                [3, -26],
                [-5, -22],
                [-7, -22],
                [-33, -8],
                [-31, -11],
                [-37, 1],
                [14, 23],
                [-33, -8],
                [-31, -8],
                [-21, 17],
                [-1, 23],
                [30, 23],
                [19, 7],
                [32, -2],
                [8, 29],
                [2, 21],
                [-1, 47],
                [16, 27],
                [26, 9],
                [14, -22],
                [7, -21],
                [12, -26],
                [9, -25],
                [8, -26],
            ],
            [
                [108, 339],
                [4, 0],
                [3, -1],
                [41, -24],
                [35, 24],
                [6, 3],
                [82, 11],
                [26, -14],
                [13, -7],
                [42, -19],
                [79, -15],
                [62, -18],
                [108, -13],
                [80, 16],
                [118, -12],
                [66, -18],
                [74, 17],
                [77, 16],
                [6, 27],
                [-109, 2],
                [-90, 14],
                [-23, 23],
                [-75, 12],
                [5, 26],
                [10, 24],
                [11, 21],
                [-6, 24],
                [-46, 16],
                [-21, 20],
                [-43, 18],
                [67, -4],
                [64, 10],
                [41, -20],
                [49, 17],
                [46, 22],
                [22, 19],
                [-10, 24],
                [-35, 15],
                [-41, 17],
                [-57, 4],
                [-50, 8],
                [-54, 5],
                [-18, 22],
                [-36, 18],
                [-22, 20],
                [-9, 65],
                [14, -5],
                [25, -18],
                [46, 5],
                [44, 8],
                [23, -25],
                [44, 6],
                [37, 13],
                [35, 15],
                [31, 20],
                [42, 5],
                [-1, 22],
                [-10, 21],
                [8, 20],
                [36, 10],
                [16, -19],
                [43, 12],
                [32, 14],
                [40, 1],
                [37, 6],
                [38, 14],
                [30, 12],
                [33, 12],
                [22, -3],
                [19, -5],
                [42, 8],
                [37, -10],
                [38, 1],
                [36, 8],
                [38, -5],
                [41, -6],
                [39, 2],
                [40, -1],
                [41, -1],
                [38, 2],
                [29, 17],
                [33, 9],
                [35, -12],
                [33, 10],
                [30, 20],
                [18, -18],
                [10, -20],
                [18, -19],
                [29, 17],
                [33, -22],
                [37, -7],
                [33, -15],
                [39, 3],
                [35, 10],
                [42, -2],
                [37, -8],
                [39, -10],
                [14, 25],
                [-18, 19],
                [-13, 20],
                [-36, 5],
                [-16, 21],
                [-6, 22],
                [-10, 42],
                [21, -7],
                [37, -4],
                [36, 4],
                [32, -9],
                [29, -17],
                [12, -21],
                [37, -3],
                [36, 8],
                [38, 11],
                [35, 7],
                [28, -14],
                [37, 5],
                [24, 44],
                [22, -26],
                [32, -10],
                [35, 5],
                [23, -22],
                [36, -2],
                [34, -7],
                [33, -13],
                [22, 22],
                [11, 20],
                [28, -22],
                [38, 5],
                [28, -12],
                [19, -19],
                [37, 5],
                [29, 13],
                [28, 14],
                [34, 8],
                [39, 7],
                [35, 8],
                [28, 12],
                [16, 18],
                [6, 25],
                [-3, 24],
                [-9, 22],
                [-9, 23],
                [-9, 23],
                [-7, 20],
                [-2, 22],
                [3, 23],
                [13, 21],
                [11, 24],
                [4, 23],
                [-5, 25],
                [-4, 22],
                [14, 26],
                [15, 17],
                [18, 21],
                [19, 18],
                [23, 17],
                [10, 25],
                [16, 16],
                [17, 15],
                [27, 3],
                [17, 18],
                [20, 11],
                [23, 7],
                [20, 15],
                [16, 18],
                [21, 7],
                [17, -15],
                [-11, -19],
                [-28, -17],
                [-12, -13],
                [-21, 9],
                [-22, -5],
                [-20, -14],
                [-20, -14],
                [-13, -17],
                [-4, -23],
                [2, -21],
                [13, -19],
                [-19, -14],
                [-27, -5],
                [-15, -19],
                [-16, -18],
                [-17, -25],
                [-5, -21],
                [10, -24],
                [15, -18],
                [22, -13],
                [22, -18],
                [11, -23],
                [6, -21],
                [8, -23],
                [13, -19],
                [8, -22],
                [4, -53],
                [8, -21],
                [3, -23],
                [8, -22],
                [-3, -31],
                [-16, -23],
                [-16, -20],
                [-37, -7],
                [-13, -21],
                [-16, -19],
                [-42, -21],
                [-37, -9],
                [-35, -13],
                [-38, -12],
                [-22, -24],
                [-45, -2],
                [-49, 2],
                [-44, -4],
                [-46, 0],
                [8, -23],
                [43, -10],
                [31, -16],
                [17, -20],
                [-31, -18],
                [-48, 5],
                [-39, -14],
                [-2, -24],
                [-1, -23],
                [33, -19],
                [6, -21],
                [35, -22],
                [59, -9],
                [50, -15],
                [39, -18],
                [51, -19],
                [69, -9],
                [68, -15],
                [47, -17],
                [52, -19],
                [27, -28],
                [14, -21],
                [34, 20],
                [45, 17],
                [49, 18],
                [57, 15],
                [50, 16],
                [69, 1],
                [68, -8],
                [56, -14],
                [18, 25],
                [39, 17],
                [70, 1],
                [55, 13],
                [52, 12],
                [58, 8],
                [61, 10],
                [43, 15],
                [-20, 20],
                [-12, 20],
                [0, 22],
                [-53, -2],
                [-57, -9],
                [-55, 0],
                [-8, 21],
                [4, 43],
                [13, 12],
                [40, 14],
                [46, 13],
                [34, 17],
                [34, 17],
                [25, 23],
                [38, 10],
                [37, 8],
                [19, 4],
                [43, 3],
                [41, 8],
                [34, 11],
                [34, 13],
                [31, 14],
                [38, 18],
                [25, 19],
                [26, 17],
                [8, 23],
                [-29, 13],
                [9, 24],
                [19, 18],
                [29, 11],
                [30, 14],
                [29, 18],
                [21, 22],
                [14, 27],
                [20, 16],
                [33, -3],
                [14, -19],
                [33, -3],
                [1, 22],
                [14, 22],
                [30, -5],
                [7, -22],
                [33, -3],
                [36, 10],
                [35, 7],
                [32, -3],
                [12, -24],
                [30, 19],
                [28, 10],
                [32, 8],
                [31, 8],
                [28, 14],
                [31, 9],
                [24, 12],
                [17, 20],
                [21, -14],
                [29, 8],
                [20, -28],
                [15, -20],
                [32, 11],
                [13, 23],
                [28, 16],
                [36, -4],
                [11, -21],
                [23, 21],
                [30, 7],
                [33, 2],
                [29, -1],
                [31, -6],
                [30, -4],
                [13, -19],
                [18, -17],
                [30, 10],
                [33, 2],
                [32, 0],
                [31, 2],
                [27, 7],
                [30, 7],
                [24, 16],
                [26, 10],
                [29, 6],
                [21, 16],
                [15, 31],
                [16, 19],
                [29, -9],
                [10, -20],
                [24, -13],
                [29, 4],
                [20, -20],
                [20, -15],
                [29, 14],
                [10, 24],
                [25, 11],
                [28, 19],
                [28, 8],
                [32, 11],
                [22, 12],
                [23, 14],
                [22, 12],
                [26, -6],
                [25, 20],
                [18, 16],
                [26, -2],
                [23, 14],
                [5, 20],
                [24, 16],
                [22, 11],
                [28, 9],
                [26, 5],
                [24, -3],
                [26, -6],
                [23, -16],
                [2, -25],
                [25, -19],
                [17, -16],
                [33, -6],
                [18, -16],
                [23, -16],
                [27, -3],
                [22, 11],
                [24, 24],
                [26, -13],
                [27, -7],
                [26, -6],
                [28, -5],
                [27, 0],
                [23, -60],
                [-1, -14],
                [-3, -26],
                [-27, -15],
                [-22, -21],
                [4, -23],
                [31, 1],
                [-3, -22],
                [-15, -22],
                [-13, -24],
                [22, -18],
                [32, -5],
                [32, 10],
                [15, 22],
                [9, 22],
                [15, 18],
                [18, 17],
                [7, 20],
                [15, 28],
                [17, 6],
                [32, 2],
                [27, 7],
                [29, 9],
                [13, 23],
                [8, 21],
                [19, 22],
                [28, 14],
                [23, 11],
                [15, 20],
                [16, 10],
                [20, 9],
                [28, -6],
                [25, 6],
                [27, 7],
                [31, -4],
                [20, 16],
                [14, 38],
                [10, -15],
                [13, -27],
                [24, -12],
                [26, -4],
                [27, 6],
                [28, -4],
                [26, -1],
                [18, 5],
                [23, -3],
                [21, -12],
                [25, 8],
                [30, 0],
                [26, 7],
                [29, -7],
                [18, 19],
                [14, 19],
                [19, 16],
                [35, 43],
                [18, -8],
                [21, -16],
                [19, -20],
                [35, -35],
                [27, -1],
                [26, 0],
                [30, 6],
                [30, 8],
                [23, 16],
                [19, 17],
                [31, 2],
                [20, 13],
                [22, -12],
                [14, -18],
                [20, -18],
                [30, 2],
                [19, -14],
                [34, -15],
                [34, -6],
                [29, 5],
                [22, 18],
                [18, 18],
                [25, 5],
                [25, -8],
                [29, -6],
                [26, 9],
                [25, 0],
                [25, -6],
                [26, -5],
                [25, 10],
                [29, 9],
                [29, 2],
                [31, 0],
                [26, 6],
                [25, 4],
                [8, 29],
                [1, 23],
                [17, -15],
                [5, -26],
                [9, -24],
                [12, -19],
                [23, -10],
                [32, 3],
                [36, 1],
                [25, 3],
                [36, 0],
                [27, 2],
                [36, -3],
                [31, -4],
                [20, -18],
                [-6, -22],
                [18, -17],
                [30, -13],
                [31, -15],
                [36, -10],
                [37, -9],
                [29, -9],
                [31, -1],
                [18, 19],
                [25, -16],
                [21, -18],
                [24, -13],
                [34, -6],
                [32, -7],
                [14, -22],
                [31, -14],
                [22, -20],
                [31, -9],
                [32, 1],
                [30, -3],
                [33, 1],
                [33, -5],
                [31, -8],
                [29, -13],
                [29, -12],
                [19, -17],
                [-3, -22],
                [-15, -20],
                [-12, -26],
                [-10, -21],
                [-13, -23],
                [-36, -9],
                [-17, -21],
                [-36, -12],
                [-12, -23],
                [-19, -21],
                [-20, -18],
                [-12, -24],
                [-7, -21],
                [-3, -26],
                [1, -22],
                [16, -22],
                [6, -22],
                [13, -20],
                [51, -8],
                [11, -25],
                [-50, -9],
                [-42, -12],
                [-53, -2],
                [-23, -33],
                [-5, -27],
                [-12, -22],
                [-15, -21],
                [37, -19],
                [14, -24],
                [24, -21],
                [34, -20],
                [39, -18],
                [41, -18],
                [64, -18],
                [14, -28],
                [80, -12],
                [6, -5],
                [20, -17],
                [77, 15],
                [64, -18],
                [48, -14],
                [-9998, -1],
                [25, 34],
                [50, -18],
                [3, 2],
            ],
            [[79, 321], [8, 5], [9, 6], [8, 5], [4, 2]],
            [
                [3139, 2021],
                [-9, -23],
                [-24, -18],
                [-30, 6],
                [-20, 17],
                [-29, 9],
                [-35, 32],
                [-28, 31],
                [-39, 64],
                [23, -12],
                [39, -38],
                [37, -21],
                [14, 27],
                [9, 39],
                [26, 24],
                [20, -7],
            ],
            [[3093, 2151], [10, -27], [14, -43], [36, -34], [39, -15], [-12, -29], [-27, -2], [-14, 20]],
            [[3373, 2239], [22, -25], [-8, -21], [-38, -17], [-12, 20], [-24, -26], [-14, 26], [33, 35], [24, -15], [17, 23]],
            [[6951, 2320], [-43, -4], [0, 30], [4, 24], [2, 12], [18, -18], [26, -7], [1, -11], [-8, -26]],
            [
                [9037, 2833],
                [27, -20],
                [15, 8],
                [22, 11],
                [17, -4],
                [2, -68],
                [-10, -20],
                [-3, -46],
                [-9, 15],
                [-20, -40],
                [-5, 3],
                [-18, 2],
                [-17, 50],
                [-3, 38],
                [-16, 50],
                [0, 26],
                [18, -5],
            ],
            [
                [9805, 2826],
                [6, -24],
                [20, 23],
                [8, -24],
                [0, -24],
                [-11, -27],
                [-18, -42],
                [-14, -24],
                [10, -27],
                [-21, -1],
                [-24, -22],
                [-7, -37],
                [-16, -59],
                [-22, -25],
                [-14, -17],
                [-25, 1],
                [-18, 19],
                [-31, 4],
                [-4, 22],
                [15, 42],
                [35, 57],
                [18, 11],
                [20, 22],
                [23, 30],
                [17, 30],
                [12, 43],
                [11, 14],
                [4, 33],
                [19, 26],
                [7, -24],
            ],
            [
                [9849, 3100],
                [20, -61],
                [1, 40],
                [12, -16],
                [4, -44],
                [23, -18],
                [18, -5],
                [16, 22],
                [14, -7],
                [-6, -51],
                [-9, -33],
                [-21, 1],
                [-7, -18],
                [2, -24],
                [-4, -11],
                [-10, -31],
                [-14, -40],
                [-22, -23],
                [-4, 16],
                [-12, 8],
                [16, 47],
                [-9, 32],
                [-30, 23],
                [1, 21],
                [20, 20],
                [4, 44],
                [-1, 37],
                [-11, 39],
                [1, 10],
                [-14, 24],
                [-21, 51],
                [-12, 41],
                [10, 4],
                [15, -32],
                [22, -15],
                [8, -51],
            ],
            [
                [9641, 3906],
                [-11, -14],
                [-15, 16],
                [-20, 26],
                [-18, 30],
                [-18, 41],
                [-4, 19],
                [12, -1],
                [15, -19],
                [13, -20],
                [9, -16],
                [22, -36],
                [15, -26],
            ],
            [[9953, 4183], [10, -16], [-5, -30], [-17, -8], [-15, 7], [-3, 25], [11, 20], [12, -7], [7, 9]],
            [[9981, 4214], [-18, -12], [-3, 22], [13, 12], [9, 3], [17, 18], [0, -28], [-18, -15]],
            [[2, 4232], [-2, -3], [0, 28], [5, 2], [-3, -27]],
            [[9661, 4234], [-9, -8], [-10, 25], [1, 16], [18, -33]],
            [[9640, 4322], [5, -46], [-8, 7], [-5, -3], [-4, 16], [-1, 44], [13, -18]],
            [
                [6389, 4401],
                [5, -69],
                [7, -27],
                [-3, -27],
                [-5, -17],
                [-9, 33],
                [-5, -17],
                [5, -42],
                [-3, -25],
                [-7, -13],
                [-2, -49],
                [-11, -67],
                [-14, -79],
                [-17, -109],
                [-10, -80],
                [-13, -67],
                [-23, -14],
                [-24, -24],
                [-16, 14],
                [-22, 21],
                [-7, 30],
                [-2, 51],
                [-10, 46],
                [-3, 42],
                [5, 41],
                [13, 10],
                [0, 19],
                [13, 44],
                [3, 37],
                [-6, 27],
                [-6, 36],
                [-2, 53],
                [10, 33],
                [4, 36],
                [13, 2],
                [16, 12],
                [10, 11],
                [12, 0],
                [16, 33],
                [23, 36],
                [8, 29],
                [-3, 24],
                [11, -7],
                [16, 40],
                [0, 35],
                [9, 26],
                [10, -25],
                [7, -25],
                [7, -38],
            ],
            [
                [8986, 4389],
                [10, -45],
                [18, 22],
                [9, -24],
                [14, -23],
                [-3, -25],
                [6, -50],
                [4, -29],
                [7, -7],
                [8, -49],
                [-3, -30],
                [9, -39],
                [30, -30],
                [20, -27],
                [18, -25],
                [-3, -14],
                [16, -36],
                [10, -63],
                [11, 13],
                [12, -25],
                [6, 9],
                [5, -61],
                [20, -35],
                [13, -22],
                [22, -47],
                [7, -46],
                [1, -33],
                [-2, -36],
                [13, -49],
                [-1, -51],
                [-5, -26],
                [-8, -52],
                [1, -33],
                [-5, -41],
                [-13, -52],
                [-20, -29],
                [-10, -44],
                [-10, -29],
                [-8, -49],
                [-11, -29],
                [-7, -43],
                [-3, -40],
                [1, -18],
                [-16, -20],
                [-31, -2],
                [-25, -24],
                [-13, -22],
                [-17, -25],
                [-23, 26],
                [-17, 10],
                [4, 30],
                [-15, -11],
                [-24, -42],
                [-24, 16],
                [-16, 9],
                [-16, 4],
                [-27, 17],
                [-18, 35],
                [-5, 44],
                [-6, 29],
                [-14, 23],
                [-27, 7],
                [9, 28],
                [-6, 43],
                [-14, -40],
                [-25, -10],
                [15, 31],
                [4, 34],
                [11, 28],
                [-2, 42],
                [-23, -49],
                [-17, -19],
                [-11, -46],
                [-22, 24],
                [1, 30],
                [-17, 42],
                [-15, 21],
                [5, 14],
                [-35, 35],
                [-20, 1],
                [-26, 28],
                [-50, -5],
                [-36, -21],
                [-32, -19],
                [-26, 4],
                [-30, -30],
                [-24, -13],
                [-5, -30],
                [-10, -24],
                [-24, -1],
                [-17, -5],
                [-25, 10],
                [-20, -6],
                [-19, -2],
                [-16, -31],
                [-9, 2],
                [-14, -16],
                [-13, -18],
                [-20, 2],
                [-19, 0],
                [-29, 37],
                [-15, 11],
                [0, 33],
                [14, 8],
                [5, 13],
                [-1, 20],
                [3, 41],
                [-3, 34],
                [-14, 58],
                [-5, 33],
                [1, 33],
                [-11, 37],
                [-1, 17],
                [-12, 23],
                [-3, 45],
                [-16, 46],
                [-4, 24],
                [12, -25],
                [-9, 54],
                [14, -17],
                [8, -22],
                [-1, 29],
                [-13, 45],
                [-3, 19],
                [-6, 17],
                [3, 33],
                [5, 14],
                [4, 29],
                [-3, 34],
                [12, 41],
                [2, -44],
                [11, 40],
                [23, 19],
                [13, 25],
                [22, 21],
                [12, 4],
                [8, -7],
                [22, 22],
                [17, 6],
                [4, 13],
                [7, 5],
                [16, -1],
                [29, 17],
                [15, 25],
                [7, 31],
                [16, 29],
                [2, 23],
                [0, 31],
                [20, 49],
                [11, -49],
                [12, 11],
                [-10, 27],
                [9, 28],
                [12, -12],
                [4, 44],
                [15, 28],
                [7, 23],
                [13, 9],
                [1, 17],
                [12, -7],
                [1, 14],
                [12, 9],
                [13, 7],
                [21, -26],
                [15, -34],
                [18, -1],
                [17, -5],
                [-6, 32],
                [14, 46],
                [12, 15],
                [-4, 14],
                [12, 33],
                [17, 20],
                [14, -6],
                [23, 10],
                [0, 30],
                [-21, 19],
                [15, 8],
                [19, -14],
                [14, -24],
                [24, -14],
                [8, 5],
                [17, -17],
                [16, 16],
                [11, -5],
                [6, 11],
                [13, -28],
                [-8, -31],
                [-10, -23],
                [-10, -2],
                [4, -23],
                [-9, -29],
                [-9, -28],
                [2, -17],
                [22, -31],
                [21, -19],
                [14, -20],
                [20, -34],
                [8, 0],
                [15, -15],
                [4, -17],
                [26, -20],
                [19, 20],
                [5, 31],
                [6, 25],
                [3, 32],
                [9, 46],
                [-4, 27],
                [2, 17],
                [-3, 33],
                [3, 44],
                [6, 11],
                [-5, 19],
                [7, 31],
                [5, 32],
                [1, 16],
                [10, 22],
                [8, -29],
                [2, -36],
                [7, -7],
                [1, -24],
                [10, -29],
                [2, -33],
                [-1, -21],
            ],
            [[9502, 4578], [8, -19], [-20, 0], [-10, 35], [16, -14], [6, -2]],
            [[8352, 4592], [-12, -1], [-37, 41], [26, 11], [15, -18], [10, -17], [-2, -16]],
            [[9467, 4613], [-11, -1], [-17, 6], [-6, 9], [2, 23], [18, -9], [9, -12], [5, -16]],
            [[9490, 4629], [-5, -10], [-20, 49], [-6, 35], [10, 0], [10, -46], [11, -28]],
            [[8470, 4670], [3, 13], [24, 13], [20, 2], [8, 8], [11, -8], [-10, -15], [-29, -25], [-24, -17]],
            [[8473, 4641], [-18, -43], [-24, -13], [-3, 7], [3, 20], [12, 35], [27, 23]],
            [[8274, 4716], [10, -16], [17, 5], [7, -24], [-32, -12], [-20, -8], [-15, 1], [10, 33], [15, 0], [8, 21]],
            [[8413, 4716], [-4, -32], [-42, -16], [-37, 7], [0, 21], [22, 12], [17, -18], [19, 5], [25, 21]],
            [[9440, 4702], [1, -12], [-22, 25], [-15, 20], [-11, 20], [4, 5], [13, -13], [23, -27], [7, -18]],
            [[9375, 4759], [-6, -3], [-12, 13], [-11, 23], [1, 10], [17, -24], [11, -19]],
            [
                [8016, 4792],
                [53, -6],
                [6, 24],
                [52, -28],
                [10, -37],
                [42, -11],
                [34, -34],
                [-32, -22],
                [-31, 23],
                [-25, -1],
                [-28, 4],
                [-26, 10],
                [-33, 22],
                [-20, 6],
                [-12, -7],
                [-50, 23],
                [-5, 25],
                [-25, 4],
                [19, 55],
                [33, -3],
                [23, -22],
                [11, -5],
                [4, -20],
            ],
            [[8741, 4824], [-14, -39], [-3, 43], [5, 21], [6, 19], [6, -16], [0, -28]],
            [[9329, 4789], [-8, -6], [-12, 23], [-12, 36], [-6, 44], [3, 6], [3, -18], [9, -13], [13, -36], [13, -20], [-3, -16]],
            [
                [9220, 4867],
                [-14, -5],
                [-5, -16],
                [-15, -14],
                [-14, -14],
                [-15, 0],
                [-23, 17],
                [-15, 16],
                [2, 18],
                [25, -9],
                [15, 5],
                [4, 28],
                [4, 1],
                [3, -31],
                [16, 5],
                [8, 20],
                [15, 20],
                [-3, 34],
                [17, 1],
                [5, -9],
                [0, -32],
                [-10, -35],
            ],
            [[8533, 4983], [-10, -19], [-19, 10], [-6, 25], [28, 3], [7, -19]],
            [[8623, 5004], [10, -44], [-24, 24], [-23, 4], [-15, -3], [-20, 2], [7, 31], [34, 3], [31, -17]],
            [
                [9252, 4923],
                [-8, -15],
                [-5, 33],
                [-7, 23],
                [-12, 19],
                [-16, 24],
                [-20, 17],
                [7, 14],
                [15, -16],
                [10, -13],
                [11, -14],
                [12, -24],
                [10, -18],
                [3, -30],
            ],
            [
                [8915, 5032],
                [48, -39],
                [52, -33],
                [19, -30],
                [15, -29],
                [5, -34],
                [46, -35],
                [7, -31],
                [-26, -6],
                [6, -38],
                [25, -38],
                [18, -61],
                [16, 2],
                [-1, -26],
                [21, -10],
                [-8, -11],
                [30, -24],
                [-4, -16],
                [-18, -4],
                [-7, 14],
                [-24, 7],
                [-28, 9],
                [-21, 36],
                [-16, 32],
                [-15, 50],
                [-36, 26],
                [-23, -17],
                [-17, -19],
                [3, -42],
                [-21, -20],
                [-16, 10],
                [-29, 2],
            ],
            [
                [8916, 4657],
                [-25, 47],
                [-28, 12],
                [-7, -17],
                [-35, -1],
                [12, 46],
                [18, 16],
                [-8, 63],
                [-13, 48],
                [-54, 49],
                [-23, 5],
                [-42, 53],
                [-8, -28],
                [-10, -5],
                [-7, 21],
                [0, 25],
                [-21, 29],
                [30, 20],
                [20, -1],
                [-3, 15],
                [-40, 1],
                [-11, 34],
                [-25, 10],
                [-12, 29],
                [38, 14],
                [14, 19],
                [44, -24],
                [5, -21],
                [8, -93],
                [28, -35],
                [24, 61],
                [31, 35],
                [25, 0],
                [24, -20],
                [20, -21],
                [30, -11],
            ],
            [
                [8478, 5264],
                [-23, -57],
                [-21, -11],
                [-26, 11],
                [-47, -3],
                [-24, -8],
                [-4, -44],
                [25, -51],
                [15, 26],
                [52, 20],
                [-2, -27],
                [-13, 8],
                [-12, -33],
                [-24, -23],
                [26, -73],
                [-5, -20],
                [25, -67],
                [0, -38],
                [-15, -16],
                [-11, 20],
                [14, 47],
                [-28, -22],
                [-7, 16],
                [4, 22],
                [-20, 34],
                [2, 56],
                [-19, -18],
                [3, -67],
                [1, -82],
                [-18, -9],
                [-12, 17],
                [8, 53],
                [-4, 56],
                [-12, 0],
                [-8, 40],
                [11, 37],
                [4, 46],
                [14, 87],
                [6, 24],
                [24, 42],
                [21, -17],
                [35, -8],
                [32, 3],
                [28, 42],
                [5, -13],
            ],
            [[8573, 5247], [-1, -50], [-14, 5], [-5, -35], [12, -30], [-8, -7], [-11, 37], [-8, 73], [5, 46], [9, 21], [2, -31], [17, -5], [2, -24]],
            [
                [7938, 4845],
                [-31, -1],
                [-23, 48],
                [-36, 47],
                [-12, 35],
                [-21, 47],
                [-13, 43],
                [-22, 81],
                [-24, 48],
                [-8, 49],
                [-10, 45],
                [-25, 36],
                [-15, 49],
                [-21, 33],
                [-29, 63],
                [-2, 30],
                [18, -3],
                [43, -11],
                [24, -56],
                [22, -39],
                [15, -24],
                [26, -62],
                [29, -1],
                [23, -39],
                [16, -49],
                [21, -26],
                [-11, -47],
                [16, -20],
                [10, -2],
                [5, -40],
                [9, -32],
                [21, -5],
                [13, -36],
                [-7, -72],
                [-1, -89],
            ],
            [[8045, 5298], [20, -20], [22, 11], [5, 48], [12, 11], [33, 13], [20, 45], [14, 37]],
            [[8171, 5443], [11, 21], [24, 32]],
            [[8206, 5496], [21, 40], [14, 45], [11, 0], [15, -29], [1, -25], [18, -16], [23, -18], [-2, -22], [-18, -3], [5, -28], [-21, -20]],
            [
                [8273, 5420],
                [-16, -52],
                [21, -54],
                [-5, -27],
                [31, -53],
                [-33, -7],
                [-9, -39],
                [1, -52],
                [-27, -40],
                [0, -57],
                [-11, -88],
                [-4, 20],
                [-32, -26],
                [-11, 36],
                [-19, 3],
                [-14, 18],
                [-33, -20],
                [-10, 28],
                [-19, -4],
                [-23, 7],
                [-4, 77],
                [-14, 16],
                [-13, 50],
                [-4, 50],
                [3, 53],
                [17, 39],
            ],
            [
                [8509, 5667],
                [3, -39],
                [2, -32],
                [-10, -53],
                [-10, 59],
                [-13, -29],
                [9, -43],
                [-8, -27],
                [-33, 34],
                [-8, 41],
                [9, 28],
                [-18, 27],
                [-8, -24],
                [-14, 2],
                [-20, -32],
                [-5, 17],
                [11, 49],
                [18, 16],
                [15, 22],
                [10, -27],
                [21, 16],
                [4, 26],
                [20, 1],
                [-2, 45],
                [23, -27],
                [2, -29],
                [2, -21],
            ],
            [[7255, 5539], [-24, -13], [-14, 45], [-4, 83], [12, 94], [19, -32], [13, -41], [14, -60], [-5, -60], [-11, -16]],
            [[3307, 5764], [-24, -6], [-5, 5], [8, 16], [0, 23], [16, 7], [6, -2], [-1, -43]],
            [[8443, 5774], [-10, -19], [-9, -36], [-9, -18], [-17, 40], [6, 16], [7, 16], [3, 36], [15, 3], [-4, -39], [20, 56], [-2, -55]],
            [[8290, 5718], [-36, -54], [13, 40], [20, 36], [17, 39], [14, 58], [5, -47], [-18, -32], [-15, -40]],
            [[8384, 5867], [17, -18], [18, 0], [-1, -24], [-13, -25], [-17, -17], [-1, 27], [2, 29], [-5, 28]],
            [
                [8485, 5882],
                [8, -64],
                [-22, 15],
                [1, -19],
                [7, -36],
                [-14, -12],
                [-1, 40],
                [-8, 3],
                [-4, 35],
                [16, -5],
                [-1, 22],
                [-16, 44],
                [26, -1],
                [8, -22],
            ],
            [[8374, 5935], [-7, -50], [-12, 29], [-14, 43], [24, -2], [9, -20]],
            [
                [8369, 6247],
                [17, -16],
                [8, 15],
                [3, -15],
                [-5, -23],
                [10, -42],
                [-7, -48],
                [-17, -19],
                [-4, -46],
                [6, -46],
                [15, -6],
                [12, 7],
                [35, -32],
                [-3, -32],
                [9, -14],
                [-3, -26],
                [-21, 28],
                [-11, 30],
                [-7, -21],
                [-17, 35],
                [-26, -9],
                [-14, 13],
                [2, 24],
                [9, 14],
                [-9, 14],
                [-3, -21],
                [-14, 33],
                [-4, 25],
                [-1, 55],
                [11, -19],
                [3, 90],
                [9, 53],
                [17, -1],
            ],
            [[3177, 6232], [-7, -15], [-21, 0], [-16, -2], [-2, 25], [4, 8], [23, 0], [14, -5], [5, -11]],
            [[2863, 6211], [-8, -10], [-16, 9], [-16, 21], [4, 14], [11, 4], [7, -2], [18, -5], [15, -14], [5, -16], [-20, -1]],
            [
                [3007, 6317],
                [4, 10],
                [21, 0],
                [17, -15],
                [7, 1],
                [5, -20],
                [15, 1],
                [-1, -17],
                [13, -2],
                [13, -21],
                [-10, -24],
                [-13, 13],
                [-13, -3],
                [-9, 3],
                [-5, -10],
                [-11, -4],
                [-4, 14],
                [-9, -8],
                [-11, -40],
                [-7, 9],
                [-2, 17],
            ],
            [
                [3007, 6221],
                [-18, 10],
                [-13, -4],
                [-17, 4],
                [-13, -11],
                [-15, 18],
                [2, 19],
                [26, -8],
                [21, -5],
                [10, 13],
                [-13, 25],
                [0, 22],
                [-17, 9],
                [6, 16],
                [17, -3],
                [24, -9],
            ],
            [[8064, 6258], [-24, -28], [-23, 18], [-1, 49], [14, 26], [30, 16], [16, -1], [6, -22], [-12, -25], [-6, -33]],
            [
                [679, 6281],
                [-4, -10],
                [-7, 8],
                [1, 17],
                [-5, 21],
                [1, 6],
                [5, 9],
                [-2, 12],
                [2, 5],
                [2, -1],
                [11, -10],
                [5, -5],
                [4, -7],
                [7, -21],
                [0, -3],
                [-11, -12],
                [-9, -9],
            ],
            [[664, 6371], [-9, -4], [-5, 12], [-3, 4], [-1, 4], [3, 5], [10, -6], [7, -8], [-2, -7]],
            [[645, 6401], [-1, -6], [-15, 2], [2, 7], [14, -3]],
            [[620, 6410], [-1, -4], [-2, 1], [-10, 2], [-3, 13], [-2, 3], [8, 7], [2, -3], [8, -19]],
            [[573, 6448], [-3, -6], [-9, 11], [1, 4], [4, 5], [7, -1], [0, -13]],
            [
                [2786, 6493],
                [11, -21],
                [26, 6],
                [10, -13],
                [23, -36],
                [18, -26],
                [9, 1],
                [16, -12],
                [-2, -16],
                [21, -2],
                [21, -24],
                [-4, -13],
                [-18, -8],
                [-19, -3],
                [-19, 5],
                [-40, -6],
                [19, 32],
                [-11, 15],
                [-18, 4],
                [-10, 17],
                [-6, 33],
                [-16, -3],
                [-26, 16],
                [-8, 12],
                [-37, 9],
                [-9, 11],
                [10, 14],
                [-27, 3],
                [-20, -30],
                [-12, 0],
                [-4, -14],
                [-13, -7],
                [-12, 6],
                [14, 18],
                [7, 20],
                [12, 13],
                [14, 11],
                [21, 6],
                [7, 6],
                [24, -4],
                [22, -1],
                [26, -19],
            ],
            [[2845, 6550], [-6, -3], [-7, 33], [-11, 17], [6, 37], [9, -3], [9, -47], [0, -34]],
            [[8365, 6494], [-12, -47], [-15, 49], [-3, 42], [16, 57], [23, 44], [12, -18], [-5, -34], [-16, -93]],
            [[2838, 6713], [-31, -10], [-2, 22], [13, 4], [19, -2], [1, -14]],
            [[2860, 6713], [-4, -41], [-6, 8], [1, 30], [-13, 22], [0, 7], [22, -26]],
            [[8739, 7148], [3, -19], [-16, -35], [-11, 18], [-14, -13], [-8, -34], [-18, 17], [1, 27], [15, 34], [16, -6], [11, 24], [21, -13]],
            [[5943, 7201], [0, -5], [-28, -23], [-14, 7], [-6, 23], [13, 2]],
            [[5908, 7205], [2, 1], [4, 14], [20, -1], [25, 17], [-18, -24], [2, -11]],
            [[5657, 7238], [15, -19], [22, 3], [21, -4], [-1, -10], [15, 7], [-3, -17], [-40, -5], [0, 9], [-34, 11], [5, 25]],
            [[5430, 7383], [-10, -45], [4, -18], [-6, -29], [-21, 21], [-14, 7], [-39, 29], [4, 29], [33, -5], [28, 6], [21, 5]],
            [[5255, 7555], [16, -41], [-3, -76], [-13, 4], [-11, -20], [-11, 16], [-1, 69], [-6, 33], [15, -3], [14, 18]],
            [
                [8915, 7321],
                [-11, -46],
                [5, -29],
                [-14, -41],
                [-36, -27],
                [-49, -3],
                [-39, -66],
                [-19, 22],
                [-1, 43],
                [-48, -13],
                [-33, -27],
                [-33, -1],
                [28, -42],
                [-18, -98],
                [-18, -24],
                [-14, 22],
                [7, 52],
                [-17, 17],
                [-12, 39],
                [27, 18],
                [14, 36],
                [28, 30],
                [20, 39],
                [56, 17],
                [29, -11],
                [30, 102],
                [18, -27],
                [41, 57],
                [16, 23],
                [17, 70],
                [-5, 65],
                [12, 36],
                [30, 11],
                [15, -80],
                [-1, -47],
                [-26, -58],
                [1, -59],
            ],
            [[5265, 7609], [-10, -44], [-12, 11], [-7, 39], [6, 22], [18, 22], [5, -50]],
            [
                [8996, 7726],
                [20, -13],
                [19, 25],
                [6, -65],
                [-41, -16],
                [-24, -57],
                [-44, 40],
                [-15, -63],
                [-31, -1],
                [-4, 57],
                [14, 44],
                [30, 3],
                [8, 80],
                [8, 45],
                [33, -60],
                [21, -19],
            ],
            [[3231, 7862], [20, -7], [26, 1], [-14, -23], [-10, -4], [-36, 24], [-7, 20], [11, 17], [10, -28]],
            [[3282, 8010], [-13, -1], [-36, 18], [-26, 27], [10, 5], [36, -15], [29, -24], [0, -10]],
            [
                [1569, 7975],
                [-14, -8],
                [-46, 26],
                [-8, 21],
                [-25, 20],
                [-5, 16],
                [-29, 11],
                [-11, 31],
                [3, 13],
                [29, -12],
                [17, -9],
                [26, -6],
                [10, -20],
                [13, -27],
                [28, -24],
                [12, -32],
            ],
            [
                [3440, 8101],
                [-19, -51],
                [19, 20],
                [18, -13],
                [-9, -20],
                [24, -15],
                [13, 14],
                [28, -18],
                [-9, -42],
                [20, 10],
                [3, -31],
                [9, -36],
                [-12, -50],
                [-13, -3],
                [-18, 11],
                [6, 47],
                [-8, 8],
                [-32, -50],
                [-16, 2],
                [19, 27],
                [-26, 14],
                [-30, -4],
                [-54, 2],
                [-4, 17],
                [17, 20],
                [-12, 16],
                [23, 35],
                [29, 91],
                [17, 33],
                [24, 20],
                [13, -2],
                [-5, -16],
                [-15, -36],
            ],
            [[1313, 8294], [27, 4], [-9, -65], [25, -46], [-12, 0], [-16, 26], [-11, 27], [-14, 18], [-5, 25], [2, 18], [13, -7]],
            [
                [8989, 8104],
                [28, -102],
                [-41, 19],
                [-17, -83],
                [27, -59],
                [-1, -40],
                [-21, 34],
                [-18, -44],
                [-6, 48],
                [4, 56],
                [-4, 62],
                [7, 44],
                [1, 77],
                [-16, 57],
                [2, 78],
                [26, 27],
                [-11, 26],
                [12, 9],
                [7, -39],
                [10, -55],
                [-1, -57],
                [12, -58],
            ],
            [[4789, 8357], [23, 2], [30, -36], [-15, -39]],
            [[4827, 8284], [4, -41], [-21, -52], [-49, -34], [-39, 9], [22, 60], [-14, 59], [38, 45], [21, 27]],
            [[5351, 8384], [-16, -46], [-29, 32], [-4, 24], [41, 19], [8, -29]],
            [[749, 8471], [-27, -22], [-15, 15], [-4, 27], [25, 20], [15, 9], [19, -4], [11, -18], [-24, -27]],
            [
                [4916, 8558],
                [-30, -62],
                [28, 8],
                [31, 0],
                [-8, -47],
                [-25, -52],
                [29, -4],
                [27, -74],
                [19, -9],
                [17, -65],
                [8, -23],
                [34, -11],
                [-4, -37],
                [-14, -17],
                [11, -30],
                [-25, -30],
                [-37, 1],
                [-47, -16],
                [-13, 11],
                [-18, -27],
                [-26, 7],
                [-20, -22],
                [-14, 11],
                [40, 61],
                [25, 12],
                [-43, 10],
                [-8, 23],
                [29, 18],
                [-15, 31],
                [5, 37],
                [41, -5],
                [4, 34],
                [-19, 36],
                [-34, 10],
                [-6, 16],
                [10, 25],
                [-9, 16],
                [-15, -27],
                [-2, 55],
                [-14, 30],
                [10, 59],
                [22, 47],
                [22, -5],
                [34, 5],
            ],
            [[400, 8632], [-17, -9], [-18, 11], [-17, 15], [27, 10], [22, -5], [3, -22]],
            [[2797, 8761], [-10, -30], [-13, 5], [-7, 17], [1, 4], [11, 17], [11, -1], [7, -12]],
            [[2724, 8793], [-32, -32], [-20, 2], [-6, 15], [21, 27], [38, -1], [-1, -11]],
            [[229, 8855], [17, -11], [18, 6], [22, -15], [28, -8], [-3, -6], [-21, -13], [-21, 13], [-10, 10], [-25, -3], [-6, 5], [1, 22]],
            [
                [2634, 8963],
                [5, -26],
                [14, 9],
                [16, -15],
                [31, -20],
                [31, -18],
                [3, -27],
                [20, 5],
                [20, -20],
                [-25, -18],
                [-43, 14],
                [-15, 26],
                [-28, -31],
                [-39, -29],
                [-10, 33],
                [-38, -5],
                [25, 28],
                [3, 46],
                [10, 52],
                [20, -4],
            ],
            [
                [4596, 9009],
                [-6, -38],
                [31, -39],
                [-36, -44],
                [-80, -39],
                [-24, -11],
                [-37, 9],
                [-77, 18],
                [27, 25],
                [-60, 29],
                [49, 11],
                [-1, 17],
                [-59, 13],
                [19, 38],
                [42, 8],
                [43, -39],
                [43, 31],
                [35, -16],
                [45, 31],
                [46, -4],
            ],
            [[2892, 9049], [-31, -3], [-7, 28], [12, 32], [25, 8], [22, -16], [0, -24], [-3, -8], [-18, -17]],
            [
                [138, 9016],
                [19, -14],
                [-7, 42],
                [76, -9],
                [54, -54],
                [-27, -25],
                [-46, -6],
                [-1, -56],
                [-11, -12],
                [-26, 2],
                [-21, 20],
                [-37, 16],
                [-6, 25],
                [-28, 10],
                [-32, -8],
                [-15, 20],
                [6, 22],
                [-33, -14],
                [12, -27],
                [-15, -24],
                [0, 229],
                [68, -44],
                [72, -57],
                [-2, -36],
            ],
            [[2342, 9161], [-17, -20], [-37, 18], [-23, -7], [-38, 26], [25, 18], [19, 25], [29, -16], [17, -11], [8, -11], [17, -22]],
            [[9999, 9261], [-31, -3], [-5, 18], [36, 24], [0, -39]],
            [[36, 9264], [-36, -3], [0, 39], [3, 2], [24, 0], [40, -16], [-3, -8], [-28, -14]],
            [[3134, 7781], [5, -19], [-30, -28], [-28, -20], [-29, -17]],
            [[3052, 7697], [-16, -37], [-4, -10]],
            [
                [3032, 7650],
                [0, -30],
                [9, -31],
                [12, -1],
                [-3, 21],
                [8, -13],
                [-2, -16],
                [-19, -10],
                [-13, 1],
                [-21, -10],
                [-12, -3],
                [-16, -2],
                [-23, -17],
            ],
            [[2952, 7539], [41, 11], [8, -11]],
            [
                [3001, 7539],
                [-39, -17],
                [-18, -1],
                [1, 8],
                [-8, -16],
                [8, -3],
                [-6, -41],
                [-21, -45],
                [-2, 15],
                [-6, 3],
                [-9, 14],
                [6, -31],
                [7, -10],
                [0, -22],
                [-9, -22],
                [-15, -46],
                [-3, 2],
                [9, 39],
            ],
            [[2896, 7366], [-14, 23], [-4, 47]],
            [
                [2878, 7436],
                [-5, -25],
                [6, -36],
                [-18, 9],
                [19, -19],
                [1, -54],
                [8, -4],
                [3, -20],
                [4, -58],
                [-18, -43],
                [-29, -17],
                [-18, -34],
                [-14, -3],
                [-14, -21],
                [-4, -20],
                [-30, -37],
                [-16, -27],
                [-13, -35],
                [-4, -41],
                [5, -39],
                [9, -50],
                [12, -41],
                [0, -24],
                [13, -67],
                [0, -39],
                [-2, -22],
                [-7, -36],
                [-8, -7],
                [-14, 7],
                [-4, 25],
                [-10, 14],
                [-15, 49],
                [-13, 44],
                [-4, 23],
                [5, 38],
                [-7, 32],
                [-22, 48],
                [-11, 9],
                [-28, -26],
                [-5, 2],
                [-13, 27],
                [-18, 14],
                [-31, -7],
                [-25, 7],
                [-21, -4],
            ],
            [[2522, 6928], [-12, -8], [6, -17]],
            [
                [2516, 6903],
                [-1, -23],
                [6, -11],
                [-5, -8],
                [-11, 9],
                [-10, -11],
                [-20, 1],
                [-21, 31],
                [-24, -7],
                [-20, 13],
                [-18, -4],
                [-23, -13],
                [-25, -43],
                [-28, -25],
            ],
            [[2316, 6812], [-15, -27], [-6, -26]],
            [[2295, 6759], [-1, -40], [2, -28], [5, -19]],
            [[2301, 6672], [0, -1], [-11, -50]],
            [
                [2290, 6621],
                [-5, -41],
                [-2, -78],
                [-2, -28],
                [4, -31],
                [9, -28],
                [6, -45],
                [18, -43],
                [6, -33],
                [11, -28],
                [30, -15],
                [11, -24],
                [25, 16],
                [21, 6],
                [21, 10],
                [17, 10],
                [18, 23],
                [6, 34],
                [3, 48],
                [5, 17],
                [18, 15],
                [30, 14],
                [24, -2],
                [17, 5],
                [7, -13],
                [-1, -27],
                [-15, -35],
                [-7, -35],
                [5, -10],
                [-4, -25],
                [-7, -45],
                [-7, 15],
                [-6, -1],
            ],
            [[2546, 6247], [1, -8], [5, 0], [-1, -16], [-4, -25], [2, -9], [-3, -21], [2, -5], [-3, -29], [-6, -15], [-5, -2], [-5, -20]],
            [[2529, 6097], [9, -11], [2, 9], [9, -7]],
            [
                [2549, 6088],
                [2, -3],
                [7, 10],
                [7, 1],
                [3, -4],
                [4, 2],
                [13, -5],
                [13, 2],
                [9, 6],
                [3, 7],
                [9, -3],
                [7, -4],
                [7, 1],
                [6, 5],
                [12, -8],
                [5, -1],
                [8, -11],
                [8, -13],
                [10, -9],
                [8, -16],
            ],
            [
                [2690, 6045],
                [-3, -5],
                [-1, -13],
                [3, -21],
                [-7, -20],
                [-3, -23],
                [-1, -25],
                [2, -15],
                [1, -26],
                [-5, -6],
                [-2, -24],
                [2, -15],
                [-6, -15],
                [1, -16],
                [4, -9],
            ],
            [[2675, 5812], [8, -31], [10, -24], [13, -24]],
            [
                [2706, 5733],
                [10, -21],
                [0, -12],
                [11, -3],
                [2, 5],
                [8, -14],
                [14, 4],
                [12, 15],
                [16, 11],
                [10, 17],
                [15, -3],
                [-1, -6],
                [16, -2],
                [12, -10],
                [9, -17],
                [11, -16],
            ],
            [
                [2851, 5681],
                [14, -2],
                [21, 41],
                [11, 6],
                [0, 19],
                [6, 48],
                [15, 27],
                [18, 1],
                [2, 12],
                [22, -5],
                [22, 30],
                [11, 12],
                [13, 28],
                [10, -3],
                [7, -16],
                [-5, -19],
            ],
            [
                [3018, 5860],
                [-1, -14],
                [-16, -6],
                [9, -26],
                [-1, -30],
                [-12, -34],
                [11, -46],
                [12, 4],
                [6, 42],
                [-9, 20],
                [-1, 44],
                [34, 23],
                [-3, 27],
                [9, 18],
                [10, -40],
                [20, -1],
                [18, -32],
                [1, -19],
                [25, -1],
                [30, 6],
                [15, -26],
                [22, -7],
                [15, 18],
                [1, 15],
                [34, 3],
                [33, 1],
                [-23, -17],
                [9, -27],
                [22, -5],
                [21, -28],
                [5, -46],
                [14, 1],
                [11, -13],
            ],
            [[3339, 5664], [18, -21], [18, -38], [0, -30], [11, -1], [15, -28], [11, -20]],
            [[3412, 5526], [33, -12], [3, 11], [22, 4], [30, -16]],
            [[3500, 5513], [10, -6], [20, -14], [30, -48], [4, -24]],
            [
                [3564, 5421],
                [10, 3],
                [7, -32],
                [15, -101],
                [15, -9],
                [1, -40],
                [-21, -47],
                [8, -18],
                [50, -9],
                [1, -57],
                [21, 37],
                [35, -20],
                [46, -35],
                [13, -34],
                [-4, -32],
                [32, 18],
                [54, -31],
                [42, 3],
                [41, -48],
                [35, -65],
                [22, -16],
                [23, -3],
                [10, -18],
                [10, -73],
                [4, -35],
                [-11, -95],
                [-14, -38],
                [-39, -80],
                [-18, -65],
                [-20, -50],
                [-7, -1],
                [-8, -42],
                [2, -108],
                [-8, -89],
                [-3, -38],
                [-8, -23],
                [-5, -77],
                [-29, -75],
                [-4, -59],
                [-23, -25],
                [-6, -35],
                [-30, 0],
                [-44, -22],
                [-20, -25],
                [-31, -17],
                [-32, -46],
                [-24, -57],
                [-4, -43],
                [5, -32],
                [-6, -58],
                [-6, -28],
                [-19, -32],
                [-31, -101],
                [-25, -46],
                [-18, -27],
                [-13, -55],
                [-18, -33],
            ],
            [[3517, 3237], [-12, -36], [-32, -32], [-20, 12], [-15, -6], [-26, 24], [-19, -1], [-17, 31]],
            [
                [3376, 3229],
                [-2, -30],
                [36, -49],
                [-4, -40],
                [17, -25],
                [-1, -28],
                [-27, -74],
                [-41, -31],
                [-56, -12],
                [-30, 6],
                [5, -34],
                [-5, -43],
                [5, -29],
                [-17, -21],
                [-28, -8],
                [-27, 21],
                [-11, -15],
                [4, -57],
                [19, -17],
                [15, 18],
                [8, -30],
                [-25, -18],
                [-22, -36],
                [-5, -58],
                [-6, -30],
                [-26, -1],
                [-22, -29],
                [-8, -43],
                [27, -42],
                [27, -12],
                [-10, -52],
                [-33, -32],
                [-18, -68],
                [-25, -22],
                [-11, -27],
                [9, -60],
                [18, -34],
                [-12, 3],
            ],
            [
                [3094, 2170],
                [-24, 1],
                [-14, -14],
                [-25, -21],
                [-4, -54],
                [-12, -1],
                [-31, 18],
                [-32, 41],
                [-35, 33],
                [-8, 36],
                [8, 34],
                [-14, 38],
                [-4, 98],
                [12, 56],
                [29, 44],
                [-42, 17],
                [26, 51],
                [10, 95],
                [31, -20],
                [14, 119],
                [-18, 16],
                [-9, -72],
                [-18, 8],
                [9, 82],
                [10, 107],
                [12, 39],
                [-8, 57],
                [-2, 64],
                [12, 2],
                [17, 93],
                [19, 92],
                [12, 86],
                [-7, 86],
                [9, 48],
                [-4, 71],
                [17, 70],
                [5, 112],
                [9, 119],
                [8, 129],
                [-2, 94],
                [-6, 81],
            ],
            [
                [3044, 4125],
                [-27, 33],
                [-3, 24],
                [-55, 58],
                [-50, 63],
                [-21, 35],
                [-12, 48],
                [5, 17],
                [-24, 75],
                [-27, 106],
                [-26, 115],
                [-12, 26],
                [-8, 43],
                [-22, 37],
                [-20, 24],
                [9, 25],
                [-13, 55],
                [8, 41],
                [23, 36],
            ],
            [[2769, 4986], [14, 43], [-6, 25], [-10, -27], [-17, 26], [6, 16], [-5, 52], [10, 9], [5, 36], [10, 37], [-2, 23], [16, 13], [19, 22]],
            [
                [2809, 5261],
                [-4, 18],
                [10, 5],
                [-1, 29],
                [7, 20],
                [13, 4],
                [12, 36],
                [11, 31],
                [-10, 13],
                [5, 34],
                [-6, 52],
                [5, 16],
                [-4, 48],
                [-11, 31],
            ],
            [
                [2836, 5598],
                [-9, 17],
                [-6, 31],
                [6, 15],
                [-7, 4],
                [-5, 19],
                [-14, 16],
                [-12, -4],
                [-5, -20],
                [-12, -14],
                [-6, -2],
                [-2, -12],
                [13, -31],
                [-8, -8],
                [-4, -8],
                [-13, -3],
                [-4, 34],
                [-4, -10],
                [-9, 4],
                [-6, 23],
                [-11, 4],
                [-7, 6],
                [-12, 0],
                [-1, -12],
                [-3, 9],
            ],
            [
                [2695, 5656],
                [-15, 12],
                [-6, 12],
                [3, 10],
                [-1, 13],
                [-8, 14],
                [-11, 11],
                [-9, 8],
                [-2, 17],
                [-7, 10],
                [2, -17],
                [-6, -14],
                [-6, 16],
                [-9, 6],
                [-4, 12],
                [0, 17],
                [4, 18],
                [-8, 8],
                [6, 11],
            ],
            [[2618, 5820], [-9, 19], [-13, 23], [-6, 19], [-12, 18], [-14, 26], [3, 9], [5, -8], [2, 4]],
            [[2574, 5930], [-5, 18], [-8, 5]],
            [[2561, 5953], [-4, -14], [-16, 1]],
            [[2541, 5940], [-10, 5], [-11, 12]],
            [[2520, 5957], [-16, 4], [-7, 12]],
            [[2497, 5973], [-15, 10], [-17, 1], [-13, 11], [-15, 24]],
            [
                [2437, 6019],
                [-31, 62],
                [-14, 19],
                [-23, 15],
                [-16, -4],
                [-22, -22],
                [-14, -6],
                [-19, 16],
                [-21, 10],
                [-26, 27],
                [-21, 8],
                [-32, 27],
                [-23, 27],
                [-7, 16],
                [-15, 3],
                [-29, 18],
                [-11, 27],
                [-30, 32],
                [-14, 37],
                [-7, 28],
                [10, 5],
                [-3, 17],
                [6, 15],
                [0, 20],
                [-9, 25],
                [-3, 23],
                [-9, 29],
                [-24, 58],
                [-28, 45],
                [-14, 35],
                [-24, 24],
                [-5, 14],
                [4, 36],
                [-14, 13],
                [-16, 28],
                [-7, 40],
                [-15, 5],
                [-16, 30],
                [-13, 28],
                [-1, 18],
                [-15, 44],
                [-10, 44],
                [0, 22],
                [-20, 23],
                [-9, -3],
                [-16, 16],
                [-4, -23],
                [4, -28],
                [3, -43],
                [9, -24],
                [21, -40],
                [5, -13],
                [4, -4],
                [4, -20],
                [4, 1],
                [6, -37],
                [8, -15],
                [6, -20],
                [18, -30],
                [9, -53],
                [8, -25],
                [8, -27],
                [2, -31],
                [13, -2],
                [11, -26],
                [10, -26],
                [-1, -10],
                [-11, -21],
                [-5, 0],
                [-7, 35],
                [-19, 33],
                [-20, 28],
                [-14, 14],
                [1, 43],
                [-4, 31],
                [-13, 18],
                [-19, 25],
                [-4, -7],
                [-7, 15],
                [-17, 14],
                [-17, 33],
                [2, 5],
                [12, -4],
                [10, 22],
                [1, 26],
                [-21, 41],
                [-17, 16],
                [-10, 36],
                [-10, 38],
                [-13, 46],
                [-11, 51],
            ],
            [
                [1746, 7055],
                [-5, 30],
                [-18, 33],
                [-13, 7],
                [-3, 16],
                [-15, 3],
                [-10, 16],
                [-26, 6],
                [-7, 9],
                [-4, 31],
                [-27, 58],
                [-23, 80],
                [1, 14],
                [-12, 19],
                [-22, 48],
                [-3, 47],
                [-15, 31],
                [6, 48],
                [-1, 49],
                [-9, 45],
                [11, 54],
                [7, 104],
                [-5, 78],
                [-9, 49],
                [-8, 27],
                [3, 11],
                [40, -20],
                [15, -54],
                [7, 15],
                [-4, 47],
                [-10, 48],
            ],
            [[1587, 8004], [-4, 0], [-53, 56], [-20, 25], [-51, 24], [-15, 51], [4, 35], [-36, 25], [-4, 46], [-34, 42], [-1, 30]],
            [
                [1373, 8338],
                [-15, 21],
                [-24, 19],
                [-8, 50],
                [-36, 46],
                [-15, 55],
                [-27, 4],
                [-44, 1],
                [-32, 17],
                [-58, 59],
                [-26, 11],
                [-49, 21],
                [-38, -5],
                [-55, 26],
                [-33, 25],
                [-31, -12],
                [6, -40],
                [-16, -4],
                [-32, -12],
                [-24, -19],
                [-31, -13],
                [-4, 34],
                [13, 57],
                [29, 17],
                [-8, 15],
                [-35, -32],
                [-19, -39],
                [-40, -40],
                [20, -28],
                [-26, -42],
                [-30, -24],
                [-27, -17],
                [-7, -26],
                [-44, -30],
                [-8, -27],
                [-33, -24],
                [-19, 4],
                [-26, -16],
                [-28, -20],
                [-23, -19],
                [-48, -16],
                [-4, 9],
                [30, 27],
                [27, 18],
                [30, 32],
                [34, 6],
                [14, 24],
                [39, 34],
                [6, 12],
                [20, 20],
                [5, 44],
                [14, 34],
                [-32, -18],
                [-9, 10],
                [-15, -21],
                [-18, 29],
                [-7, -20],
                [-11, 28],
                [-28, -23],
                [-17, 0],
                [-2, 35],
                [5, 21],
                [-18, 20],
                [-36, -11],
                [-23, 27],
                [-19, 14],
                [-1, 33],
                [-21, 24],
                [11, 33],
                [22, 33],
                [10, 29],
                [23, 4],
                [19, -9],
                [22, 28],
                [20, -5],
                [22, 18],
                [-6, 26],
                [-15, 10],
                [20, 23],
                [-17, -1],
                [-29, -13],
                [-9, -12],
                [-22, 12],
                [-39, -6],
                [-40, 14],
                [-12, 23],
                [-35, 33],
                [39, 25],
                [62, 28],
                [23, 0],
                [-4, -29],
                [58, 2],
                [-22, 36],
                [-34, 22],
                [-20, 29],
                [-27, 24],
                [-38, 18],
                [16, 30],
                [49, 2],
                [35, 27],
                [7, 28],
                [28, 27],
                [27, 7],
                [53, 25],
                [25, -4],
                [43, 31],
                [42, -12],
                [20, -26],
                [12, 11],
                [47, -3],
                [-1, -14],
                [42, -9],
                [29, 5],
                [58, -18],
                [53, -5],
                [22, -8],
                [37, 10],
                [42, -18],
                [30, -8],
            ],
            [[1083, 9196], [52, -14], [44, -27], [29, -6], [24, 24], [34, 18], [41, -7], [41, 26], [46, 14], [19, -24], [21, 14], [6, 27]],
            [[1440, 9241], [19, -6], [47, -52]],
            [
                [1506, 9183],
                [37, 39],
                [4, -44],
                [34, 10],
                [10, 16],
                [34, -3],
                [43, -24],
                [65, -21],
                [38, -10],
                [27, 4],
                [37, -29],
                [-39, -29],
                [51, -12],
                [75, 6],
                [23, 11],
                [30, -35],
                [30, 29],
                [-28, 25],
                [18, 19],
                [33, 3],
                [23, 6],
                [22, -14],
                [28, -31],
                [31, 4],
                [49, -26],
                [43, 9],
                [41, -1],
                [-3, 36],
                [24, 10],
                [43, -20],
                [0, -54],
                [18, 46],
                [22, -2],
                [13, 58],
                [-30, 36],
                [-32, 23],
                [2, 64],
                [33, 41],
                [36, -9],
                [28, -25],
                [38, -65],
            ],
            [[2457, 9224], [-25, -28], [52, -12]],
            [
                [2484, 9184],
                [0, -59],
                [37, 45],
                [33, -37],
                [-8, -43],
                [27, -39],
                [29, 42],
                [20, 50],
                [2, 63],
                [39, -4],
                [41, -9],
                [37, -28],
                [2, -29],
                [-21, -31],
                [20, -31],
                [-4, -28],
                [-54, -40],
                [-39, -9],
                [-28, 18],
                [-9, -29],
                [-26, -49],
                [-8, -25],
                [-33, -39],
                [-39, -4],
                [-22, -24],
                [-2, -38],
                [-32, -7],
                [-34, -46],
                [-31, -65],
                [-10, -46],
                [-2, -67],
                [41, -9],
                [12, -54],
                [13, -44],
                [39, 12],
                [52, -25],
                [28, -22],
                [20, -27],
                [34, -16],
                [30, -25],
                [46, -3],
                [30, -5],
                [-5, -50],
                [9, -58],
                [20, -65],
                [41, -54],
                [22, 18],
                [15, 60],
                [-15, 91],
                [-19, 30],
                [44, 27],
                [32, 40],
                [15, 40],
                [-2, 39],
                [-19, 49],
                [-34, 43],
                [33, 60],
                [-12, 53],
                [-9, 90],
                [19, 13],
                [48, -16],
                [28, -5],
                [23, 15],
                [26, -20],
                [34, -33],
                [9, -23],
                [49, -4],
                [-1, -48],
                [10, -73],
                [25, -9],
                [20, -34],
                [40, 32],
                [27, 63],
                [18, 27],
                [22, -51],
                [36, -74],
                [31, -69],
                [-11, -36],
                [36, -32],
                [25, -33],
                [45, -15],
                [18, -18],
                [11, -49],
                [21, -8],
                [11, -22],
                [2, -64],
                [-20, -22],
                [-20, -20],
                [-45, -21],
                [-35, -47],
                [-47, -9],
                [-60, 12],
                [-41, 0],
                [-29, -4],
                [-23, -41],
                [-36, -26],
                [-40, -76],
                [-32, -53],
                [24, 10],
                [44, 75],
                [59, 48],
                [41, 6],
                [25, -28],
                [-27, -39],
                [9, -62],
                [9, -43],
                [36, -29],
                [46, 8],
                [28, 65],
                [2, -42],
                [18, -21],
                [-34, -38],
                [-62, -34],
                [-27, -23],
                [-31, -42],
                [-22, 5],
                [-1, 48],
                [49, 48],
                [-45, -2],
                [-31, -7],
            ],
            [
                [1852, 9128],
                [-15, 28],
                [-38, 15],
                [-25, -6],
                [-34, 45],
                [18, 7],
                [43, 9],
                [39, -2],
                [37, 10],
                [-54, 13],
                [-59, -4],
                [-40, 1],
                [-14, 21],
                [64, 23],
                [-43, -1],
                [-48, 16],
                [23, 43],
                [19, 23],
                [75, 35],
                [28, -11],
                [-14, -27],
                [62, 17],
                [39, -29],
                [31, 29],
                [25, -19],
                [23, -56],
                [14, 24],
                [-20, 59],
                [25, 8],
                [27, -9],
                [31, -23],
                [18, -56],
                [8, -41],
                [47, -29],
                [50, -27],
                [-3, -25],
                [-45, -5],
                [17, -22],
                [-9, -21],
                [-50, 9],
                [-48, 16],
                [-32, -4],
                [-52, -19],
            ],
            [[1972, 9143], [-83, -11], [-37, -4]],
            [[2097, 9410], [-25, -38], [-43, 40], [9, 8], [37, 2], [22, -12]],
            [[2879, 9391], [2, -15], [-29, 1], [-30, 1], [-31, -7], [-8, 3], [-30, 31], [1, 20], [13, 4], [64, -6], [48, -32]],
            [
                [2595, 9395],
                [22, -36],
                [25, 46],
                [71, 24],
                [47, -60],
                [-4, -37],
                [55, 16],
                [26, 23],
                [62, -29],
                [38, -27],
                [4, -26],
                [51, 13],
                [29, -36],
                [67, -23],
                [25, -23],
                [26, -54],
                [-51, -27],
                [65, -38],
                [44, -12],
                [40, -53],
                [44, -4],
                [-9, -40],
                [-48, -67],
                [-35, 24],
                [-43, 56],
                [-36, -7],
                [-4, -33],
                [30, -34],
                [37, -26],
                [12, -16],
                [18, -57],
                [-10, -41],
                [-35, 16],
                [-69, 46],
                [39, -50],
                [29, -35],
                [4, -20],
                [-75, 23],
                [-60, 34],
                [-33, 28],
                [9, 16],
                [-41, 29],
                [-41, 28],
                [1, -16],
                [-80, -10],
                [-24, 20],
                [18, 43],
                [53, 1],
                [57, 7],
                [-10, 21],
                [10, 28],
                [36, 56],
                [-8, 26],
                [-10, 20],
                [-43, 28],
                [-56, 19],
                [18, 15],
                [-30, 36],
                [-24, 3],
                [-22, 19],
                [-15, -17],
                [-50, -7],
                [-101, 13],
                [-59, 17],
                [-45, 8],
                [-23, 21],
                [29, 26],
                [-40, 0],
                [-8, 58],
                [21, 52],
                [28, 23],
                [72, 16],
                [-20, -37],
            ],
            [
                [2212, 9434],
                [33, -12],
                [49, 7],
                [7, -16],
                [-25, -28],
                [42, -25],
                [-5, -52],
                [-46, -22],
                [-27, 5],
                [-19, 22],
                [-69, 44],
                [1, 19],
                [56, -7],
                [-30, 37],
                [33, 28],
            ],
            [[8988, 9398], [-43, -1], [-56, 7], [-5, 3], [26, 23], [35, 5], [39, -22], [4, -15]],
            [[2410, 9372], [-29, -43], [-32, 3], [-17, 50], [0, 29], [15, 24], [27, 16], [58, -2], [53, -14], [-41, -51], [-34, -12]],
            [[1580, 9265], [-15, 25], [-64, 30]],
            [[1501, 9320], [10, 19], [21, 48]],
            [[1532, 9387], [25, 38], [-28, 35], [94, 9], [40, -12], [71, -3], [27, -17], [30, -24], [-35, -15], [-68, -40], [-35, -40]],
            [[1653, 9318], [0, -25], [-73, -28]],
            [[9186, 9506], [-33, -23], [-44, 5], [-52, 23], [7, 18], [52, -8], [70, -15]],
            [[2399, 9500], [-15, -23], [-41, 5], [-33, 15], [15, 25], [40, 16], [24, -20], [10, -18]],
            [[9029, 9533], [-22, -43], [-102, 2], [-47, -14], [-55, 38], [15, 39], [37, 11], [73, -2], [101, -31]],
            [[2263, 9600], [21, -27], [1, -30], [-12, -42], [-46, -6], [-30, 9], [1, 34], [-46, -5], [-2, 45], [30, -2], [42, 19], [39, -3], [2, 8]],
            [
                [1993, 9570],
                [11, -21],
                [25, 10],
                [29, -2],
                [5, -29],
                [-17, -27],
                [-94, -9],
                [-70, -25],
                [-42, -1],
                [-4, 19],
                [58, 25],
                [-126, -7],
                [-38, 10],
                [37, 57],
                [27, 16],
                [78, -20],
                [49, -34],
                [49, -4],
                [-40, 55],
                [25, 21],
                [29, -7],
                [9, -27],
            ],
            [
                [6597, 9254],
                [-16, -5],
                [-91, 8],
                [-7, 25],
                [-51, 16],
                [-4, 31],
                [29, 12],
                [-1, 32],
                [55, 49],
                [-26, 7],
                [67, 50],
                [-8, 26],
                [62, 31],
                [92, 37],
                [92, 11],
                [48, 21],
                [54, 7],
                [19, -22],
                [-18, -18],
                [-99, -29],
                [-85, -27],
                [-86, -55],
                [-41, -56],
                [-44, -56],
                [6, -48],
                [53, -47],
            ],
            [
                [2369, 9621],
                [31, -18],
                [55, 0],
                [24, -19],
                [-7, -21],
                [32, -13],
                [18, -14],
                [37, -3],
                [41, -5],
                [44, 13],
                [57, 5],
                [45, -4],
                [29, -22],
                [7, -24],
                [-18, -15],
                [-41, -12],
                [-36, 7],
                [-79, -9],
                [-57, -1],
                [-45, 7],
                [-74, 18],
                [-10, 32],
                [-3, 29],
                [-28, 25],
                [-57, 7],
                [-33, 18],
                [11, 23],
                [57, -4],
            ],
            [[1772, 9653], [-4, -44], [-22, -20], [-26, -3], [-51, -25], [-45, -8], [-37, 12], [47, 43], [57, 37], [42, 0], [39, 8]],
            [
                [8162, 9520],
                [-31, -17],
                [-73, -32],
                [-20, -18],
                [34, -8],
                [41, -14],
                [25, 11],
                [14, -37],
                [12, 15],
                [45, 9],
                [89, -10],
                [7, -27],
                [116, -8],
                [1, 44],
                [59, -10],
                [45, 0],
                [45, -30],
                [12, -37],
                [-16, -24],
                [35, -46],
                [44, -23],
                [26, 61],
                [45, -26],
                [47, 15],
                [54, -18],
                [20, 17],
                [46, -9],
                [-20, 54],
                [37, 25],
                [250, -38],
                [24, -34],
                [73, -44],
                [112, 11],
                [55, -9],
                [23, -24],
                [-3, -42],
                [34, -17],
                [37, 12],
                [50, 2],
                [52, -12],
                [53, 7],
                [48, -51],
                [34, 18],
                [-22, 37],
                [12, 25],
                [89, -16],
                [58, 4],
                [80, -28],
                [39, -25],
                [0, -229],
                [-1, -1],
                [-35, -25],
                [-36, 4],
                [25, -30],
                [16, -48],
                [13, -15],
                [3, -24],
                [-7, -15],
                [-52, 12],
                [-77, -43],
                [-25, -7],
                [-43, -40],
                [-40, -36],
                [-10, -26],
                [-40, 40],
                [-72, -45],
                [-13, 21],
                [-27, -25],
                [-37, 8],
                [-9, -37],
                [-33, -56],
                [1, -23],
                [32, -13],
                [-4, -84],
                [-26, -2],
                [-12, -49],
                [12, -24],
                [-49, -30],
                [-9, -65],
                [-42, -15],
                [-8, -58],
                [-40, -54],
                [-10, 40],
                [-12, 84],
                [-16, 128],
                [14, 80],
                [23, 34],
                [1, 27],
                [44, 13],
                [49, 72],
                [48, 60],
                [50, 46],
                [22, 81],
                [-33, -5],
                [-17, -47],
                [-71, -64],
                [-22, 71],
                [-72, -19],
                [-70, -97],
                [23, -35],
                [-62, -15],
                [-43, -6],
                [2, 41],
                [-43, 9],
                [-34, -28],
                [-85, 10],
                [-91, -17],
                [-90, -113],
                [-107, -136],
                [44, -7],
                [14, -36],
                [27, -13],
                [17, 29],
                [31, -4],
                [40, -63],
                [1, -49],
                [-22, -58],
                [-2, -68],
                [-13, -92],
                [-42, -84],
                [-9, -39],
                [-38, -68],
                [-37, -66],
                [-18, -34],
                [-37, -34],
                [-18, -1],
                [-17, 28],
                [-37, -42],
                [-5, -19],
            ],
            [
                [8631, 7613],
                [-10, 4],
                [-12, -20],
                [-8, -20],
                [1, -41],
                [-15, -13],
                [-5, -10],
                [-10, -17],
                [-19, -9],
                [-12, -16],
                [-1, -25],
                [-3, -6],
                [11, -9],
                [16, -26],
            ],
            [
                [8564, 7405],
                [24, -68],
                [7, -37],
                [0, -66],
                [-10, -32],
                [-26, -11],
                [-22, -24],
                [-25, -5],
                [-3, 32],
                [5, 43],
                [-12, 60],
                [21, 9],
                [-19, 50],
            ],
            [
                [8504, 7356],
                [-14, 11],
                [-3, -11],
                [-8, -5],
                [-1, 11],
                [-8, 5],
                [-7, 9],
                [7, 26],
                [7, 6],
                [-2, 11],
                [7, 31],
                [-2, 9],
                [-16, 7],
                [-14, 15],
            ],
            [
                [8450, 7481],
                [-38, -17],
                [-21, -26],
                [-30, -16],
                [15, 26],
                [-6, 23],
                [22, 39],
                [-14, 30],
                [-25, -21],
                [-31, -40],
                [-17, -37],
                [-27, -3],
                [-15, -26],
                [15, -39],
                [23, -10],
                [1, -26],
                [22, -16],
                [31, 41],
                [25, -23],
                [17, -1],
                [5, -30],
                [-39, -17],
                [-13, -31],
                [-27, -29],
                [-15, -40],
                [30, -31],
                [11, -57],
                [17, -53],
                [19, -44],
                [0, -43],
                [-18, -16],
                [7, -30],
                [16, -18],
                [-4, -47],
                [-7, -46],
                [-16, -5],
                [-20, -62],
                [-23, -76],
                [-25, -69],
                [-39, -53],
                [-38, -48],
                [-31, -7],
                [-17, -25],
                [-10, 18],
                [-16, -28],
                [-39, -29],
                [-29, -9],
                [-9, -61],
                [-16, -3],
                [-7, 42],
                [6, 22],
                [-37, 18],
                [-13, -9],
            ],
            [
                [8000, 6423],
                [-37, -49],
                [-23, -55],
                [-6, -40],
                [21, -60],
                [26, -76],
                [25, -35],
                [17, -46],
                [13, -107],
                [-4, -101],
                [-23, -38],
                [-32, -37],
                [-23, -48],
                [-34, -54],
                [-10, 37],
                [7, 39],
                [-20, 33],
            ],
            [[7897, 5786], [-23, 8], [-12, 30], [-14, 60]],
            [
                [7848, 5884],
                [-25, 26],
                [-23, -1],
                [4, 45],
                [-25, 0],
                [-2, -63],
                [-15, -84],
                [-9, -51],
                [2, -42],
                [18, -2],
                [11, -52],
                [5, -50],
                [16, -33],
                [17, -7],
                [14, -30],
            ],
            [
                [7836, 5540],
                [6, -5],
                [17, -35],
                [11, -38],
                [2, -39],
                [-3, -26],
                [3, -20],
                [2, -34],
                [10, -16],
                [10, -51],
                [0, -20],
                [-20, -3],
                [-26, 42],
                [-33, 46],
                [-3, 29],
                [-16, 39],
                [-4, 47],
                [-10, 32],
                [3, 42],
                [-6, 24],
            ],
            [[7779, 5554], [-11, 22], [-5, 29], [-15, 32], [-13, 28], [-5, -34], [-5, 32], [3, 36], [8, 55]],
            [
                [7736, 5754],
                [-2, 43],
                [8, 44],
                [-9, 34],
                [2, 63],
                [-11, 29],
                [-9, 69],
                [-5, 73],
                [-12, 48],
                [-19, -29],
                [-31, -41],
                [-16, 5],
                [-17, 13],
                [10, 72],
                [-6, 54],
                [-22, 66],
                [3, 21],
                [-16, 7],
                [-19, 47],
            ],
            [[7565, 6372], [-8, 30], [-2, 30], [-5, 27], [-12, 34], [-25, 2], [2, -24], [-9, -32], [-11, 12], [-5, -10], [-7, 6], [-11, 5]],
            [
                [7472, 6452],
                [-4, -21],
                [-19, 1],
                [-34, -12],
                [1, -44],
                [-14, -34],
                [-40, -38],
                [-32, -68],
                [-20, -36],
                [-28, -38],
                [0, -27],
                [-14, -14],
                [-25, -20],
                [-13, -4],
                [-8, -43],
                [5, -75],
                [2, -48],
                [-12, -55],
                [0, -98],
                [-14, -2],
                [-13, -44],
                [8, -19],
                [-25, -17],
                [-9, -39],
                [-11, -16],
                [-27, 53],
                [-12, 81],
                [-11, 58],
                [-10, 27],
                [-15, 56],
                [-7, 72],
                [-4, 36],
                [-26, 79],
                [-11, 111],
                [-8, 74],
                [0, 70],
                [-6, 54],
                [-40, -35],
                [-20, 7],
                [-36, 70],
                [13, 21],
                [-8, 22],
                [-32, 49],
            ],
            [[6893, 6546], [-21, 15], [-8, 41], [-21, 44], [-52, -11], [-45, -1], [-39, -8]],
            [
                [6707, 6626],
                [-52, 17],
                [-30, 14],
                [-32, 7],
                [-12, 71],
                [-13, 10],
                [-21, -10],
                [-28, -28],
                [-34, 19],
                [-28, 44],
                [-27, 17],
                [-19, 54],
                [-20, 77],
                [-15, -9],
                [-18, 19],
                [-10, -23],
            ],
            [[6348, 6905], [-17, 3]],
            [[6331, 6908], [6, -25], [-2, -13], [9, -44]],
            [[6344, 6826], [11, -50], [13, -13], [5, -20], [19, -24], [2, -24], [-3, -19], [3, -19], [8, -17], [4, -18], [4, -15]],
            [[6410, 6607], [-2, 42], [8, 31], [7, 6], [9, -18], [0, -34], [-6, -34]],
            [[6426, 6600], [6, -22]],
            [[6432, 6578], [5, 3], [1, -16], [21, 9], [23, -1], [17, -2], [19, 39], [21, 37], [17, 35]],
            [[6556, 6682], [8, 20], [4, -5], [-3, -24], [-3, -10]],
            [[6562, 6663], [3, -46]],
            [
                [6565, 6617],
                [13, -39],
                [15, -21],
                [21, -8],
                [16, -10],
                [13, -33],
                [7, -19],
                [10, -7],
                [0, -13],
                [-10, -35],
                [-4, -16],
                [-12, -18],
                [-10, -40],
                [-13, 3],
                [-6, -13],
                [-4, -30],
                [3, -38],
                [-3, -7],
                [-12, 0],
                [-18, -21],
                [-2, -29],
                [-7, -12],
                [-17, 1],
                [-11, -15],
                [0, -23],
                [-13, -16],
                [-16, 5],
                [-18, -19],
                [-13, -3],
            ],
            [
                [6474, 6141],
                [-20, -16],
                [-5, -25],
                [-1, -20],
                [-28, -24],
                [-44, -27],
                [-25, -41],
                [-12, -3],
                [-8, 4],
                [-17, -24],
                [-17, -11],
                [-24, -3],
                [-7, -4],
                [-6, -15],
                [-7, -4],
                [-4, -15],
                [-14, 2],
                [-9, -8],
                [-19, 3],
                [-7, 33],
                [0, 32],
                [-4, 17],
                [-6, 42],
                [-8, 24],
                [6, 3],
                [-3, 26],
                [3, 11],
                [-1, 25],
            ],
            [
                [6187, 6123],
                [-3, 25],
                [-9, 17],
                [-2, 23],
                [-14, 21],
                [-15, 48],
                [-8, 47],
                [-19, 40],
                [-13, 9],
                [-18, 55],
                [-3, 40],
                [1, 34],
                [-16, 64],
                [-13, 22],
                [-15, 12],
                [-9, 33],
                [1, 13],
                [-7, 30],
                [-8, 13],
                [-11, 43],
                [-17, 46],
                [-14, 40],
                [-14, 0],
                [4, 31],
                [1, 20],
                [4, 23],
            ],
            [[5970, 6872], [-1, 9]],
            [
                [5969, 6881],
                [-8, -23],
                [-6, -44],
                [-7, -30],
                [-7, -10],
                [-9, 19],
                [-13, 25],
                [-19, 83],
                [-3, -5],
                [11, -61],
                [17, -58],
                [21, -90],
                [11, -31],
                [8, -33],
                [25, -63],
                [-5, -10],
                [1, -38],
                [32, -51],
                [5, -12],
            ],
            [[6023, 6449], [9, -57], [-6, -10], [4, -59], [10, -69], [11, -14], [15, -22]],
            [[6066, 6218], [16, -66], [8, -53], [15, -28], [38, -55], [15, -32], [15, -34], [9, -19], [14, -18]],
            [[6196, 5913], [6, -18], [-1, -23], [-16, -14], [12, -16]],
            [[6197, 5842], [9, -11], [6, -23], [12, -25], [14, 0], [26, 15], [31, 7], [24, 18], [14, 3], [10, 11], [16, 2]],
            [
                [6359, 5839],
                [8, 1],
                [13, 9],
                [15, 6],
                [13, 19],
                [11, 0],
                [0, -16],
                [-2, -33],
                [0, -30],
                [-6, -21],
                [-8, -62],
                [-13, -65],
                [-17, -73],
                [-24, -85],
                [-24, -64],
                [-33, -79],
                [-27, -46],
                [-42, -57],
                [-26, -44],
                [-30, -70],
                [-7, -30],
                [-6, -14],
            ],
            [[6154, 5085], [-19, -23], [-7, -24], [-11, -4], [-4, -41], [-9, -23], [-5, -38], [-11, -19]],
            [[6088, 4913], [-13, -71], [2, -33], [17, -21], [1, -15], [-7, -35], [1, -17], [-2, -28], [10, -36], [12, -57], [10, -12]],
            [
                [6119, 4588],
                [4, -26],
                [-1, -57],
                [4, -51],
                [1, -90],
                [5, -28],
                [-9, -41],
                [-11, -40],
                [-17, -36],
                [-26, -22],
                [-31, -28],
                [-31, -62],
                [-11, -10],
                [-19, -41],
                [-12, -13],
                [-2, -41],
                [13, -44],
                [5, -34],
                [1, -17],
                [5, 3],
                [-1, -57],
                [-5, -26],
                [7, -10],
                [-4, -24],
                [-12, -21],
                [-23, -19],
                [-33, -31],
                [-12, -21],
                [2, -25],
                [7, -4],
                [-2, -30],
            ],
            [
                [5911, 3642],
                [-7, -42],
                [-3, -48],
                [-8, -26],
                [-19, -29],
                [-5, -8],
                [-12, -29],
                [-7, -30],
                [-16, -41],
                [-32, -60],
                [-19, -34],
                [-21, -26],
                [-29, -23],
                [-14, -3],
                [-4, -16],
                [-17, 9],
                [-14, -11],
                [-30, 11],
                [-17, -7],
                [-11, 3],
                [-29, -23],
                [-23, -9],
                [-18, -22],
                [-12, -1],
                [-12, 21],
                [-9, 1],
                [-12, 25],
                [-2, -8],
                [-3, 16],
                [0, 34],
                [-9, 38],
                [9, 11],
                [-1, 44],
                [-18, 54],
                [-14, 48],
                [0, 1],
                [-20, 74],
            ],
            [
                [5453, 3536],
                [-21, 44],
                [-11, 42],
                [-6, 56],
                [-7, 42],
                [-9, 88],
                [0, 69],
                [-4, 32],
                [-11, 23],
                [-14, 48],
                [-15, 69],
                [-6, 36],
                [-22, 56],
                [-2, 45],
            ],
            [
                [5325, 4186],
                [-3, 36],
                [4, 51],
                [10, 52],
                [1, 25],
                [9, 52],
                [7, 23],
                [16, 38],
                [9, 26],
                [3, 42],
                [-2, 33],
                [-8, 21],
                [-8, 35],
                [-6, 34],
                [1, 12],
                [9, 23],
                [-9, 56],
                [-5, 38],
                [-14, 37],
                [2, 11],
            ],
            [[5341, 4831], [-4, 18]],
            [[5337, 4849], [-7, 43]],
            [[5330, 4892], [-23, 61]],
            [[5307, 4953], [-28, 58], [-19, 47], [-17, 60], [1, 19], [6, 19], [7, 41], [6, 43]],
            [[5263, 5240], [-6, 9], [10, 64]],
            [[5267, 5313], [4, 46], [-11, 38], [-12, 10], [-6, 26], [-7, 8], [0, 16]],
            [[5235, 5457], [-29, -21], [-10, 3], [-11, -13], [-22, 1], [-15, 36], [-9, 42], [-20, 38], [-21, -1], [-24, 0]],
            [[5074, 5542], [-23, -6]],
            [[5051, 5536], [-23, -13]],
            [[5028, 5523], [-43, -33], [-16, -20], [-25, -17], [-24, 17]],
            [[4920, 5470], [-13, -1], [-19, 11], [-18, 0], [-33, -10], [-19, -17], [-28, -21], [-5, 1]],
            [[4785, 5433], [-7, 0], [-29, 27], [-25, 44], [-24, 32], [-19, 37]],
            [[4681, 5573], [-7, 4], [-20, 23], [-15, 31], [-5, 21], [-3, 43]],
            [[4631, 5695], [-12, 34], [-11, 22], [-7, 8], [-7, 11], [-3, 26], [-4, 12], [-8, 10]],
            [[4579, 5818], [-15, 24], [-12, 4], [-6, 16], [0, 9], [-8, 12], [-2, 12]],
            [[4536, 5895], [-5, 44]],
            [[4531, 5939], [4, 26]],
            [[4535, 5965], [-12, 45], [-14, 20], [13, 11], [13, 40], [7, 30]],
            [[4542, 6111], [-3, 31], [8, 28], [4, 55], [-4, 57], [-3, 28], [3, 29], [-7, 27], [-15, 25]],
            [[4525, 6391], [1, 25]],
            [
                [4526, 6416],
                [2, 26],
                [10, 16],
                [9, 30],
                [-1, 19],
                [9, 41],
                [16, 37],
                [9, 9],
                [7, 34],
                [1, 30],
                [10, 36],
                [18, 21],
                [18, 59],
                [15, 22],
                [25, 7],
                [22, 39],
                [14, 16],
                [23, 48],
                [-7, 71],
                [11, 50],
                [4, 30],
                [18, 39],
                [27, 26],
                [21, 24],
                [19, 60],
                [8, 35],
                [21, 0],
                [16, -25],
                [27, 4],
                [29, -12],
                [12, -1],
            ],
            [[4939, 7207], [26, 32], [30, 10], [18, 23], [27, 18], [47, 10], [46, 5], [14, -9], [26, 23], [30, 0], [11, -13], [19, 3]],
            [
                [5233, 7309],
                [30, 24],
                [20, -7],
                [-1, -29],
                [23, 21],
                [2, -11],
                [-14, -28],
                [0, -27],
                [10, -14],
                [-4, -50],
                [-18, -29],
                [5, -32],
                [15, -1],
                [7, -27],
                [10, -9],
            ],
            [
                [5318, 7090],
                [33, -20],
                [11, 5],
                [24, -9],
                [36, -26],
                [13, -51],
                [25, -11],
                [40, -25],
                [29, -28],
                [14, 15],
                [13, 26],
                [-6, 44],
                [8, 28],
                [20, 27],
                [19, 8],
                [38, -12],
                [9, -25],
                [11, -1],
                [9, -9],
                [27, -7],
                [7, -19],
            ],
            [
                [5698, 7000],
                [37, 1],
                [27, -15],
                [27, -17],
                [13, -9],
                [21, 18],
                [12, 16],
                [24, 5],
                [20, -7],
                [8, -29],
                [6, 19],
                [22, -14],
                [22, -3],
                [14, 15],
            ],
            [[5951, 6980], [8, 19], [-2, 3], [7, 27], [6, 43], [4, 15], [1, 0]],
            [[5975, 7087], [10, 47], [13, 41], [1, 2]],
            [[5999, 7177], [-3, 44], [7, 24]],
            [
                [6003, 7245],
                [-10, 26],
                [10, 21],
                [-17, -4],
                [-23, 13],
                [-19, -33],
                [-42, -7],
                [-23, 31],
                [-29, 2],
                [-7, -24],
                [-19, -7],
                [-27, 31],
                [-30, -1],
                [-17, 57],
                [-20, 32],
                [14, 45],
                [-18, 27],
                [31, 55],
                [42, 3],
                [12, 44],
                [53, -8],
                [33, 37],
                [33, 17],
                [46, 1],
                [48, -41],
                [40, -22],
                [32, 9],
                [24, -5],
                [33, 30],
            ],
            [[6153, 7574], [4, 24], [-7, 40], [-16, 21], [-15, 6], [-10, 18]],
            [[6109, 7683], [-36, 49], [-31, 21], [-24, 34], [20, 9], [23, 49], [-16, 22], [41, 24], [0, 13], [-25, -10]],
            [
                [6061, 7894],
                [-23, -4],
                [-18, -19],
                [-26, -3],
                [-24, -21],
                [2, -36],
                [13, -14],
                [29, 3],
                [-6, -20],
                [-30, -10],
                [-38, -34],
                [-15, 12],
                [6, 27],
                [-31, 17],
                [5, 11],
                [27, 19],
                [-8, 13],
                [-43, 15],
                [-2, 21],
                [-26, -7],
                [-10, -31],
                [-22, -43],
            ],
            [[5821, 7790], [1, -15], [-14, -12], [-8, 5], [-8, -69]],
            [[5792, 7699], [-14, -24], [-10, -41], [9, -33]],
            [[5777, 7601], [3, -22], [24, -19], [-5, -14], [-33, -3], [-12, -18], [-23, -31], [-9, 27], [1, 12]],
            [
                [5723, 7533],
                [-17, 1],
                [-15, 6],
                [-33, -15],
                [19, -33],
                [-14, -9],
                [-16, 0],
                [-14, 30],
                [-6, -13],
                [7, -34],
                [14, -27],
                [-11, -13],
                [16, -27],
                [13, -16],
                [1, -33],
                [-26, 16],
                [8, -30],
                [-17, -6],
                [10, -51],
                [-18, 0],
                [-23, 25],
                [-11, 46],
                [-4, 38],
                [-11, 26],
                [-14, 33],
                [-2, 17],
            ],
            [[5559, 7464], [-5, 4], [-1, 12], [-15, 20], [-2, 27], [2, 39], [4, 18], [-5, 9]],
            [[5537, 7593], [-6, 5], [-8, 19], [-12, 11]],
            [[5511, 7628], [-26, 21], [-16, 21], [-25, 17], [-24, 43], [6, 4], [-13, 24], [0, 20], [-18, 9], [-8, -25], [-9, 19], [1, 20], [1, 1]],
            [[5380, 7802], [6, 5]],
            [
                [5386, 7807],
                [-22, 9],
                [-23, -21],
                [2, -28],
                [-3, -17],
                [9, -29],
                [26, -29],
                [14, -48],
                [31, -46],
                [21, 0],
                [7, -13],
                [-8, -11],
                [25, -21],
                [21, -17],
                [23, -30],
                [3, -11],
                [-5, -21],
                [-15, 27],
                [-24, 10],
                [-12, -37],
                [20, -22],
                [-3, -30],
                [-12, -3],
                [-15, -50],
                [-11, -4],
                [0, 17],
                [5, 31],
                [6, 13],
                [-10, 33],
                [-9, 29],
                [-11, 7],
                [-9, 25],
                [-17, 11],
                [-12, 23],
                [-21, 3],
                [-22, 26],
                [-25, 38],
                [-19, 33],
                [-9, 57],
                [-14, 7],
                [-22, 19],
                [-13, -8],
                [-16, -27],
                [-11, -4],
            ],
            [[5206, 7698], [-26, -33], [-54, 16], [-41, -19], [-3, -34]],
            [
                [5082, 7628],
                [1, -34],
                [-26, -38],
                [-35, -12],
                [-3, -20],
                [-17, -32],
                [-11, -46],
                [11, -33],
                [-16, -26],
                [-6, -37],
                [-21, -12],
                [-20, -44],
                [-35, -1],
                [-26, 1],
                [-18, -20],
                [-10, -22],
                [-14, 5],
                [-10, 19],
                [-8, 33],
                [-26, 9],
            ],
            [
                [4792, 7318],
                [-11, -15],
                [-15, 8],
                [-14, -6],
                [4, 45],
                [-3, 35],
                [-12, 6],
                [-7, 22],
                [3, 37],
                [11, 21],
                [2, 23],
                [5, 35],
                [0, 24],
                [-6, 21],
                [-1, 20],
            ],
            [[4748, 7594], [1, 41], [-11, 25], [39, 41], [34, -10], [38, 0], [29, -10], [23, 3], [45, -2]],
            [
                [4946, 7682],
                [15, 35],
                [5, 115],
                [-29, 60],
                [-20, 29],
                [-43, 22],
                [-3, 42],
                [36, 13],
                [47, -15],
                [-9, 65],
                [27, -24],
                [64, 44],
                [9, 48],
                [24, 11],
            ],
            [[5069, 8127], [22, 12]],
            [[5091, 8139], [14, 15], [25, 85], [38, 24], [23, -1]],
            [[5191, 8262], [5, 12], [23, 3], [6, -13], [18, 29], [-6, 21], [-1, 33]],
            [
                [5236, 8347],
                [-11, 32],
                [-1, 59],
                [4, 15],
                [8, 18],
                [25, 3],
                [10, 16],
                [22, 16],
                [-1, -29],
                [-8, -19],
                [3, -16],
                [15, -9],
                [-7, -22],
                [-8, 7],
                [-20, -42],
                [8, -28],
            ],
            [[5275, 8348], [0, -22], [28, -14], [0, -20], [28, 11], [16, 16], [31, -23], [13, -19]],
            [[5391, 8277], [19, 17], [43, 27], [35, 19], [28, -9], [2, -14], [27, -1]],
            [[5545, 8316], [6, 25], [39, 19]],
            [[5590, 8360], [-6, 48]],
            [[5584, 8408], [1, 44], [13, 36], [27, 20], [22, -43], [22, 1], [5, 44]],
            [[5674, 8510], [4, 34], [-11, -7], [-17, 20], [-3, 33], [35, 16], [35, 9], [31, -10], [28, 2]],
            [[5776, 8607], [32, 32], [-29, 27]],
            [
                [5779, 8666],
                [-51, -5],
                [-49, -21],
                [-45, -12],
                [-16, 32],
                [-27, 18],
                [6, 57],
                [-13, 52],
                [13, 34],
                [25, 36],
                [64, 62],
                [18, 12],
                [-2, 25],
                [-39, 27],
            ],
            [
                [5663, 8983],
                [-48, -16],
                [-27, -41],
                [5, -35],
                [-45, -46],
                [-53, -50],
                [-21, -81],
                [20, -40],
                [27, -32],
                [-26, -65],
                [-29, -14],
                [-10, -96],
                [-16, -54],
                [-34, 5],
                [-15, -45],
                [-33, -3],
                [-8, 54],
                [-24, 66],
                [-21, 81],
            ],
            [
                [5305, 8571],
                [-18, 35],
                [-55, -66],
                [-37, -14],
                [-39, 30],
                [-10, 62],
                [-8, 132],
                [25, 37],
                [74, 49],
                [54, 59],
                [51, 81],
                [67, 111],
                [47, 43],
                [76, 72],
                [61, 25],
                [45, -3],
                [43, 48],
                [50, -2],
                [50, 11],
                [87, -42],
                [-36, -16],
                [31, -36],
            ],
            [
                [5863, 9187],
                [28, 20],
                [46, -34],
                [76, -14],
                [105, -65],
                [21, -28],
                [2, -38],
                [-31, -30],
                [-45, -16],
                [-124, 44],
                [-20, -7],
                [45, -42],
                [4, -86],
                [35, -17],
                [22, -15],
                [4, 28],
            ],
            [[6031, 8887], [-18, 25], [19, 21]],
            [
                [6032, 8933],
                [67, -36],
                [23, 14],
                [-19, 42],
                [65, 57],
                [26, -3],
                [26, -21],
                [16, 40],
                [-23, 34],
                [13, 35],
                [-20, 35],
                [78, -18],
                [15, -32],
                [-35, -7],
                [0, -32],
                [22, -20],
                [43, 12],
                [7, 37],
                [58, 28],
                [97, 49],
                [21, -3],
                [-27, -35],
                [34, -6],
                [20, 20],
                [52, 1],
                [41, 24],
                [32, -34],
                [31, 38],
                [-29, 33],
                [15, 19],
                [82, -17],
                [38, -18],
                [101, -66],
                [18, 30],
            ],
            [[6920, 9133], [-28, 30], [-1, 13]],
            [
                [6891, 9176],
                [-33, 5],
                [9, 28],
                [-15, 45],
                [-1, 18],
                [52, 52],
                [18, 53],
                [21, 11],
                [73, -15],
                [6, -32],
                [-26, -47],
                [17, -18],
                [9, -41],
                [-7, -79],
                [31, -35],
                [-12, -38],
                [-54, -82],
                [32, -9],
                [11, 21],
                [30, 15],
                [8, 28],
                [24, 28],
                [-17, 33],
                [13, 38],
                [-30, 4],
                [-7, 32],
                [22, 58],
                [-36, 47],
                [50, 39],
                [-6, 41],
                [14, 1],
                [14, -32],
                [-11, -55],
                [30, -11],
                [-13, 42],
                [47, 22],
                [57, 3],
                [52, -32],
                [-25, 48],
                [-3, 61],
                [48, 11],
                [67, -2],
                [60, 7],
                [-22, 31],
                [32, 37],
                [32, 2],
                [54, 29],
                [73, 7],
                [10, 16],
                [73, 5],
                [22, -13],
                [63, 31],
                [51, -1],
                [7, 25],
                [27, 24],
                [65, 24],
                [48, -19],
                [-38, -14],
                [63, -9],
                [8, -28],
                [25, 14],
                [81, -1],
                [63, -28],
            ],
            [[8147, 9571], [22, -21], [-7, -30]],
            [[6357, 7389], [9, -43], [26, -12], [19, -29], [40, -10], [43, 16], [3, 13]],
            [
                [6497, 7324],
                [-5, 41],
                [4, 60],
                [-22, 19],
                [7, 40],
                [-18, 3],
                [6, 49],
                [26, -14],
                [24, 18],
                [-20, 35],
                [-8, 33],
                [-22, -15],
                [-3, -42],
                [-9, 37],
            ],
            [
                [6457, 7588],
                [-1, 14],
                [7, 24],
                [-6, 20],
                [-32, 20],
                [-12, 51],
                [-16, 15],
                [-1, 19],
                [27, -6],
                [1, 42],
                [24, 10],
                [24, -9],
                [5, 56],
                [-5, 36],
                [-28, -3],
                [-23, 14],
                [-32, -25],
                [-26, -12],
            ],
            [[6363, 7854], [-13, -34], [-27, -10], [-27, -59], [25, -55], [-3, -39], [30, -68]],
            [[6348, 7589], [15, -30], [14, -41], [13, -2], [9, -16], [-23, -5], [-5, -44], [-5, -20], [-10, -14], [1, -28]],
            [[2393, 9646], [-13, -2], [-52, 4], [-8, 16], [56, -1], [20, -11], [-3, -6]],
            [[1939, 9656], [-52, -17], [-41, 19], [22, 18], [41, 6], [39, -9], [-9, -17]],
            [[5686, 9665], [-62, -24], [-49, 14], [19, 15], [-17, 18], [58, 12], [11, -22], [40, -13]],
            [[1953, 9708], [-34, -11], [-46, 0], [1, 8], [28, 17], [15, -2], [36, -12]],
            [[2337, 9677], [-41, -12], [-22, 13], [-12, 22], [-3, 24], [36, -3], [17, -3], [33, -20], [-8, -21]],
            [[2220, 9692], [11, -24], [-46, 7], [-45, 18], [-62, 2], [27, 17], [-34, 14], [-2, 22], [54, -7], [76, -21], [21, -28]],
            [[7917, 9691], [-156, -22], [51, 75], [22, 7], [21, -4], [71, -32], [-9, -24]],
            [
                [5506, 9771],
                [91, -43],
                [-70, -22],
                [-15, -43],
                [-24, -10],
                [-14, -48],
                [-33, -2],
                [-60, 35],
                [25, 20],
                [-41, 17],
                [-54, 48],
                [-22, 46],
                [76, 20],
                [15, -20],
                [40, 1],
                [10, 20],
                [41, 2],
                [35, -21],
            ],
            [[5706, 9812], [54, -20], [-41, -31], [-80, -7], [-82, 10], [-5, 16], [-40, 1], [-31, 26], [86, 16], [40, -14], [29, 17], [70, -14]],
            [
                [6419, 9820],
                [-37, -7],
                [-25, -5],
                [-4, -9],
                [-32, -10],
                [-30, 14],
                [15, 18],
                [-61, 2],
                [54, 10],
                [42, 1],
                [6, -16],
                [16, 14],
                [26, 10],
                [41, -13],
                [-11, -9],
            ],
            [[7775, 9724], [-61, -7], [-77, 17], [-46, 22], [-22, 41], [-37, 11], [72, 40], [60, 13], [54, -29], [64, -56], [-7, -52]],
            [
                [2582, 9769],
                [34, -19],
                [-39, -17],
                [-51, -43],
                [-49, -4],
                [-58, 7],
                [-29, 24],
                [0, 21],
                [22, 15],
                [-51, -1],
                [-31, 20],
                [-17, 26],
                [19, 25],
                [19, 18],
                [29, 4],
                [-12, 13],
                [64, 3],
                [36, -31],
                [47, -12],
                [45, -11],
                [22, -38],
            ],
            [
                [3096, 9967],
                [75, -4],
                [59, -7],
                [51, -16],
                [-1, -15],
                [-68, -25],
                [-67, -12],
                [-25, -13],
                [60, 0],
                [-65, -35],
                [-46, -16],
                [-47, -47],
                [-57, -9],
                [-18, -12],
                [-84, -6],
                [38, -8],
                [-19, -10],
                [23, -28],
                [-26, -20],
                [-43, -16],
                [-14, -23],
                [-38, -17],
                [4, -13],
                [47, 2],
                [1, -14],
                [-75, -34],
                [-72, 15],
                [-82, -9],
                [-41, 7],
                [-53, 3],
                [-3, 28],
                [51, 13],
                [-13, 42],
                [17, 4],
                [74, -25],
                [-38, 37],
                [-45, 11],
                [22, 22],
                [50, 14],
                [7, 20],
                [-39, 22],
                [-12, 30],
                [76, -2],
                [22, -7],
                [44, 21],
                [-63, 7],
                [-97, -4],
                [-49, 20],
                [-23, 23],
                [-33, 17],
                [-6, 20],
                [41, 11],
                [33, 2],
                [54, 9],
                [41, 21],
                [35, -3],
                [30, -16],
                [21, 31],
                [36, 10],
                [50, 6],
                [85, 2],
                [15, -6],
                [80, 10],
                [60, -4],
                [60, -4],
            ],
            [
                [4246, 9991],
                [174, -45],
                [-51, -23],
                [-107, -2],
                [-149, -6],
                [14, -10],
                [98, 6],
                [84, -19],
                [54, 17],
                [23, -21],
                [-31, -33],
                [71, 21],
                [135, 23],
                [83, -11],
                [16, -25],
                [-113, -41],
                [-16, -13],
                [-89, -10],
                [65, -3],
                [-33, -42],
                [-22, -37],
                [1, -64],
                [33, -38],
                [-43, -2],
                [-46, -19],
                [51, -30],
                [7, -49],
                [-30, -5],
                [36, -50],
                [-62, -4],
                [32, -23],
                [-9, -21],
                [-39, -9],
                [-39, 0],
                [35, -39],
                [1, -25],
                [-55, 23],
                [-15, -15],
                [38, -14],
                [36, -36],
                [11, -46],
                [-50, -11],
                [-21, 22],
                [-34, 33],
                [9, -39],
                [-32, -30],
                [73, -3],
                [38, -3],
                [-74, -50],
                [-76, -45],
                [-81, -20],
                [-31, 0],
                [-28, -23],
                [-39, -60],
                [-60, -41],
                [-19, -2],
                [-37, -14],
                [-40, -14],
                [-24, -35],
                [0, -41],
                [-14, -38],
                [-45, -46],
                [11, -45],
                [-13, -47],
                [-14, -56],
                [-39, -4],
                [-41, 47],
                [-56, 0],
                [-26, 32],
                [-19, 56],
                [-48, 72],
                [-14, 37],
                [-4, 52],
                [-38, 53],
                [10, 43],
                [-19, 20],
                [27, 67],
                [42, 22],
                [11, 24],
                [6, 45],
                [-32, -21],
                [-15, -8],
                [-25, -8],
                [-34, 18],
                [-2, 39],
                [11, 31],
                [26, 1],
                [57, -15],
                [-48, 36],
                [-25, 20],
                [-28, -8],
                [-23, 14],
                [31, 54],
                [-17, 21],
                [-22, 40],
                [-33, 61],
                [-36, 22],
                [1, 24],
                [-75, 34],
                [-59, 4],
                [-74, -2],
                [-68, -4],
                [-32, 18],
                [-48, 36],
                [73, 18],
                [56, 4],
                [-119, 14],
                [-63, 24],
                [4, 22],
                [105, 28],
                [102, 28],
                [11, 21],
                [-75, 20],
                [24, 23],
                [96, 41],
                [40, 6],
                [-11, 26],
                [66, 15],
                [85, 9],
                [85, 0],
                [31, -18],
                [73, 32],
                [67, -21],
                [39, -5],
                [57, -19],
                [-66, 31],
                [4, 25],
                [93, 34],
                [98, -2],
                [35, 21],
                [98, 6],
                [222, -8],
            ],
            [[6847, 7333], [15, 0], [21, -12]],
            [
                [6883, 7321],
                [8, -7],
                [21, 18],
                [9, -11],
                [9, 27],
                [16, -2],
                [5, 9],
                [3, 23],
                [12, 20],
                [15, -13],
                [-3, -18],
                [8, -2],
                [-3, -49],
                [11, -19],
                [10, 13],
                [12, 5],
                [18, 26],
                [19, -4],
                [29, 0],
            ],
            [[7082, 7337], [5, -17]],
            [
                [7087, 7320],
                [-17, -6],
                [-14, -11],
                [-32, -7],
                [-29, -12],
                [-17, -25],
                [7, -24],
                [3, -29],
                [-14, -24],
                [1, -22],
                [-7, -21],
                [-27, 2],
                [11, -38],
                [-17, -15],
                [-12, -34],
                [1, -35],
                [-11, -16],
                [-10, 5],
                [-21, -7],
                [-3, -17],
                [-21, 1],
                [-15, -33],
                [-1, -49],
                [-36, -24],
                [-20, 5],
                [-5, -13],
                [-17, 8],
                [-28, -9],
                [-46, 30],
            ],
            [[6690, 6900], [25, 52], [-2, 37], [-21, 10], [-3, 36], [-9, 46], [12, 32], [-12, 8], [8, 42], [11, 72]],
            [[6699, 7235], [28, -22], [21, 8], [6, 26], [22, 8], [16, 18], [5, 46], [24, 11], [4, 21], [13, -16], [9, -2]],
            [[5663, 4553], [3, -18], [-3, -28], [5, -27], [-4, -22], [2, -19], [-58, 0], [-1, -183], [19, -47], [18, -36]],
            [[5644, 4173], [-51, -24], [-67, 9], [-20, 27], [-112, -2], [-5, -4], [-16, 26], [-18, 1], [-17, -9], [-13, -11]],
            [
                [5341, 4831],
                [12, 7],
                [8, -1],
                [10, 7],
                [81, 0],
                [7, -43],
                [8, -35],
                [7, -19],
                [10, -30],
                [19, 5],
                [9, 8],
                [15, -8],
                [4, 14],
                [7, 34],
                [18, 2],
                [1, 10],
                [14, 0],
                [-2, -20],
                [33, 0],
                [1, -36],
                [6, -22],
                [-4, -35],
                [2, -35],
                [9, -22],
                [-2, -68],
                [7, 5],
                [12, -1],
                [18, 8],
                [12, -3],
            ],
            [[5330, 4892], [11, 25], [9, 9], [10, -19]],
            [[5360, 4907], [-10, -12], [-5, -15], [-1, -25], [-7, -6]],
            [[5583, 7534], [-1, -15], [-9, -9], [-1, -18], [-13, -28]],
            [[5537, 7593], [-2, 19], [12, 28], [2, -11], [7, 5]],
            [[5556, 7634], [6, -15], [7, -6], [2, -21]],
            [[5571, 7592], [-4, -19], [4, -25], [12, -14]],
            [[6556, 6682], [6, -19]],
            [[6565, 6617], [-14, 0], [-2, -38], [5, -8], [-13, -11], [0, -23], [-8, -24], [-1, -23]],
            [[6532, 6490], [-5, -13], [-84, 29], [-10, 59], [-1, 13]],
            [[3139, 2021], [-17, 1], [-29, 0], [0, 129]],
            [[3258, 3901], [51, -94], [23, -9], [34, -42], [28, -23], [4, -25], [-27, -88], [28, -16], [31, -8], [22, 9], [25, 44], [5, 51]],
            [[3482, 3700], [14, 11], [13, -33], [0, -46], [-23, -32], [-19, -24], [-31, -55], [-38, -79]],
            [[3398, 3442], [-6, -46], [-8, -59], [0, -58], [-6, -12], [-2, -38]],
            [
                [3094, 2170],
                [-25, 9],
                [-67, 8],
                [-12, 34],
                [1, 43],
                [-19, -4],
                [-10, 21],
                [-2, 61],
                [21, 25],
                [9, 37],
                [-3, 29],
                [14, 49],
                [11, 76],
                [-3, 34],
                [12, 11],
                [-3, 22],
                [-13, 11],
                [9, 25],
                [-12, 21],
                [-7, 67],
                [11, 12],
                [-4, 70],
                [6, 59],
                [7, 51],
                [17, 21],
                [-8, 56],
                [0, 53],
                [21, 38],
                [-1, 48],
                [16, 56],
                [0, 53],
                [-7, 10],
                [-13, 100],
                [17, 59],
                [-3, 56],
                [10, 52],
                [18, 54],
                [20, 36],
                [-8, 23],
                [6, 18],
                [-1, 96],
                [30, 29],
                [9, 59],
                [-3, 15],
            ],
            [[3135, 3873], [23, 52], [37, -14], [16, -42], [11, 47], [31, -3], [5, -12]],
            [[6291, 7414], [-10, -1]],
            [[6281, 7413], [-12, 33], [0, 9], [-12, 0], [-8, 15], [-6, -1]],
            [[6243, 7469], [-11, 17], [-20, 14], [2, 28], [-4, 20]],
            [[6210, 7548], [38, 9]],
            [[6248, 7557], [6, -15], [10, -10], [-5, -14], [15, -20], [-8, -18], [12, -16], [12, -10], [1, -40]],
            [
                [3371, 1488],
                [-12, -13],
                [-21, 9],
                [-22, -5],
                [-20, -14],
                [-20, -14],
                [-13, -17],
                [-4, -23],
                [2, -21],
                [13, -19],
                [-19, -14],
                [-27, -5],
                [-15, -19],
                [-16, -18],
                [-17, -25],
                [-5, -21],
                [10, -24],
                [15, -18],
                [22, -13],
                [22, -18],
                [11, -23],
                [6, -21],
                [8, -23],
                [13, -19],
                [8, -22],
                [4, -53],
                [8, -21],
                [3, -23],
                [8, -22],
                [-3, -31],
                [-16, -23],
                [-16, -20],
                [-37, -7],
                [-13, -21],
                [-16, -19],
                [-42, -21],
                [-37, -9],
                [-35, -13],
                [-38, -12],
                [-22, -24],
                [-45, -2],
                [-49, 2],
                [-44, -4],
                [-46, 0],
                [8, -23],
                [43, -10],
                [31, -16],
                [17, -20],
                [-31, -18],
                [-48, 5],
                [-39, -14],
                [-2, -24],
                [-1, -23],
                [33, -19],
                [6, -21],
                [35, -22],
                [59, -9],
                [50, -15],
                [39, -18],
                [51, -19],
                [69, -9],
                [68, -15],
                [47, -17],
                [52, -19],
                [27, -28],
                [14, -21],
                [34, 20],
                [45, 17],
                [49, 18],
                [57, 15],
                [50, 16],
                [69, 1],
                [68, -8],
                [56, -14],
                [18, 25],
                [39, 17],
                [70, 1],
                [55, 13],
                [52, 12],
                [58, 8],
                [61, 10],
                [43, 15],
                [-20, 20],
                [-12, 20],
                [0, 22],
                [-53, -2],
                [-57, -9],
                [-55, 0],
                [-8, 21],
                [4, 43],
                [13, 12],
                [40, 14],
                [46, 13],
                [34, 17],
                [34, 17],
                [25, 23],
                [38, 10],
                [37, 8],
                [19, 4],
                [43, 3],
                [41, 8],
                [34, 11],
                [34, 13],
                [31, 14],
                [38, 18],
                [25, 19],
                [26, 17],
                [8, 23],
                [-29, 13],
                [9, 24],
                [19, 18],
                [29, 11],
                [30, 14],
                [29, 18],
                [21, 22],
                [14, 27],
                [20, 16],
                [33, -3],
                [14, -19],
                [33, -3],
                [1, 22],
                [14, 22],
                [30, -5],
                [7, -22],
                [33, -3],
                [36, 10],
                [35, 7],
                [32, -3],
                [12, -24],
                [30, 19],
                [28, 10],
                [32, 8],
                [31, 8],
                [28, 14],
                [31, 9],
                [24, 12],
                [17, 20],
                [21, -14],
                [29, 8],
                [20, -28],
                [15, -20],
                [32, 11],
                [13, 23],
                [28, 16],
                [36, -4],
                [11, -21],
                [23, 21],
                [30, 7],
                [33, 2],
                [29, -1],
                [31, -6],
                [30, -4],
                [13, -19],
                [18, -17],
                [30, 10],
                [33, 2],
                [32, 0],
                [31, 2],
                [27, 7],
                [30, 7],
                [24, 16],
                [26, 10],
                [29, 6],
                [21, 16],
                [15, 31],
                [16, 19],
                [29, -9],
                [10, -20],
                [24, -13],
                [29, 4],
                [20, -20],
                [20, -15],
                [29, 14],
                [10, 24],
                [25, 11],
                [28, 19],
                [28, 8],
                [32, 11],
                [22, 12],
                [23, 14],
                [22, 12],
                [26, -6],
                [25, 20],
                [18, 16],
                [26, -2],
                [23, 14],
                [5, 20],
                [24, 16],
                [22, 11],
                [28, 9],
                [26, 5],
                [24, -3],
                [26, -6],
                [23, -16],
                [2, -25],
                [25, -19],
                [17, -16],
                [33, -6],
                [18, -16],
                [23, -16],
                [27, -3],
                [22, 11],
                [24, 24],
                [26, -13],
                [27, -7],
                [26, -6],
                [28, -5],
                [27, 0],
                [23, -60],
                [-1, -14],
                [-3, -26],
                [-27, -15],
                [-22, -21],
                [4, -23],
                [31, 1],
                [-3, -22],
                [-15, -22],
                [-13, -24],
                [22, -18],
                [32, -5],
                [32, 10],
                [15, 22],
                [9, 22],
                [15, 18],
                [18, 17],
                [7, 20],
                [15, 28],
                [17, 6],
                [32, 2],
                [27, 7],
                [29, 9],
                [13, 23],
                [8, 21],
                [19, 22],
                [28, 14],
                [23, 11],
                [15, 20],
                [16, 10],
                [20, 9],
                [28, -6],
                [25, 6],
                [27, 7],
                [31, -4],
                [20, 16],
                [14, 38],
                [10, -15],
                [13, -27],
                [24, -12],
                [26, -4],
                [27, 6],
                [28, -4],
                [26, -1],
                [18, 5],
                [23, -3],
                [21, -12],
                [25, 8],
                [30, 0],
                [26, 7],
                [29, -7],
                [18, 19],
                [14, 19],
                [19, 16],
                [35, 43],
                [18, -8],
                [21, -16],
                [19, -20],
                [35, -35],
                [27, -1],
                [26, 0],
                [30, 6],
                [30, 8],
                [23, 16],
                [19, 17],
                [31, 2],
                [20, 13],
                [22, -12],
                [14, -18],
                [20, -18],
                [30, 2],
                [19, -14],
                [34, -15],
                [34, -6],
                [29, 5],
                [22, 18],
                [18, 18],
                [25, 5],
                [25, -8],
                [29, -6],
                [26, 9],
                [25, 0],
                [25, -6],
                [26, -5],
                [25, 10],
                [29, 9],
                [29, 2],
                [31, 0],
                [26, 6],
                [25, 4],
                [8, 29],
                [1, 23],
                [17, -15],
                [5, -26],
                [9, -24],
                [12, -19],
                [23, -10],
                [32, 3],
                [36, 1],
                [25, 3],
                [36, 0],
                [27, 2],
                [36, -3],
                [31, -4],
                [20, -18],
                [-6, -22],
                [18, -17],
                [30, -13],
                [31, -15],
                [36, -10],
                [37, -9],
                [29, -9],
                [31, -1],
                [18, 19],
                [25, -16],
                [21, -18],
                [24, -13],
                [34, -6],
                [32, -7],
                [14, -22],
                [31, -14],
                [22, -20],
                [31, -9],
                [32, 1],
                [30, -3],
                [33, 1],
                [33, -5],
                [31, -8],
                [29, -13],
                [29, -12],
                [19, -17],
                [-3, -22],
                [-15, -20],
                [-12, -26],
                [-10, -21],
                [-13, -23],
                [-36, -9],
                [-17, -21],
                [-36, -12],
                [-12, -23],
                [-19, -21],
                [-20, -18],
                [-12, -24],
                [-7, -21],
                [-3, -26],
                [1, -22],
                [16, -22],
                [6, -22],
                [13, -20],
                [51, -8],
                [11, -25],
                [-50, -9],
                [-42, -12],
                [-53, -2],
                [-23, -33],
                [-5, -27],
                [-12, -22],
                [-15, -21],
                [37, -19],
                [14, -24],
                [24, -21],
                [34, -20],
                [39, -18],
                [41, -18],
                [64, -18],
                [14, -28],
                [80, -12],
                [6, -5],
                [20, -17],
                [77, 15],
                [64, -18],
                [48, -14],
                [-9998, -1],
                [25, 34],
                [50, -18],
                [3, 2],
                [29, 18],
                [4, 0],
                [3, -1],
                [41, -24],
                [35, 24],
                [6, 3],
                [82, 11],
                [26, -14],
                [13, -7],
                [42, -19],
                [79, -15],
                [62, -18],
                [108, -13],
                [80, 16],
                [118, -12],
                [66, -18],
                [74, 17],
                [77, 16],
                [6, 27],
                [-109, 2],
                [-90, 14],
                [-23, 23],
                [-75, 12],
                [5, 26],
                [10, 24],
                [11, 21],
                [-6, 24],
                [-46, 16],
                [-21, 20],
                [-43, 18],
                [67, -4],
                [64, 10],
                [41, -20],
                [49, 17],
                [46, 22],
                [22, 19],
                [-10, 24],
                [-35, 15],
                [-41, 17],
                [-57, 4],
                [-50, 8],
                [-54, 5],
                [-18, 22],
                [-36, 18],
                [-22, 20],
                [-9, 65],
                [14, -5],
                [25, -18],
                [46, 5],
                [44, 8],
                [23, -25],
                [44, 6],
                [37, 13],
                [35, 15],
                [31, 20],
                [42, 5],
                [-1, 22],
                [-10, 21],
                [8, 20],
                [36, 10],
                [16, -19],
                [43, 12],
                [32, 14],
                [40, 1],
                [37, 6],
                [38, 14],
                [30, 12],
                [33, 12],
                [22, -3],
                [19, -5],
                [42, 8],
                [37, -10],
                [38, 1],
                [36, 8],
                [38, -5],
                [41, -6],
                [39, 2],
                [40, -1],
                [41, -1],
                [38, 2],
                [29, 17],
                [33, 9],
                [35, -12],
                [33, 10],
                [30, 20],
                [18, -18],
                [10, -20],
                [18, -19],
                [29, 17],
                [33, -22],
                [37, -7],
                [33, -15],
                [39, 3],
                [35, 10],
                [42, -2],
                [37, -8],
                [39, -10],
                [14, 25],
                [-18, 19],
                [-13, 20],
                [-36, 5],
                [-16, 21],
                [-6, 22],
                [-10, 42],
                [21, -7],
                [37, -4],
                [36, 4],
                [32, -9],
                [29, -17],
                [12, -21],
                [37, -3],
                [36, 8],
                [38, 11],
                [35, 7],
                [28, -14],
                [37, 5],
                [24, 44],
                [22, -26],
                [32, -10],
                [35, 5],
                [23, -22],
                [36, -2],
                [34, -7],
                [33, -13],
                [22, 22],
                [11, 20],
                [28, -22],
                [38, 5],
                [28, -12],
                [19, -19],
                [37, 5],
                [29, 13],
                [28, 14],
                [34, 8],
                [39, 7],
                [35, 8],
                [28, 12],
                [16, 18],
                [6, 25],
                [-3, 24],
                [-9, 22],
                [-9, 23],
                [-9, 23],
                [-7, 20],
                [-2, 22],
                [3, 23],
                [13, 21],
                [11, 24],
                [4, 23],
                [-5, 25],
                [-4, 22],
                [14, 26],
                [15, 17],
                [18, 21],
                [19, 18],
                [23, 17],
                [10, 25],
                [16, 16],
                [17, 15],
                [27, 3],
                [17, 18],
                [20, 11],
                [23, 7],
                [20, 15],
                [16, 18],
                [21, 7],
                [17, -15],
                [-11, -19],
                [-28, -17],
            ],
            [[6914, 2382], [18, -18], [26, -7], [1, -11], [-8, -26], [-43, -4], [0, 30], [4, 24], [2, 12]],
            [[5449, 7880], [-5, -10], [-25, -1], [-14, -13], [-23, 4]],
            [[5382, 7860], [-39, 15], [-6, 20], [-28, -10], [-3, -11], [-17, 8]],
            [[5289, 7882], [-14, 2], [-13, 10], [5, 14], [-2, 11]],
            [[5265, 7919], [9, 3], [14, -16], [4, 15], [24, -2], [20, 10], [14, -2], [8, -12], [3, 10], [-4, 38], [10, 7], [10, 26]],
            [[5377, 7996], [20, -18], [16, 23], [10, 5], [21, -18], [13, 3], [13, -11]],
            [[5470, 7980], [-2, -7], [3, -20]],
            [[5471, 7953], [-2, -23], [-16, -1], [5, -12], [-9, -37]],
            [[6281, 7413], [-19, 7], [-14, 27], [-5, 22]],
            [[6357, 7389], [-7, -3], [-18, 30], [10, 28], [-8, 17], [-11, -4], [-32, -43]],
            [[6248, 7557], [7, 10], [21, -17], [15, -3], [3, 6], [-13, 31], [7, 8]],
            [[6288, 7592], [8, -2], [19, -34], [12, -4], [5, 14], [16, 23]],
            [[5805, 5018], [17, -4], [9, 33], [14, -4]],
            [[5845, 5043], [2, -23], [6, -13], [0, -18], [-7, -13], [-11, -30], [-10, -20], [-11, -3]],
            [[5814, 4923], [-2, 69], [-7, 26]],
            [[5170, 8107], [-3, -39]],
            [[5167, 8068], [-7, -2], [-3, -32]],
            [[5157, 8034], [-25, 26], [-14, -4], [-19, 27], [-13, 23], [-13, 1], [-4, 20]],
            [[5091, 8139], [20, -5], [26, 12], [18, -25], [15, -14]],
            [[5024, 5815], [10, 7], [5, 25], [14, 5], [6, 18]],
            [[5059, 5870], [9, 16], [10, 1], [21, -34]],
            [[5099, 5853], [-1, -19], [6, -34], [-5, -23], [3, -16], [-14, -35], [-8, -18], [-5, -36], [0, -37], [-1, -93]],
            [[5051, 5536], [-7, 39], [1, 133], [-5, 11], [-1, 29], [-10, 20], [-9, 17], [4, 30]],
            [
                [4849, 5779],
                [-2, 34],
                [8, 24],
                [-1, 19],
                [22, 48],
                [4, 40],
                [8, 14],
                [13, -8],
                [12, 12],
                [4, 15],
                [21, 25],
                [5, 18],
                [26, 24],
                [16, 8],
                [7, -11],
                [17, 1],
            ],
            [[5009, 6042], [-2, -28], [4, -27], [16, -37], [0, -28], [32, -13], [0, -39]],
            [[5024, 5815], [-24, 1]],
            [[5000, 5816], [-13, 5], [-9, -9], [-12, 4], [-49, -3], [0, -32], [3, -44]],
            [[4920, 5737], [-19, 15], [-13, -2], [-9, -15], [-13, 13], [-5, 19], [-12, 12]],
            [
                [7472, 6452],
                [-4, 47],
                [-10, 44],
                [5, 34],
                [-17, 16],
                [6, 21],
                [17, 21],
                [-20, 31],
                [10, 39],
                [22, -25],
                [13, -3],
                [3, -40],
                [26, -8],
                [26, 1],
                [16, -10],
                [-13, -49],
                [-12, -3],
                [-9, -33],
                [15, -29],
                [5, 36],
                [7, 1],
                [15, -92],
            ],
            [[7573, 6451], [-1, -41], [-9, 9], [2, -47]],
            [[5777, 7601], [-24, 8], [-29, -19]],
            [[5724, 7590], [0, -28], [-25, -6], [-20, 20], [-22, -15], [-20, 1]],
            [[5637, 7562], [-2, 38], [-14, 19]],
            [[5621, 7619], [4, 8], [-3, 7], [5, 18], [10, 18], [-13, 25], [-3, 21], [7, 13]],
            [[5628, 7729], [8, -24], [11, 5], [21, -9], [41, -3], [14, 14], [33, 14], [20, -21], [16, -6]],
            [[5533, 7688], [-5, -5], [-9, -13], [-4, -32]],
            [
                [5515, 7638],
                [-25, 22],
                [-11, 24],
                [-10, 12],
                [-13, 22],
                [-6, 18],
                [-14, 27],
                [6, 24],
                [10, -14],
                [6, 12],
                [13, 2],
                [24, -10],
                [19, 1],
                [13, -13],
            ],
            [[5527, 7765], [10, 0], [-7, -25], [13, -22], [-4, -27], [-6, -3]],
            [[5735, 8384], [17, 10], [30, 22]],
            [
                [5782, 8416],
                [29, -14],
                [4, -14],
                [14, 6],
                [28, -13],
                [2, -27],
                [-6, -16],
                [18, -38],
                [11, -10],
                [-2, -10],
                [19, -11],
                [8, -15],
                [-11, -12],
                [-22, 2],
                [-5, -6],
                [6, -19],
                [7, -37],
            ],
            [
                [5882, 8182],
                [-24, -3],
                [-9, -13],
                [-1, -29],
                [-11, 6],
                [-25, -3],
                [-8, 13],
                [-10, -10],
                [-11, 9],
                [-21, 1],
                [-31, 14],
                [-29, 4],
                [-21, -1],
                [-15, -16],
                [-14, -2],
            ],
            [[5652, 8152], [0, 26], [-9, 26], [17, 12], [0, 23], [-8, 22], [-1, 25]],
            [[5651, 8286], [27, 0], [30, 22], [7, 32], [22, 19], [-2, 25]],
            [[2529, 6097], [-8, 0], [2, 65], [0, 45]],
            [[2523, 6207], [0, 9], [3, 3], [5, -7], [10, 34], [5, 1]],
            [[3135, 3873], [-20, -8], [-11, 79], [-15, 65], [9, 56], [-15, 24], [-3, 41], [-14, 40]],
            [[3066, 4170], [18, 62], [-12, 48], [6, 20], [-5, 21], [11, 29], [0, 49], [2, 40], [6, 20], [-24, 92]],
            [
                [3068, 4551],
                [20, -5],
                [15, 2],
                [6, 17],
                [24, 23],
                [15, 22],
                [36, 10],
                [-3, -43],
                [4, -22],
                [-3, -39],
                [31, -52],
                [31, -9],
                [11, -22],
                [18, -11],
                [12, -17],
                [17, 1],
                [17, -17],
                [1, -34],
                [5, -16],
                [1, -25],
                [-9, -1],
                [11, -67],
                [53, -3],
                [-4, -33],
                [3, -23],
                [15, -16],
                [7, -36],
                [-5, -45],
                [-8, -25],
                [3, -33],
                [-9, -12],
            ],
            [[3383, 4020], [0, 18], [-26, 29], [-26, 1], [-48, -17], [-13, -50], [-1, -31], [-11, -69]],
            [
                [3482, 3700],
                [5, 33],
                [4, 34],
                [0, 32],
                [-10, 10],
                [-10, -9],
                [-11, 2],
                [-3, 23],
                [-3, 52],
                [-5, 18],
                [-19, 15],
                [-11, -11],
                [-29, 11],
                [2, 78],
                [-9, 32],
            ],
            [
                [3068, 4551],
                [-16, -10],
                [-12, 7],
                [1, 87],
                [-22, -33],
                [-25, 1],
                [-10, 31],
                [-19, 3],
                [6, 25],
                [-15, 35],
                [-12, 52],
                [7, 10],
                [0, 25],
                [17, 16],
                [-3, 31],
                [7, 20],
                [2, 27],
                [32, 39],
                [23, 12],
                [4, 8],
                [25, -3],
            ],
            [
                [3058, 4934],
                [12, 158],
                [1, 25],
                [-5, 33],
                [-12, 21],
                [0, 42],
                [16, 10],
                [6, -6],
                [0, 22],
                [-16, 6],
                [0, 36],
                [54, -2],
                [9, 20],
                [8, -18],
                [5, -34],
                [6, 7],
            ],
            [
                [3142, 5254],
                [15, -30],
                [21, 3],
                [6, 18],
                [20, 13],
                [12, 10],
                [3, 24],
                [20, 17],
                [-2, 12],
                [-23, 5],
                [-4, 36],
                [1, 39],
                [-12, 15],
                [5, 5],
                [21, -8],
                [22, -14],
                [8, 14],
                [20, 9],
                [31, 21],
                [10, 22],
                [-4, 16],
            ],
            [
                [3312, 5481],
                [15, 3],
                [6, -13],
                [-4, -26],
                [10, -8],
                [6, -27],
                [-7, -20],
                [-5, -49],
                [7, -29],
                [2, -27],
                [17, -27],
                [14, -3],
                [3, 11],
                [9, 3],
                [12, 10],
                [9, 15],
                [16, -5],
                [7, 2],
            ],
            [[3429, 5291], [15, -4], [2, 11], [-4, 12], [2, 17], [12, -6], [13, 6], [16, -12]],
            [[3485, 5315], [12, -12], [8, 16], [7, -3], [3, -16], [14, 4], [10, 22], [9, 43], [16, 52]],
            [[3517, 3237], [-8, 33], [12, 27], [-16, 40], [-22, 31], [-28, 37], [-11, -1], [-28, 44], [-18, -6]],
            [[8206, 5496], [-2, -29], [-1, -36], [-13, 1], [-6, -19], [-13, 30]],
            [[7466, 6754], [18, 43], [15, 14], [20, -13], [15, -1], [12, -16]],
            [[7546, 6781], [11, -18], [-2, -36], [-22, -1], [-24, 4], [-17, -9], [-26, 21], [0, 12]],
            [
                [5816, 3910],
                [-39, -43],
                [-25, -43],
                [-9, -38],
                [-8, -22],
                [-15, -4],
                [-5, -28],
                [-3, -18],
                [-18, -13],
                [-23, 3],
                [-13, 16],
                [-12, 7],
                [-13, -13],
                [-7, -28],
                [-13, -17],
                [-14, -26],
                [-20, -6],
                [-6, 20],
                [3, 35],
                [-17, 55],
                [-7, 9],
            ],
            [[5552, 3756], [0, 168], [27, 2], [1, 205], [20, 2], [43, 20], [11, -24], [18, 23], [8, 0], [16, 13]],
            [[5696, 4165], [5, -4]],
            [[5701, 4161], [10, -46], [6, -11], [9, -33], [31, -63], [12, -6], [0, -21], [8, -36], [22, -9], [17, -26]],
            [[5634, 5824], [3, -25], [16, -36], [0, -24], [-4, -24], [2, -17], [9, -17]],
            [[5660, 5681], [21, -25]],
            [[5681, 5656], [16, -23], [0, -19], [19, -30], [11, -25], [7, -35], [21, -22], [4, -19]],
            [
                [5759, 5483],
                [-9, -6],
                [-18, 2],
                [-21, 6],
                [-10, -5],
                [-4, -14],
                [-9, -2],
                [-11, 12],
                [-31, -29],
                [-13, 6],
                [-3, -4],
                [-9, -35],
                [-20, 11],
                [-21, 6],
                [-17, 21],
                [-23, 20],
                [-15, -19],
                [-11, -29],
                [-2, -40],
            ],
            [[5512, 5384], [-18, 3], [-19, 10], [-17, -31], [-14, -53]],
            [[5444, 5313], [-3, 16], [-1, 26], [-13, 19], [-10, 30], [-3, 20], [-13, 30], [2, 18], [-2, 24], [2, 45], [6, 10], [14, 58]],
            [
                [5423, 5609],
                [23, 5],
                [5, 14],
                [5, -1],
                [7, -13],
                [35, 22],
                [12, 23],
                [14, 20],
                [-2, 20],
                [7, 6],
                [27, -4],
                [26, 27],
                [20, 62],
                [14, 24],
                [18, 10],
            ],
            [[1300, 8301], [13, -7], [27, 4], [-9, -65], [25, -46], [-12, 0], [-16, 26], [-11, 27], [-14, 18], [-5, 25], [2, 18]],
            [
                [3134, 7781],
                [-18, 33],
                [0, 78],
                [-12, 17],
                [-19, -10],
                [-9, 15],
                [-21, -43],
                [-9, -45],
                [-10, -26],
                [-11, -9],
                [-9, -3],
                [-3, -14],
                [-51, 0],
                [-42, -1],
                [-13, -10],
                [-29, -42],
                [-4, -4],
                [-9, -23],
                [-25, 0],
                [-27, 0],
                [-13, -9],
                [4, -11],
                [3, -18],
                [-1, -6],
                [-36, -28],
                [-28, -10],
                [-33, -30],
                [-7, 0],
                [-9, 9],
                [-3, 8],
                [0, 6],
                [6, 20],
                [14, 32],
                [8, 34],
                [-6, 50],
                [-6, 52],
                [-29, 27],
                [4, 10],
                [-4, 8],
                [-8, 0],
                [-6, 9],
                [-1, 13],
                [-5, -6],
                [-8, 2],
                [2, 6],
                [-7, 5],
                [-2, 16],
                [-22, 18],
                [-22, 19],
                [-28, 22],
                [-26, 21],
                [-25, -16],
                [-9, 0],
                [-34, 14],
                [-22, -7],
                [-27, 18],
                [-29, 9],
                [-19, 4],
                [-9, 9],
                [-5, 32],
                [-9, 0],
                [0, -22],
                [-58, 0],
                [-95, 0],
                [-94, 0],
                [-83, 0],
                [-84, 0],
                [-82, 0],
                [-84, 0],
                [-28, 0],
                [-82, 0],
                [-79, 0],
            ],
            [
                [1373, 8338],
                [16, 27],
                [-1, 37],
                [-47, 36],
                [-29, 66],
                [-17, 41],
                [-25, 26],
                [-19, 24],
                [-15, 30],
                [-28, -19],
                [-27, -32],
                [-24, 38],
                [-20, 25],
                [-27, 16],
                [-27, 2],
                [0, 327],
                [0, 214],
            ],
            [[1440, 9241], [19, -7], [47, -51]],
            [[2457, 9224], [-25, -29], [52, -11]],
            [[1972, 9143], [-71, -9], [-49, -6]],
            [[1501, 9320], [12, 25], [19, 42]],
            [[1653, 9318], [0, -26], [-73, -27]],
            [[5289, 7882], [-2, -23], [-12, -10], [-21, 7], [-6, -23], [-13, -2], [-5, 9], [-16, -19], [-13, -3], [-12, 12]],
            [[5189, 7830], [-9, 26], [-14, -9], [1, 26], [20, 32], [-1, 15], [13, -6], [7, 10]],
            [[5206, 7924], [24, 0], [6, 12], [29, -17]],
            [[3139, 2021], [-9, -23], [-24, -18]],
            [[3106, 1980], [-13, 2], [-17, 4]],
            [
                [3076, 1986],
                [-20, 17],
                [-29, 9],
                [-35, 32],
                [-28, 31],
                [-39, 64],
                [23, -12],
                [39, -38],
                [37, -21],
                [14, 27],
                [9, 39],
                [26, 24],
                [20, -7],
            ],
            [[3044, 4125], [15, 15], [7, 30]],
            [[8628, 7623], [-18, 34], [-11, -32], [-43, -25], [4, -30], [-24, 2], [-13, 18], [-19, -41], [-31, -31], [-23, -37]],
            [[8000, 6423], [-28, 15], [-13, 23], [4, 34], [-25, 10], [-13, 22], [-24, -31], [-27, -7], [-22, 1], [-15, -14]],
            [[7837, 6476], [-15, -9], [5, -66], [-15, 2], [-3, 13]],
            [
                [7809, 6416],
                [-1, 24],
                [-20, -17],
                [-12, 11],
                [-21, 22],
                [9, 47],
                [-18, 12],
                [-7, 53],
                [-29, -10],
                [3, 68],
                [27, 48],
                [1, 48],
                [-1, 44],
                [-12, 14],
                [-9, 34],
                [-17, -5],
            ],
            [[7702, 6809], [-30, 9], [10, 24], [-13, 36], [-20, -24], [-23, 14], [-32, -37], [-26, -43], [-22, -7]],
            [[7466, 6754], [-3, 45], [-16, -12]],
            [[7447, 6787], [-33, 6], [-31, 13], [-23, 25], [-21, 12], [-10, 27], [-15, 9], [-28, 37], [-23, 18], [-11, -14]],
            [[7252, 6920], [-39, 40], [-27, 37], [-8, 63], [20, -8], [1, 30], [-11, 29], [3, 47], [-30, 68]],
            [[7161, 7226], [-46, 23], [-8, 44], [-20, 27]],
            [[7082, 7337], [-5, 33], [1, 22], [-16, 13], [-10, -6], [-7, 54]],
            [[7045, 7453], [8, 13], [-4, 13], [27, 27], [19, 12], [30, -8], [10, 37], [36, 7], [10, 22], [43, 32], [4, 13]],
            [[7228, 7621], [-2, 32], [19, 15], [-25, 100], [55, 23], [14, 13], [20, 103], [55, -19], [16, 26], [1, 58], [23, 6], [21, 38]],
            [[7425, 8016], [11, 5]],
            [
                [7436, 8021],
                [8, -41],
                [23, -30],
                [40, -22],
                [19, -46],
                [-11, -67],
                [10, -25],
                [33, -10],
                [37, -8],
                [34, -36],
                [17, -6],
                [13, -54],
                [16, -34],
                [31, 2],
                [57, -13],
                [37, 8],
                [27, -9],
                [41, -35],
                [34, 0],
                [12, -18],
                [33, 31],
                [45, 20],
                [41, 2],
                [33, 21],
                [20, 30],
                [19, 20],
                [-4, 19],
                [-9, 22],
                [14, 37],
                [16, -5],
                [28, -12],
                [28, 31],
                [42, 22],
                [21, 38],
                [19, 17],
                [41, 7],
                [22, -6],
                [3, 20],
                [-25, 40],
                [-23, 19],
                [-21, -21],
                [-28, 9],
                [-15, -8],
                [-7, 24],
                [19, 57],
                [14, 44],
            ],
            [
                [8240, 8055],
                [33, -22],
                [39, 36],
                [0, 26],
                [25, 61],
                [16, 18],
                [-1, 32],
                [-15, 14],
                [23, 28],
                [34, 11],
                [37, 1],
                [42, -17],
                [24, -21],
                [17, -58],
                [11, -25],
                [9, -35],
                [11, -57],
                [48, -18],
                [33, -41],
                [11, -54],
                [42, 0],
                [24, 23],
                [46, 16],
                [-14, -51],
                [-11, -21],
                [-10, -63],
                [-18, -57],
                [-34, 11],
                [-24, -21],
                [8, -49],
                [-4, -68],
                [-15, -2],
                [1, -29],
            ],
            [[4785, 5433], [2, 48], [3, 7], [-1, 23], [-12, 24], [-9, 4], [-8, 15], [6, 26], [-3, 28], [2, 17]],
            [[4765, 5625], [4, 0], [2, 25], [-3, 11], [3, 8], [10, 7], [-6, 46], [-7, 24], [2, 19], [6, 5]],
            [[4776, 5770], [4, 5], [7, -9], [22, 0], [5, 17], [5, -2], [8, 7], [4, -25], [6, 8], [12, 8]],
            [[4920, 5737], [8, -82], [-12, -48], [-7, -65], [12, -50], [-1, -22]],
            [[5312, 5312], [-45, 1]],
            [
                [5235, 5457],
                [7, 41],
                [13, 55],
                [8, 1],
                [17, 33],
                [11, 1],
                [15, -23],
                [19, 19],
                [3, 24],
                [6, 23],
                [4, 29],
                [15, 24],
                [6, 40],
                [6, 13],
                [4, 30],
                [7, 37],
                [23, 44],
                [2, 19],
                [3, 11],
                [-11, 23],
            ],
            [[5393, 5901], [1, 18], [8, 3]],
            [
                [5402, 5922],
                [11, -36],
                [2, -39],
                [-1, -38],
                [15, -52],
                [-16, 0],
                [-8, -4],
                [-12, 6],
                [-6, -27],
                [16, -34],
                [12, -10],
                [4, -23],
                [9, -40],
                [-5, -16],
            ],
            [[5444, 5313], [-2, -32], [-22, 14], [-23, 15], [-35, 3]],
            [[5362, 5313], [-3, 3], [-17, -8], [-17, 8], [-13, -4]],
            [[5821, 5105], [-8, -16], [-1, -35], [-4, -4], [-3, -32]],
            [[5814, 4923], [5, -53], [-3, -30], [6, -33], [16, -33], [15, -72]],
            [
                [5853, 4702],
                [-11, 6],
                [-37, -10],
                [-8, -7],
                [-8, -37],
                [6, -25],
                [-5, -68],
                [-3, -58],
                [8, -10],
                [19, -23],
                [8, 11],
                [2, -62],
                [-21, 0],
                [-12, 32],
                [-10, 24],
                [-21, 8],
                [-7, 31],
                [-16, -19],
                [-23, 8],
                [-9, 26],
                [-18, 6],
                [-13, -2],
                [-1, 18],
                [-10, 2],
            ],
            [
                [5360, 4907],
                [7, -6],
                [10, 22],
                [15, -1],
                [2, -16],
                [10, -10],
                [16, 36],
                [17, 28],
                [7, 18],
                [-1, 48],
                [12, 56],
                [12, 29],
                [19, 28],
                [3, 19],
                [1, 21],
                [4, 20],
                [-1, 32],
                [3, 51],
                [6, 36],
                [8, 31],
                [2, 35],
            ],
            [[5759, 5483], [17, -47], [13, -7], [7, 10], [13, -4], [15, 12], [7, -25], [24, -38]],
            [[5855, 5384], [-1, -67], [11, -8], [-9, -21], [-11, -15], [-10, -30], [-6, -27], [-2, -46], [-6, -22], [0, -43]],
            [
                [5307, 4953],
                [21, 32],
                [-10, 38],
                [9, 14],
                [19, 7],
                [2, 26],
                [15, -28],
                [25, -2],
                [8, 27],
                [4, 38],
                [-3, 45],
                [-14, 34],
                [13, 67],
                [-7, 11],
                [-21, -4],
                [-8, 29],
                [2, 26],
            ],
            [[2836, 5598], [3, 28], [9, -4], [6, 17], [-7, 34], [4, 8]],
            [
                [3018, 5860],
                [-18, -10],
                [-7, -28],
                [-11, -17],
                [-8, -21],
                [-3, -41],
                [-8, -34],
                [14, -4],
                [4, -26],
                [6, -13],
                [2, -23],
                [-3, -22],
                [1, -12],
                [7, -4],
                [6, -20],
                [36, 5],
                [16, -7],
                [20, -50],
                [11, 6],
                [20, -3],
                [16, 7],
                [10, -10],
                [-5, -31],
                [-7, -19],
                [-2, -42],
                [6, -38],
                [8, -17],
                [1, -13],
                [-14, -29],
                [10, -12],
                [7, -20],
                [9, -58],
            ],
            [
                [3058, 4934],
                [-14, 31],
                [-8, 1],
                [17, 59],
                [-21, 27],
                [-17, -5],
                [-10, 10],
                [-15, -15],
                [-21, 7],
                [-16, 60],
                [-13, 15],
                [-9, 27],
                [-18, 28],
                [-7, -6],
            ],
            [[2906, 5173], [-12, 14], [-14, 19], [-8, -9], [-23, 8], [-7, 25], [-5, -1], [-28, 32]],
            [[2618, 5820], [5, 8], [18, -15], [6, 7], [9, -5], [5, -12], [8, -3], [6, 12]],
            [[2706, 5733], [-10, -5], [0, -24], [5, -8], [-4, -7], [1, -10], [-2, -12], [-1, -11]],
            [
                [2714, 6517],
                [24, -4],
                [22, -1],
                [26, -19],
                [11, -21],
                [26, 6],
                [10, -13],
                [23, -36],
                [18, -26],
                [9, 1],
                [16, -12],
                [-2, -16],
                [21, -2],
                [21, -24],
                [-4, -13],
                [-18, -8],
                [-19, -3],
                [-19, 5],
                [-40, -6],
                [19, 32],
                [-11, 15],
                [-18, 4],
                [-10, 17],
                [-6, 33],
                [-16, -3],
                [-26, 16],
                [-8, 12],
                [-37, 9],
                [-9, 11],
                [10, 14],
                [-27, 3],
                [-20, -30],
                [-12, 0],
                [-4, -14],
                [-13, -7],
                [-12, 6],
                [14, 18],
                [7, 20],
                [12, 13],
                [14, 11],
                [21, 6],
                [7, 6],
            ],
            [[5943, 7201], [-3, 2], [-6, -5], [-4, 2], [-1, -3], [-1, 6], [-2, 4], [-5, 0], [-8, -5], [-5, 3]],
            [[5377, 7996], [-16, 25], [-14, 14], [-3, 24], [-5, 17], [20, 13], [10, 14], [20, 11], [7, 11], [8, -6], [12, 6]],
            [[5416, 8125], [13, -19], [21, -5], [-2, -16], [15, -11], [5, 14], [19, -6], [2, -18], [21, -4], [13, -28]],
            [[5523, 8032], [-9, 0], [-4, -10], [-6, -3], [-2, -13], [-5, -3], [-1, -5], [-10, -6], [-12, 1], [-4, -13]],
            [[5391, 8277], [7, -29], [-8, -15], [10, -21], [7, -31], [-2, -19], [11, -37]],
            [[5206, 7924], [4, 41], [14, 40], [-40, 10], [-13, 15]],
            [[5171, 8030], [1, 25], [-5, 13]],
            [[5170, 8107], [-5, 61], [17, 0], [7, 21], [7, 53], [-5, 20]],
            [[5236, 8347], [21, -8], [18, 9]],
            [[6197, 5842], [-10, -31]],
            [[6187, 5811], [-6, 10], [-7, -4], [-15, 1], [-1, 18], [-2, 16], [10, 27], [9, 25]],
            [[6175, 5904], [12, -5], [9, 14]],
            [[3007, 6221], [1, 16], [-7, 17], [6, 10], [3, 22], [-3, 31]],
            [[5118, 6285], [-31, -6], [-1, 37], [-12, 9], [-18, 17], [-6, 27], [-94, 125], [-94, 126]],
            [[4862, 6620], [-104, 139]],
            [[4758, 6759], [0, 12], [0, 4]],
            [
                [4758, 6775],
                [0, 68],
                [45, 42],
                [28, 9],
                [22, 15],
                [11, 29],
                [32, 23],
                [2, 43],
                [16, 5],
                [12, 21],
                [37, 10],
                [5, 22],
                [-8, 12],
                [-9, 61],
                [-2, 35],
                [-10, 37],
            ],
            [[5233, 7309], [-6, -29], [5, -55], [-7, -47], [-17, -33], [2, -43], [23, -34], [0, -14], [18, -23], [11, -104]],
            [
                [5262, 6927],
                [9, -51],
                [2, -26],
                [-5, -47],
                [2, -27],
                [-4, -31],
                [3, -36],
                [-11, -24],
                [16, -42],
                [1, -25],
                [10, -32],
                [13, 10],
                [22, -26],
                [12, -36],
            ],
            [[5332, 6534], [-95, -110], [-80, -113], [-39, -26]],
            [
                [2906, 5173],
                [3, -44],
                [-8, -37],
                [-31, -60],
                [-33, -23],
                [-17, -50],
                [-5, -39],
                [-16, -24],
                [-12, 29],
                [-11, 7],
                [-11, -5],
                [-1, 21],
                [8, 14],
                [-3, 24],
            ],
            [[6023, 6449], [-110, 0], [-108, 0], [-112, 0]],
            [[5693, 6449], [0, 212], [0, 205], [-8, 46], [7, 36], [-4, 24], [10, 28]],
            [[5951, 6980], [18, -99]],
            [[6011, 6012], [-3, 23], [12, 85], [3, 38], [8, 18], [21, 9], [14, 33]],
            [[6175, 5904], [-9, 19], [-12, 34], [-12, 18], [-7, 20], [-24, 23], [-19, 1], [-7, 12], [-16, -14], [-17, 26], [-9, -43], [-32, 12]],
            [[4946, 7682], [11, -22], [51, -26], [10, 12], [32, -26], [32, 8]],
            [
                [4792, 7318],
                [-2, 19],
                [10, 22],
                [4, 15],
                [-10, 18],
                [8, 37],
                [-11, 35],
                [12, 5],
                [1, 27],
                [4, 8],
                [1, 45],
                [13, 16],
                [-8, 29],
                [-16, 2],
                [-5, -8],
                [-17, 0],
                [-7, 29],
                [-11, -9],
                [-10, -14],
            ],
            [[5776, 8607], [4, -10], [-19, -33], [8, -54], [-12, -18]],
            [[5757, 8492], [-23, 0], [-24, 21], [-12, 7], [-24, -10]],
            [[6187, 5811], [-6, -20], [10, -32], [11, -28], [10, -20], [91, -69], [23, 1]],
            [[6326, 5643], [-78, -173], [-36, -3], [-25, -40], [-18, -1], [-7, -18]],
            [
                [6162, 5408],
                [-19, 0],
                [-12, 19],
                [-25, -24],
                [-8, -24],
                [-19, 5],
                [-6, 6],
                [-6, -1],
                [-9, 0],
                [-35, 49],
                [-20, 0],
                [-9, 19],
                [0, 32],
                [-15, 10],
            ],
            [[5979, 5499], [-16, 63], [-13, 13], [-5, 23], [-14, 28], [-17, 4], [10, 33], [15, 1], [4, 18]],
            [[5943, 5682], [-1, 52]],
            [[5942, 5734], [9, 60], [13, 16], [2, 24], [12, 44], [17, 28], [11, 57], [5, 49]],
            [[5663, 8983], [-9, 22], [-1, 89], [-44, 39], [-37, 28]],
            [[5572, 9161], [17, 16], [31, -31], [36, 3], [30, -14], [27, 25], [13, 43], [43, 19], [36, -23], [-12, -40]],
            [
                [5793, 9159],
                [-4, -40],
                [43, -39],
                [-26, -43],
                [32, -66],
                [-18, -49],
                [25, -43],
                [-12, -37],
                [41, -40],
                [-10, -29],
                [-26, -34],
                [-59, -73],
            ],
            [[3299, 2196], [33, 35], [24, -15], [17, 23], [22, -25], [-8, -21], [-38, -17], [-12, 20], [-24, -26], [-14, 26]],
            [[3485, 5315], [7, 25], [2, 26]],
            [[3494, 5366], [5, 25], [-11, 34]],
            [[3488, 5425], [-2, 39], [14, 49]],
            [[5157, 8034], [6, -5], [8, 1]],
            [[5189, 7830], [-1, -16], [8, -22], [-10, -17], [8, -45], [15, -7], [-3, -25]],
            [[5263, 5240], [9, 3], [40, 0], [0, 69]],
            [[4827, 8284], [-21, 12], [-17, -1], [5, 31], [-5, 31]],
            [
                [4968, 8327],
                [19, -9],
                [17, -65],
                [8, -23],
                [34, -11],
                [-4, -37],
                [-14, -17],
                [11, -30],
                [-25, -30],
                [-37, 1],
                [-47, -16],
                [-13, 11],
                [-18, -27],
                [-26, 7],
                [-20, -22],
                [-14, 11],
                [40, 61],
                [25, 12],
                [-43, 10],
                [-8, 23],
                [29, 18],
                [-15, 31],
                [5, 37],
                [41, -5],
                [4, 34],
            ],
            [[4917, 8291], [-18, 35], [-1, 1]],
            [
                [4898, 8327],
                [-34, 10],
                [-6, 16],
                [10, 25],
                [-9, 16],
                [-15, -27],
                [-2, 55],
                [-14, 30],
                [10, 59],
                [22, 47],
                [22, -5],
                [34, 5],
                [-30, -62],
                [28, 8],
                [31, 0],
                [-8, -47],
                [-25, -52],
                [29, -4],
            ],
            [[4941, 8401], [2, -6], [25, -68]],
            [[6109, 7683], [3, 7], [24, -10], [41, -9], [37, -28], [5, -11], [17, 9], [26, -12], [8, -23], [18, -14]],
            [[6210, 7548], [-27, 28], [-30, -2]],
            [[5000, 5816], [-2, -17], [11, -30], [0, -42], [3, -45], [7, -21], [-6, -52], [2, -29], [7, -36], [6, -21]],
            [[4715, 5666], [-8, -3], [1, 21], [-5, 15], [1, 17], [-6, 24], [-8, 20], [-22, 1], [-6, -11], [-8, -2], [-5, -12], [-3, -16], [-15, -25]],
            [[4579, 5818], [12, 28], [9, -1], [7, 9], [6, 0], [4, 8], [-2, 19], [3, 6], [0, 20]],
            [[4618, 5907], [14, -1], [20, -14], [6, 1], [2, 7], [15, -5], [4, 3]],
            [
                [4679, 5898],
                [2, -21],
                [4, 0],
                [7, 8],
                [5, -2],
                [8, -14],
                [12, -5],
                [7, 12],
                [9, 8],
                [7, 8],
                [5, -1],
                [7, -13],
                [3, -16],
                [11, -24],
                [-5, -15],
                [-1, -19],
                [5, 6],
                [4, -7],
                [-2, -17],
                [9, -16],
            ],
            [[4765, 5625], [-8, 1], [-6, -23], [-8, 0], [-5, 12], [2, 24], [-12, 35], [-7, -7], [-6, -1]],
            [
                [4535, 5965],
                [30, 1],
                [6, 14],
                [9, 1],
                [11, -14],
                [9, 0],
                [9, 9],
                [5, -16],
                [-12, -13],
                [-12, 1],
                [-12, 12],
                [-10, -13],
                [-5, -1],
                [-6, -8],
                [-26, 1],
            ],
            [[4536, 5895], [14, 10], [10, -2], [7, 6], [51, -2]],
            [[5583, 7534], [18, 5], [11, 12], [15, -1], [4, 10], [6, 2]],
            [[5724, 7590], [14, -15], [-9, -36], [-6, -6]],
            [
                [3700, 9940],
                [93, 34],
                [98, -2],
                [35, 21],
                [98, 6],
                [222, -8],
                [174, -45],
                [-51, -23],
                [-107, -2],
                [-149, -6],
                [14, -10],
                [98, 6],
                [84, -19],
                [54, 17],
                [23, -21],
                [-31, -33],
                [71, 21],
                [135, 23],
                [83, -11],
                [16, -25],
                [-113, -41],
                [-16, -13],
                [-89, -10],
                [65, -3],
                [-33, -42],
                [-22, -37],
                [1, -64],
                [33, -38],
                [-43, -2],
                [-46, -19],
                [51, -30],
                [7, -49],
                [-30, -5],
                [36, -50],
                [-62, -4],
                [32, -23],
                [-9, -21],
                [-39, -9],
                [-39, 0],
                [35, -39],
                [1, -25],
                [-55, 23],
                [-15, -15],
                [38, -14],
                [36, -36],
                [11, -46],
                [-50, -11],
                [-21, 22],
                [-34, 33],
                [9, -39],
                [-32, -30],
                [73, -3],
                [38, -3],
                [-74, -50],
                [-76, -45],
                [-81, -20],
                [-31, 0],
                [-28, -23],
                [-39, -60],
                [-60, -41],
                [-19, -2],
                [-37, -14],
                [-40, -14],
                [-24, -35],
                [0, -41],
                [-14, -38],
                [-45, -46],
                [11, -45],
                [-13, -47],
                [-14, -56],
                [-39, -4],
                [-41, 47],
                [-56, 0],
                [-26, 32],
                [-19, 56],
                [-48, 72],
                [-14, 37],
                [-4, 52],
                [-38, 53],
                [10, 43],
                [-19, 20],
                [27, 67],
                [42, 22],
                [11, 24],
                [6, 45],
                [-32, -21],
                [-15, -8],
                [-25, -8],
                [-34, 18],
                [-2, 39],
                [11, 31],
                [26, 1],
                [57, -15],
                [-48, 36],
                [-25, 20],
                [-28, -8],
                [-23, 14],
                [31, 54],
                [-17, 21],
                [-22, 40],
                [-33, 61],
                [-36, 22],
                [1, 24],
                [-75, 34],
                [-59, 4],
                [-74, -2],
                [-68, -4],
                [-32, 18],
                [-48, 36],
                [73, 18],
                [56, 4],
                [-119, 14],
                [-63, 24],
                [4, 22],
                [105, 28],
                [102, 28],
                [11, 21],
                [-75, 20],
                [24, 23],
                [96, 41],
                [40, 6],
                [-11, 26],
                [66, 15],
                [85, 9],
                [85, 0],
                [31, -18],
                [73, 32],
                [67, -21],
                [39, -5],
                [57, -19],
                [-66, 31],
                [4, 25],
            ],
            [
                [2437, 6019],
                [1, 17],
                [3, 13],
                [-4, 11],
                [14, 47],
                [35, 0],
                [1, 20],
                [-4, 3],
                [-4, 13],
                [-10, 13],
                [-10, 19],
                [12, 1],
                [0, 32],
                [26, 0],
                [26, -1],
            ],
            [[2549, 6088], [-13, -22], [-13, -16], [-2, -12], [2, -11], [-6, -14]],
            [[2517, 6013], [-6, -4], [1, -7], [-5, -6], [-10, -15], [0, -8]],
            [[3412, 5526], [-5, -52], [-17, -15], [2, -13], [-5, -30], [12, -42], [9, 0], [4, -33], [17, -50]],
            [[3312, 5481], [-19, 44], [8, 16], [-1, 27], [17, 9], [7, 11], [-9, 21], [2, 21], [22, 34]],
            [[2561, 5953], [1, 23], [-3, 6], [-6, 4], [-12, -7], [-1, 8], [-9, 9], [-6, 12], [-8, 5]],
            [
                [2690, 6045],
                [-10, 2],
                [-4, -8],
                [-9, -8],
                [-7, 0],
                [-7, -7],
                [-5, 3],
                [-5, 8],
                [-3, -1],
                [-3, -14],
                [-3, 0],
                [0, -11],
                [-10, -16],
                [-5, -7],
                [-3, -7],
                [-8, 11],
                [-6, -15],
                [-6, 0],
                [-7, -1],
                [1, -28],
                [-4, -1],
                [-4, -13],
                [-8, -2],
            ],
            [[5515, 7638], [-4, -10]],
            [[5380, 7802], [19, -2], [5, 10], [10, -10], [11, -1], [0, 16], [9, 6], [3, 23], [22, 16]],
            [[5459, 7860], [9, -7], [21, -25], [23, -11], [10, 9]],
            [[5522, 7826], [7, -23], [9, -16], [-11, -22]],
            [[5471, 7953], [14, -15], [10, -6], [23, 7], [3, 12], [11, 1], [13, 9], [3, -3], [13, 7], [7, 13], [9, 4], [30, -18], [5, 6]],
            [[5612, 7970], [16, -15], [2, -16]],
            [[5630, 7939], [-17, -12], [-13, -39], [-17, -39], [-22, -11]],
            [[5561, 7838], [-18, 3], [-21, -15]],
            [[5459, 7860], [-5, 19], [-5, 1]],
            [[8470, 4670], [3, -11], [0, -18]],
            [[8915, 5032], [1, -187], [0, -188]],
            [
                [8045, 5298],
                [5, -39],
                [19, -33],
                [17, 12],
                [18, -4],
                [16, 29],
                [14, 5],
                [26, -16],
                [23, 12],
                [14, 80],
                [11, 20],
                [9, 66],
                [32, 0],
                [24, -10],
            ],
            [
                [7252, 6920],
                [-18, -26],
                [-11, -54],
                [27, -22],
                [27, -28],
                [36, -32],
                [38, -8],
                [16, -29],
                [21, -6],
                [34, -13],
                [23, 1],
                [3, 23],
                [-4, 36],
                [3, 25],
            ],
            [
                [7702, 6809],
                [2, -21],
                [-9, -11],
                [2, -35],
                [-20, 10],
                [-36, -40],
                [1, -33],
                [-15, -48],
                [-2, -28],
                [-12, -48],
                [-22, 13],
                [-1, -59],
                [-6, -20],
                [3, -24],
                [-14, -14],
            ],
            [
                [6893, 6546],
                [18, 39],
                [61, -1],
                [-5, 50],
                [-16, 29],
                [-3, 44],
                [-18, 26],
                [30, 61],
                [33, -5],
                [29, 61],
                [17, 58],
                [27, 58],
                [0, 41],
                [23, 33],
                [-22, 29],
                [-10, 39],
                [-10, 50],
                [14, 25],
                [42, -14],
                [31, 8],
                [27, 49],
            ],
            [[6690, 6900], [14, -31], [11, -34], [26, -26], [1, -50], [13, -10], [3, -26], [-40, -30], [-11, -67]],
            [[6348, 6905], [-15, 31], [-1, 30], [-9, 0], [5, 42], [-14, 44], [-34, 31], [-20, 55], [7, 45], [14, 20], [-2, 33], [-18, 18], [-18, 68]],
            [[6243, 7322], [-16, 46], [6, 18], [-9, 66], [19, 17]],
            [[6497, 7324], [24, 11], [20, 33], [18, -2], [13, 11], [19, -5], [31, -29], [22, -7], [32, -51], [21, -2], [2, -48]],
            [[6331, 6908], [-18, 5], [-21, -55]],
            [[6292, 6858], [-51, 4], [-79, 116], [-41, 40], [-33, 16]],
            [[6088, 7034], [-12, 70]],
            [[6076, 7104], [62, 60], [10, 70], [-2, 42], [15, 14], [14, 36]],
            [[6175, 7326], [12, 9], [32, -8], [10, -14], [14, 9]],
            [[5982, 6995], [1, -22], [-14, -92]],
            [[5975, 7087], [9, 0], [2, 10], [8, 1]],
            [[5994, 7098], [0, -23], [-3, -9], [0, -1]],
            [[5991, 7065], [-5, -18]],
            [[5986, 7047], [-10, 8], [-6, -38], [7, -7], [-7, -7], [-1, -16], [13, 8]],
            [[5382, 7860], [-3, -28], [7, -25]],
            [[2845, 6247], [18, -5], [15, -14], [5, -16], [-20, -1], [-8, -10], [-16, 9], [-16, 21], [4, 14], [11, 4], [7, -2]],
            [[6088, 7034], [-6, -9], [-55, -29], [27, -57], [-9, -10], [-4, -19], [-22, -8], [-6, -21], [-12, -18], [-31, 9]],
            [[5982, 6995], [4, 17], [0, 35]],
            [[5991, 7065], [31, -22], [54, 61]],
            [[6554, 7561], [-15, -3], [-19, 45], [-19, 16], [-31, -12], [-13, -19]],
            [
                [6363, 7854],
                [-14, 9],
                [2, 30],
                [-17, 38],
                [-21, -1],
                [-23, 39],
                [16, 43],
                [-8, 12],
                [22, 63],
                [28, -33],
                [4, 42],
                [57, 63],
                [43, 1],
                [62, -40],
                [33, -23],
                [29, 24],
                [44, 1],
                [36, -29],
                [8, 17],
                [39, -3],
                [7, 27],
                [-45, 40],
                [26, 28],
                [-5, 16],
                [27, 15],
                [-20, 39],
                [12, 20],
                [104, 20],
                [14, 14],
                [69, 21],
                [25, 24],
                [50, -12],
                [9, -60],
                [29, 14],
                [36, -20],
                [-3, -31],
                [27, 3],
                [70, 55],
                [-11, -18],
                [36, -45],
                [62, -146],
                [15, 30],
                [38, -33],
                [40, 15],
                [15, -11],
                [14, -33],
                [19, -11],
                [12, -25],
                [36, 8],
                [14, -35],
            ],
            [[7228, 7621], [-17, 8], [-14, 21], [-41, 6], [-46, 1], [-10, -6], [-40, 24], [-16, -12], [-4, -34], [-46, 20], [-18, -8], [-6, -25]],
            [
                [6970, 7616],
                [-16, -11],
                [-37, -40],
                [-12, -41],
                [-10, -1],
                [-8, 28],
                [-35, 2],
                [-6, 47],
                [-13, 0],
                [2, 58],
                [-33, 42],
                [-48, -5],
                [-33, -8],
                [-26, 52],
                [-23, 22],
                [-43, 41],
                [-5, 5],
                [-72, -34],
                [2, -212],
            ],
            [[6088, 4913], [-40, 57], [-2, 34], [-101, 117], [-4, 6]],
            [[5941, 5127], [-1, 61], [8, 24], [14, 38], [10, 42], [-12, 66], [-3, 29], [-14, 40]],
            [[5943, 5427], [18, 34], [18, 38]],
            [[6162, 5408], [-25, -66], [1, -209], [16, -48]],
            [[7045, 7453], [-52, -9], [-34, 18], [-31, -4], [3, 33], [30, -9], [10, 17]],
            [[6971, 7499], [22, -5], [35, 41], [-33, 30], [-20, -14], [-20, 22], [23, 37], [-8, 6]],
            [[7848, 5884], [-6, 69], [18, 48], [35, 11], [26, -8]],
            [[7921, 6004], [23, -23], [13, 40], [25, -21]],
            [[7982, 6000], [6, -39], [-3, -69], [-47, -44], [12, -35], [-29, -4], [-24, -23]],
            [[8504, 7356], [1, 5], [13, -2], [10, 26], [20, 3], [12, 3], [4, 14]],
            [[5556, 7634], [6, 13]],
            [[5562, 7647], [6, 4], [4, 20], [5, 3], [4, -8], [5, -4], [4, -9], [4, -3], [6, -11], [4, 1], [-3, -14], [-4, -7], [1, -4]],
            [[5598, 7615], [-6, -3], [-16, -9], [-2, -11], [-3, 0]],
            [[6344, 6826], [-20, -1], [-7, 27], [-25, 6]],
            [[7780, 6358], [6, 21], [23, 37]],
            [
                [7837, 6476],
                [16, -46],
                [12, -52],
                [35, -1],
                [10, -50],
                [-17, -15],
                [-8, -21],
                [33, -34],
                [23, -68],
                [18, -51],
                [21, -40],
                [7, -41],
                [-5, -57],
            ],
            [
                [7921, 6004],
                [9, 26],
                [2, 49],
                [-23, 50],
                [-1, 57],
                [-22, 46],
                [-21, 4],
                [-5, -20],
                [-16, -1],
                [-9, 10],
                [-29, -35],
                [-1, 52],
                [7, 61],
                [-19, 2],
                [-1, 35],
                [-12, 18],
            ],
            [[5999, 7177], [12, -3], [5, -23], [-15, -21], [-7, -32]],
            [[4681, 5573], [7, 18], [1, 17], [13, 31], [13, 27]],
            [[5262, 6927], [14, 14], [2, 24], [-3, 24], [19, 22], [9, 18], [14, 17], [1, 44]],
            [[5693, 6449], [0, -115], [-32, 0], [0, -25]],
            [[5661, 6309], [-111, 111], [-110, 110], [-29, -32]],
            [[5411, 6498], [-19, -21], [-16, 32], [-44, 25]],
            [[7271, 5615], [-5, -60], [-11, -16], [-24, -13], [-14, 45], [-4, 83], [12, 94], [19, -32], [13, -41], [14, -60]],
            [[5804, 3515], [10, -18], [-9, -28], [-5, -19], [-15, -9], [-5, -18], [-10, -6], [-21, 45], [15, 36], [15, 23], [13, 11], [12, -17]],
            [[5584, 8408], [32, 18], [46, -4], [28, 6], [3, -12], [15, -4], [27, -28]],
            [[5651, 8286], [-6, 18], [-15, 6]],
            [[5630, 8310], [-2, 15], [3, 16], [-12, 9], [-29, 10]],
            [[5757, 8492], [13, -14], [3, -28], [9, -34]],
            [
                [4758, 6775],
                [-4, 0],
                [1, -31],
                [-17, -2],
                [-9, -13],
                [-13, 0],
                [-10, 7],
                [-23, -6],
                [-9, -45],
                [-9, -4],
                [-13, -73],
                [-39, -62],
                [-9, -79],
                [-11, -26],
                [-4, -21],
                [-62, -5],
                [-1, 1],
            ],
            [[4526, 6416], [2, 26], [10, 16], [9, 30], [-1, 19], [9, 41], [16, 37], [9, 9], [7, 34], [1, 30], [10, 36], [18, 21], [18, 59]],
            [[4634, 6774], [1, 0], [14, 22]],
            [
                [4649, 6796],
                [25, 7],
                [22, 39],
                [14, 16],
                [23, 48],
                [-7, 71],
                [11, 50],
                [4, 30],
                [18, 39],
                [27, 26],
                [21, 24],
                [19, 60],
                [8, 35],
                [21, 0],
                [16, -25],
                [27, 4],
                [29, -12],
                [12, -1],
            ],
            [[5783, 7801], [-5, 27], [3, 24], [-1, 25], [-16, 35], [-9, 24], [-8, 17], [-9, 6]],
            [
                [5738, 7959],
                [7, 8],
                [18, 6],
                [21, -18],
                [11, -2],
                [13, -16],
                [-2, -19],
                [10, -10],
                [4, -24],
                [10, -14],
                [-2, -9],
                [5, -6],
                [-7, -4],
                [-17, 2],
                [-3, 8],
                [-5, -5],
                [2, -10],
                [-8, -19],
                [-5, -19],
                [-7, -7],
            ],
            [
                [6375, 4464],
                [7, -25],
                [7, -38],
                [5, -69],
                [7, -27],
                [-3, -27],
                [-5, -17],
                [-9, 33],
                [-5, -17],
                [5, -42],
                [-3, -25],
                [-7, -13],
                [-2, -49],
                [-11, -67],
                [-14, -79],
                [-17, -109],
                [-10, -80],
                [-13, -67],
                [-23, -14],
                [-24, -24],
                [-16, 14],
                [-22, 21],
                [-7, 30],
                [-2, 51],
                [-10, 46],
                [-3, 42],
                [5, 41],
                [13, 10],
                [0, 19],
                [13, 44],
                [3, 37],
                [-6, 27],
                [-6, 36],
                [-2, 53],
                [10, 33],
                [4, 36],
                [13, 2],
                [16, 12],
                [10, 11],
                [12, 0],
                [16, 33],
                [23, 36],
                [8, 29],
                [-3, 24],
                [11, -7],
                [16, 40],
                [0, 35],
                [9, 26],
                [10, -25],
            ],
            [
                [1746, 7055],
                [31, 5],
                [36, 6],
                [-3, -11],
                [42, -28],
                [63, -41],
                [56, 1],
                [22, 0],
                [0, 24],
                [48, -1],
                [10, -20],
                [14, -18],
                [17, -25],
                [9, -31],
                [7, -31],
                [14, -18],
                [23, -17],
                [18, 46],
                [22, 1],
                [20, -23],
                [14, -40],
                [9, -33],
                [17, -33],
                [6, -41],
                [8, -27],
                [21, -17],
                [20, -13],
                [11, 2],
            ],
            [
                [2301, 6672],
                [-11, -51],
                [-5, -41],
                [-2, -78],
                [-2, -28],
                [4, -31],
                [9, -28],
                [6, -45],
                [18, -43],
                [6, -33],
                [11, -28],
                [30, -15],
                [11, -24],
                [25, 16],
                [21, 6],
                [21, 10],
                [17, 10],
                [18, 23],
                [6, 34],
                [3, 48],
                [5, 17],
                [18, 15],
                [30, 14],
                [24, -2],
                [17, 5],
                [7, -13],
                [-1, -27],
                [-15, -35],
                [-7, -35],
                [5, -10],
                [-4, -25],
                [-7, -45],
                [-7, 15],
                [-6, -1],
            ],
            [[5598, 7615], [10, 3], [13, 1]],
            [[5118, 6285], [0, -133], [-16, -38], [-2, -36], [-25, -9], [-38, -5], [-10, -20], [-18, -2]],
            [[4679, 5898], [1, 18], [-2, 23], [-10, 16], [-6, 33], [-1, 36]],
            [
                [4661, 6024],
                [9, 10],
                [5, 34],
                [9, 1],
                [19, -16],
                [16, 12],
                [11, -4],
                [4, 13],
                [111, 1],
                [6, 40],
                [-4, 7],
                [-14, 249],
                [-13, 248],
                [42, 1],
            ],
            [
                [7780, 6358],
                [-16, -14],
                [-16, -25],
                [-20, -2],
                [-13, -62],
                [-11, -11],
                [13, -50],
                [18, -42],
                [11, -38],
                [-10, -51],
                [-10, -10],
                [7, -29],
                [18, -46],
                [4, -32],
                [-1, -27],
                [11, -52],
                [-15, -54],
                [-14, -59],
            ],
            [[5533, 7688], [7, -10], [4, -8], [9, -6], [11, -12], [-2, -5]],
            [
                [7436, 8021],
                [30, 10],
                [53, 49],
                [42, 27],
                [24, -17],
                [29, -1],
                [19, -27],
                [27, -2],
                [40, -15],
                [27, 40],
                [-11, 34],
                [29, 60],
                [31, -24],
                [25, -7],
                [33, -14],
                [5, -43],
                [40, -25],
                [26, 11],
                [35, 7],
                [28, -7],
                [27, -28],
                [17, -29],
                [26, 0],
                [35, -9],
                [25, 14],
                [37, 10],
                [40, 40],
                [17, -6],
                [15, -19],
                [33, 5],
            ],
            [[5911, 3642], [-21, 1]],
            [[5890, 3643], [-3, 25], [-4, 26]],
            [[5883, 3694], [-2, 21], [5, 64], [-7, 41], [-14, 81]],
            [
                [5865, 3901],
                [30, 65],
                [7, 42],
                [4, 5],
                [3, 34],
                [-4, 17],
                [1, 43],
                [5, 40],
                [0, 73],
                [-14, 18],
                [-13, 4],
                [-6, 14],
                [-13, 13],
                [-23, -1],
                [-2, 21],
            ],
            [[5840, 4289], [-3, 41], [85, 47]],
            [
                [5922, 4377],
                [16, -27],
                [7, 5],
                [11, -14],
                [2, -24],
                [-6, -26],
                [2, -41],
                [18, -36],
                [9, 40],
                [12, 13],
                [-3, 74],
                [-11, 41],
                [-10, 19],
                [-10, -1],
                [-8, 75],
                [8, 44],
            ],
            [[5959, 4519], [21, 4], [33, -16], [7, 7], [20, 2], [10, 17], [16, -1], [31, 22], [22, 34]],
            [[4525, 6391], [6, 19], [109, 0], [-5, 83], [6, 30], [26, 5], [0, 147], [91, -3], [0, 87]],
            [[4661, 6024], [-18, 39], [-17, 43], [-19, 15], [-13, 17], [-15, -1], [-14, -12], [-14, 5], [-9, -19]],
            [[5922, 4377], [-15, 15], [8, 54], [9, 20], [-5, 48], [5, 46], [5, 16], [-7, 49], [-13, 25]],
            [[5909, 4650], [27, -10], [6, -16], [9, -27], [8, -78]],
            [[7779, 5554], [5, 10], [22, -25], [3, -29], [18, 7], [9, 23]],
            [[5644, 4173], [23, 13], [18, -3], [11, -13], [0, -5]],
            [[5552, 3756], [0, -213], [-25, -29], [-15, -5], [-18, 11], [-12, 5], [-5, 24], [-11, 16], [-13, -29]],
            [
                [9604, 3968],
                [22, -36],
                [15, -26],
                [-11, -14],
                [-15, 16],
                [-20, 26],
                [-18, 30],
                [-18, 41],
                [-4, 19],
                [12, -1],
                [15, -19],
                [13, -20],
                [9, -16],
            ],
            [
                [5411, 6498],
                [7, -89],
                [11, -15],
                [0, -18],
                [12, -20],
                [-6, -25],
                [-11, -117],
                [-1, -75],
                [-36, -54],
                [-12, -76],
                [12, -21],
                [0, -37],
                [17, -1],
                [-2, -28],
            ],
            [
                [5393, 5901],
                [-5, -1],
                [-19, 63],
                [-7, 2],
                [-21, -32],
                [-22, 16],
                [-15, 4],
                [-8, -8],
                [-16, 2],
                [-16, -25],
                [-15, -1],
                [-33, 29],
                [-13, -14],
                [-15, 1],
                [-10, 22],
                [-28, 21],
                [-30, -6],
                [-7, -13],
                [-4, -33],
                [-8, -23],
                [-2, -52],
            ],
            [[5863, 9187], [-47, -23], [-23, -5]],
            [
                [5572, 9161],
                [-17, -2],
                [-4, -38],
                [-52, 9],
                [-8, -32],
                [-26, 0],
                [-19, -41],
                [-27, -63],
                [-43, -81],
                [10, -20],
                [-10, -23],
                [-28, 1],
                [-18, -54],
                [2, -76],
                [18, -30],
                [-9, -67],
                [-23, -40],
                [-13, -33],
            ],
            [[6474, 6141], [-9, 40], [-22, 95]],
            [[6443, 6276], [84, 58], [18, 115], [-13, 41]],
            [[5545, 8316], [34, -7], [51, 1]],
            [[5652, 8152], [14, -50], [-3, -16], [-13, -7], [-26, -48], [8, -26], [-6, 3]],
            [[5626, 8008], [-27, 23], [-20, -9], [-13, 6], [-16, -12], [-14, 20], [-12, -7], [-1, 3]],
            [[3158, 6248], [14, -5], [5, -11], [-7, -15], [-21, 0], [-16, -2], [-2, 25], [4, 8], [23, 0]],
            [[8628, 7623], [3, -10]],
            [[6426, 6600], [-7, -4], [-9, 11]],
            [[5783, 7801], [13, -10], [13, 9], [12, -10]],
            [[5628, 7729], [-5, 10], [7, 10], [-7, 7], [-9, -13], [-16, 17], [-2, 24], [-17, 13], [-3, 18], [-15, 23]],
            [[5630, 7939], [12, 12], [17, -6], [18, 0], [13, -14], [9, 9], [21, 5], [7, 14], [11, 0]],
            [
                [6061, 7894],
                [1, 26],
                [14, 16],
                [27, 4],
                [4, 19],
                [-6, 32],
                [11, 30],
                [0, 17],
                [-41, 19],
                [-16, -1],
                [-17, 27],
                [-22, -9],
                [-35, 20],
                [1, 12],
                [-10, 25],
                [-22, 2],
                [-3, 18],
                [7, 12],
                [-18, 33],
                [-28, -6],
                [-9, 3],
                [-7, -13],
                [-10, 2],
            ],
            [[5863, 9187], [28, 20], [46, -34], [76, -14], [105, -65], [21, -28], [2, -38], [-31, -30], [-45, -16], [-124, 44], [-20, -7], [45, -42]],
            [[5966, 8977], [2, -27], [2, -59]],
            [[5970, 8891], [35, -17], [22, -15], [4, 28]],
            [[6031, 8887], [-17, 24], [18, 22]],
            [[6920, 9133], [-28, 31], [-1, 12]],
            [[8147, 9571], [22, -22], [-7, -29]],
            [[5821, 5105], [6, -6], [17, 18]],
            [[5844, 5117], [11, -33], [-2, -34], [-8, -7]],
            [
                [6443, 6276],
                [-80, -22],
                [-26, -26],
                [-20, -60],
                [-13, -10],
                [-7, 19],
                [-10, -3],
                [-27, 6],
                [-5, 6],
                [-32, -1],
                [-8, -6],
                [-11, 15],
                [-7, -28],
                [2, -24],
                [-12, -19],
            ],
            [
                [5634, 5824],
                [0, 14],
                [-10, 16],
                [0, 34],
                [-6, 22],
                [-10, -3],
                [3, 21],
                [7, 24],
                [-3, 24],
                [9, 17],
                [-6, 14],
                [8, 36],
                [13, 42],
                [23, -4],
                [-1, 228],
            ],
            [[5942, 5734], [0, -7]],
            [
                [5942, 5727],
                [-4, 1],
                [1, 29],
                [-3, 20],
                [-15, 22],
                [-3, 42],
                [3, 42],
                [-13, 4],
                [-1, -13],
                [-17, -3],
                [7, -16],
                [2, -35],
                [-15, -32],
                [-14, -41],
                [-14, -6],
                [-24, 34],
                [-10, -12],
                [-3, -17],
                [-14, -11],
                [-1, -12],
                [-28, 0],
                [-4, 12],
                [-20, 2],
                [-10, -10],
                [-8, 5],
                [-14, 34],
                [-5, 15],
                [-20, -7],
                [-7, -27],
                [-7, -52],
                [-10, -10],
                [-9, -7],
            ],
            [[5662, 5678], [-2, 3]],
            [[5943, 5427], [-17, -27], [-19, 0], [-22, -13], [-18, 13], [-12, -16]],
            [[5681, 5656], [-19, 22]],
            [[5942, 5727], [1, -45]],
            [[2541, 5940], [-10, 6], [-11, 11]],
            [[6359, 5839], [-1, -1], [0, -24], [0, -58], [0, -30], [-12, -35], [-20, -48]],
            [[3488, 5425], [11, -35], [-5, -24]],
            [[3494, 5366], [-2, -27], [-7, -24]],
            [[5626, 8008], [-8, -15], [-6, -23]],
            [[5890, 3643], [-6, -26], [-16, -6], [-17, 31], [0, 20], [8, 22], [2, 16], [8, 4], [14, -10]],
            [[6003, 7245], [7, 12], [8, 13], [1, 32], [10, -11], [30, 16], [15, -11], [23, 0], [32, 22], [15, -1], [31, 9]],
            [[6883, 7321], [16, 58], [-6, 43], [-21, 14], [7, 25], [24, -3], [13, 32], [9, 37], [37, 14], [-6, -27], [4, -16], [11, 1]],
            [
                [6554, 7561],
                [31, 0],
                [-5, 29],
                [24, 20],
                [23, 34],
                [38, -31],
                [3, -46],
                [10, -11],
                [30, 2],
                [10, -10],
                [13, -60],
                [32, -39],
                [18, -27],
                [29, -29],
                [37, -24],
                [0, -36],
            ],
            [[3286, 5802], [16, 7], [6, -2], [-1, -43], [-24, -6], [-5, 5], [8, 16], [0, 23]],
            [[8381, 6587], [-16, -93], [-12, -47], [-15, 49], [-3, 42], [16, 57], [23, 44], [12, -18], [-5, -34]],
            [[5909, 4650], [-16, 18], [-18, 9], [-11, 10], [-11, 15]],
            [[5844, 5117], [10, 7], [30, -1], [57, 4]],
            [[3052, 7697], [-15, -34], [-5, -13]],
            [[2952, 7539], [40, 11], [9, -11]],
            [[2896, 7366], [-14, 22], [-4, 48]],
            [[2522, 6928], [-11, -9], [5, -16]],
            [[2316, 6812], [-15, -28], [-6, -25]],
            [
                [1746, 7055],
                [-5, 30],
                [-18, 33],
                [-13, 7],
                [-3, 16],
                [-15, 3],
                [-10, 16],
                [-26, 6],
                [-7, 9],
                [-4, 31],
                [-27, 58],
                [-23, 80],
                [1, 14],
                [-12, 19],
                [-22, 48],
                [-3, 47],
                [-15, 31],
                [6, 48],
                [-1, 49],
                [-9, 45],
                [11, 54],
            ],
            [[1551, 7699], [3, 52], [4, 52]],
            [[1558, 7803], [-5, 78], [-9, 49], [-8, 27], [3, 11], [40, -20], [15, -54], [7, 15], [-4, 47], [-10, 48]],
            [[5816, 3910], [12, -1], [13, -9], [10, 6], [14, -5]],
            [[5840, 4289], [-21, -8], [-16, -23], [-3, -20], [-10, -4], [-24, -48], [-16, -37], [-9, -1], [-9, 6], [-31, 7]],
        ],
    },
    preserve_domain: dict = {},
    scales: dict = {},
    scales_metadata: dict = {"color": {"dimension": "color"}, "projection": {"dimension": "geo"}},
    selected: ndarray = None,
    selected_style: dict = {},
    selected_styles: dict = {"selected_fill": "Red", "selected_stroke": None, "selected_stroke_width": 2.0},
    stroke_color: str = None,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unselected_style: dict = {},
    visible: bool = True,
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_color: typing.Callable[[dict], Any] = None,
    on_colors: typing.Callable[[dict], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_hover_highlight: typing.Callable[[bool], Any] = None,
    on_hovered_styles: typing.Callable[[dict], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_map_data: typing.Callable[[dict], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_selected_styles: typing.Callable[[dict], Any] = None,
    on_stroke_color: typing.Callable[[str], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.marks.Map]:
    """Map mark.

    Attributes
    ----------
    colors: Dict (default: {})
        default colors for items of the map when no color data is passed.
        The dictionary should be indexed by the id of the element and have
        the corresponding colors as values. The key `default_color`
        controls the items for which no color is specified.
    selected_styles: Dict (default: {'selected_fill': 'Red',
    'selected_stroke': None, 'selected_stroke_width': 2.0})
        Dictionary containing the styles for selected subunits
    hovered_styles: Dict (default: {'hovered_fill': 'Orange',
    'hovered_stroke': None, 'hovered_stroke_width': 2.0})
        Dictionary containing the styles for hovered subunits
    hover_highlight: bool (default: True)
        boolean to control if the map should be aware of which country is being
        hovered on.
    map_data: dict (default: topo_load("map_data/WorldMap.json"))
        a topojson-formatted dictionary with the objects to map under the key
        'subunits'.

    Data Attributes

    color: Dict or None (default: None)
        dictionary containing the data associated with every country for the
        color scale


    """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        color=color,
        colors=colors,
        display_legend=display_legend,
        enable_hover=enable_hover,
        hover_highlight=hover_highlight,
        hovered_styles=hovered_styles,
        interactions=interactions,
        labels=labels,
        map_data=map_data,
        preserve_domain=preserve_domain,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        selected_styles=selected_styles,
        stroke_color=stroke_color,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unselected_style=unselected_style,
        visible=visible,
        on_apply_clip=on_apply_clip,
        on_color=on_color,
        on_colors=on_colors,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_hover_highlight=on_hover_highlight,
        on_hovered_styles=on_hovered_styles,
        on_interactions=on_interactions,
        on_labels=on_labels,
        on_map_data=on_map_data,
        on_preserve_domain=on_preserve_domain,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_selected_styles=on_selected_styles,
        on_stroke_color=on_stroke_color,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
    )
    widget_cls = bqplot.marks.Map
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Mark(
    apply_clip: bool = True,
    display_legend: bool = False,
    enable_hover: bool = True,
    interactions: dict = {"hover": "tooltip"},
    labels: list = [],
    preserve_domain: dict = {},
    scales: dict = {},
    scales_metadata: dict = {},
    selected: ndarray = None,
    selected_style: dict = {},
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unselected_style: dict = {},
    visible: bool = True,
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.marks.Mark]:
    """The base mark class.

    Traitlet mark attributes may be decorated with metadata.

    **Data Attribute Decoration**

    Data attributes are decorated with the following values:

    scaled: bool
        Indicates whether the considered attribute is a data attribute which
        must be associated with a scale in order to be taken into account.
    rtype: string
        Range type of the associated scale.
    atype: string
        Key in bqplot's axis registry of the recommended axis type to represent
        this scale. When not specified, the default is 'bqplot.Axis'.

    Attributes
    ----------
    display_name: string
        Holds a user-friendly name for the trait attribute.
    mark_types: dict (class-level attribute)
        A registry of existing mark types.
    scales: Dict of scales (default: {})
        A dictionary of scales holding scales for each data attribute.
        - If a mark holds a scaled attribute named 'x', the scales dictionary
        must have a corresponding scale for the key 'x'.
        - The scale's range type should be equal to the scaled attribute's
        range type (rtype).
    scales_metadata: Dict (default: {})
        A dictionary of dictionaries holding metadata on the way scales are
        used by the mark. For example, a linear scale may be used to count
        pixels horizontally or vertically. The content of this dictionary
        may change dynamically. It is an instance-level attribute.
    preserve_domain: dict (default: {})
        Indicates if this mark affects the domain(s) of the specified scale(s).
        The keys of this dictionary are the same as the ones of the "scales"
        attribute, and values are boolean. If a key is missing, it is
        considered as False.
    display_legend: bool (default: False)
        Display toggle for the mark legend in the general figure legend
    labels: list of unicode strings (default: [])
        Labels of the items of the mark. This attribute has different meanings
        depending on the type of mark.
    apply_clip: bool (default: True)
        Indicates whether the items that are beyond the limits of the chart
        should be clipped.
    visible: bool (default: True)
        Visibility toggle for the mark.
    selected_style: dict (default: {})
        CSS style to be applied to selected items in the mark.
    unselected_style: dict (default: {})
        CSS style to be applied to items that are not selected in the mark,
        when a selection exists.
    selected: list of integers or None (default: None)
        Indices of the selected items in the mark.
    tooltip: DOMWidget or None (default: None)
        Widget to be displayed as tooltip when elements of the scatter are
        hovered on
    tooltip_style: Dictionary (default: {'opacity': 0.9})
        Styles to be applied to the tooltip widget
    enable_hover: Bool (default: True)
        Boolean attribute to control the hover interaction for the scatter. If
        this is false, the on_hover custom mssg is not sent back to the python
        side
    interactions: Dictionary (default: {'hover': 'tooltip'})
        Dictionary listing the different interactions for each mark. The key is
        the event which triggers the interaction and the value is the kind of
        interactions. Keys and values can only take strings from separate enums
        for each mark.
    tooltip_location : {'mouse', 'center'} (default: 'mouse')
        Enum specifying the location of the tooltip. 'mouse' places the tooltip
        at the location of the mouse when the tooltip is activated and 'center'
        places the tooltip at the center of the figure. If tooltip is linked to
        a click event, 'mouse' places the tooltip at the location of the click
        that triggered the tooltip to be visible.


    """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        display_legend=display_legend,
        enable_hover=enable_hover,
        interactions=interactions,
        labels=labels,
        preserve_domain=preserve_domain,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unselected_style=unselected_style,
        visible=visible,
        on_apply_clip=on_apply_clip,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_interactions=on_interactions,
        on_labels=on_labels,
        on_preserve_domain=on_preserve_domain,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
    )
    widget_cls = bqplot.marks.Mark
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Mercator(
    allow_padding: bool = True,
    center: tuple = (0, 60),
    reverse: bool = False,
    rotate: tuple = (0, 0),
    scale_factor: float = 190,
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_center: typing.Callable[[tuple], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
    on_rotate: typing.Callable[[tuple], Any] = None,
    on_scale_factor: typing.Callable[[float], Any] = None,
) -> Element[bqplot.scales.Mercator]:
    """A geographical projection scale commonly used for world maps.

    The Mercator projection is a cylindrical map projection which ensures that
    any course of constant bearing is a straight line.

    Attributes
    ----------
    scale_factor: float (default: 190)
        Specifies the scale value for the projection
    center: tuple (default: (0, 60))
        Specifies the longitude and latitude where the map is centered.
    rotate: tuple (default: (0, 0))
        Degree of rotation in each axis.
    rtype: (Number, Number) (class-level attribute)
        This attribute should not be modified. The range type of a geo
        scale is a tuple.
    dtype: type (class-level attribute)
        the associated data type / domain type


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        center=center,
        reverse=reverse,
        rotate=rotate,
        scale_factor=scale_factor,
        on_allow_padding=on_allow_padding,
        on_center=on_center,
        on_reverse=on_reverse,
        on_rotate=on_rotate,
        on_scale_factor=on_scale_factor,
    )
    widget_cls = bqplot.scales.Mercator
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def OHLC(
    apply_clip: bool = True,
    colors: list = ["green", "red"],
    display_legend: bool = False,
    enable_hover: bool = True,
    format: str = "ohlc",
    interactions: dict = {"hover": "tooltip"},
    labels: list = [],
    marker: str = "candle",
    opacities: list = [],
    preserve_domain: dict = {},
    scales: dict = {},
    scales_metadata: dict = {"x": {"orientation": "horizontal", "dimension": "x"}, "y": {"orientation": "vertical", "dimension": "y"}},
    selected: ndarray = None,
    selected_style: dict = {},
    stroke: str = None,
    stroke_width: float = 1.0,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unselected_style: dict = {},
    visible: bool = True,
    x: ndarray = np.array([]),
    y: ndarray = np.array([[]]),
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_format: typing.Callable[[str], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_marker: typing.Callable[[str], Any] = None,
    on_opacities: typing.Callable[[list], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_stroke: typing.Callable[[str], Any] = None,
    on_stroke_width: typing.Callable[[float], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_x: typing.Callable[[ndarray], Any] = None,
    on_y: typing.Callable[[ndarray], Any] = None,
) -> Element[bqplot.marks.OHLC]:
    """Open/High/Low/Close marks.

    Attributes
    ----------
    icon: string (class-level attribute)
        font-awesome icon for that mark
    name: string (class-level attribute)
        user-friendly name of the mark
    marker: {'candle', 'bar'}
        marker type
    stroke: color (default: None)
        stroke color of the marker
    stroke_width: float (default: 1.0)
        stroke width of the marker
    colors: List of colors (default: ['limegreen', 'red'])
        fill colors for the markers (up/down)
    opacities: list of floats (default: [])
        Opacities for the markers of the OHLC mark. Defaults to 1 when
        the list is too short, or the element of the list is set to None.
    format: string (default: 'ohlc')
        description of y data being passed
        supports all permutations of the strings 'ohlc', 'oc', and 'hl'

    Data Attributes

    x: numpy.ndarray
        abscissas of the data points (1d array)
    y: numpy.ndarrays
        Open/High/Low/Close ordinates of the data points (2d array)

    Notes
    -----
    The fields which can be passed to the default tooltip are:
        x: the x value associated with the bar/candle
        open: open value for the bar/candle
        high: high value for the bar/candle
        low: low value for the bar/candle
        close: close value for the bar/candle
        index: index of the bar/candle being hovered on


    """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        colors=colors,
        display_legend=display_legend,
        enable_hover=enable_hover,
        format=format,
        interactions=interactions,
        labels=labels,
        marker=marker,
        opacities=opacities,
        preserve_domain=preserve_domain,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        stroke=stroke,
        stroke_width=stroke_width,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unselected_style=unselected_style,
        visible=visible,
        x=x,
        y=y,
        on_apply_clip=on_apply_clip,
        on_colors=on_colors,
        on_display_legend=on_display_legend,
        on_enable_hover=on_enable_hover,
        on_format=on_format,
        on_interactions=on_interactions,
        on_labels=on_labels,
        on_marker=on_marker,
        on_opacities=on_opacities,
        on_preserve_domain=on_preserve_domain,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_stroke=on_stroke,
        on_stroke_width=on_stroke_width,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unselected_style=on_unselected_style,
        on_visible=on_visible,
        on_x=on_x,
        on_y=on_y,
    )
    widget_cls = bqplot.marks.OHLC
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def OrdinalColorScale(
    allow_padding: bool = True,
    colors: list = [],
    domain: list = [],
    extrapolation: str = "constant",
    max: float = None,
    mid: float = None,
    min: float = None,
    reverse: bool = False,
    scale_type: str = "linear",
    scheme: str = "RdYlGn",
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_domain: typing.Callable[[list], Any] = None,
    on_extrapolation: typing.Callable[[str], Any] = None,
    on_max: typing.Callable[[float], Any] = None,
    on_mid: typing.Callable[[float], Any] = None,
    on_min: typing.Callable[[float], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
    on_scale_type: typing.Callable[[str], Any] = None,
    on_scheme: typing.Callable[[str], Any] = None,
) -> Element[bqplot.scales.OrdinalColorScale]:
    """An ordinal color scale.

    A mapping from a discrete set of values to colors.

    Attributes
    ----------
    domain: list (default: [])
        The discrete values mapped by the ordinal scales.
    rtype: string (class-level attribute)
        This attribute should not be modified by the user.
        The range type of a color scale is 'color'.
    dtype: type (class-level attribute)
        the associated data type / domain type


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        colors=colors,
        domain=domain,
        extrapolation=extrapolation,
        max=max,
        mid=mid,
        min=min,
        reverse=reverse,
        scale_type=scale_type,
        scheme=scheme,
        on_allow_padding=on_allow_padding,
        on_colors=on_colors,
        on_domain=on_domain,
        on_extrapolation=on_extrapolation,
        on_max=on_max,
        on_mid=on_mid,
        on_min=on_min,
        on_reverse=on_reverse,
        on_scale_type=on_scale_type,
        on_scheme=on_scheme,
    )
    widget_cls = bqplot.scales.OrdinalColorScale
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def OrdinalScale(
    allow_padding: bool = True,
    domain: list = [],
    reverse: bool = False,
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_domain: typing.Callable[[list], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.scales.OrdinalScale]:
    """An ordinal scale.

    A mapping from a discrete set of values to a numerical range.

    Attributes
    ----------
    domain: list (default: [])
        The discrete values mapped by the ordinal scale
    rtype: string (class-level attribute)
        This attribute should not be modified by the user.
        The range type of a linear scale is numerical.
    dtype: type (class-level attribute)
        the associated data type / domain type


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding, domain=domain, reverse=reverse, on_allow_padding=on_allow_padding, on_domain=on_domain, on_reverse=on_reverse
    )
    widget_cls = bqplot.scales.OrdinalScale
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Orthographic(
    allow_padding: bool = True,
    center: tuple = (0, 60),
    clip_angle: float = 90.0,
    precision: float = 0.1,
    reverse: bool = False,
    rotate: tuple = (0, 0),
    scale_factor: float = 145.0,
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_center: typing.Callable[[tuple], Any] = None,
    on_clip_angle: typing.Callable[[float], Any] = None,
    on_precision: typing.Callable[[float], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
    on_rotate: typing.Callable[[tuple], Any] = None,
    on_scale_factor: typing.Callable[[float], Any] = None,
) -> Element[bqplot.scales.Orthographic]:
    """A perspective projection that depicts a hemisphere as it appears from
    outer space.

    The projection is neither equal-area nor conformal.

    Attributes
    ----------
    scale_factor: float (default: 145)
       Specifies the scale value for the projection
    center: tuple (default: (0, 60))
        Specifies the longitude and latitude where the map is centered.
    rotate: tuple (default: (96, 0))
        Degree of rotation in each axis.
    clip_angle: float (default: 90.)
        Specifies the clipping circle radius to the specified angle in degrees.
    precision: float (default: 0.1)
        Specifies the threshold for the projections adaptive resampling to the
        specified value in pixels.


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        center=center,
        clip_angle=clip_angle,
        precision=precision,
        reverse=reverse,
        rotate=rotate,
        scale_factor=scale_factor,
        on_allow_padding=on_allow_padding,
        on_center=on_center,
        on_clip_angle=on_clip_angle,
        on_precision=on_precision,
        on_reverse=on_reverse,
        on_rotate=on_rotate,
        on_scale_factor=on_scale_factor,
    )
    widget_cls = bqplot.scales.Orthographic
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def PanZoom(
    allow_pan: bool = True,
    allow_zoom: bool = True,
    scales: dict = {},
    on_allow_pan: typing.Callable[[bool], Any] = None,
    on_allow_zoom: typing.Callable[[bool], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
) -> Element[bqplot.interacts.PanZoom]:
    """An interaction to pan and zoom wrt scales.

    Attributes
    ----------
    allow_pan: bool (default: True)
        Toggle the ability to pan.
    allow_zoom: bool (default: True)
        Toggle the ability to zoom.
    scales: Dictionary of lists of Scales (default: {})
        Dictionary with keys such as 'x' and 'y' and values being the scales in
        the corresponding direction (dimensions) which should be panned or
        zoomed.


    """

    kwargs: Dict[Any, Any] = dict(
        allow_pan=allow_pan, allow_zoom=allow_zoom, scales=scales, on_allow_pan=on_allow_pan, on_allow_zoom=on_allow_zoom, on_scales=on_scales
    )
    widget_cls = bqplot.interacts.PanZoom
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Pie(
    apply_clip: bool = True,
    color: ndarray = None,
    colors: list = ["#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd", "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"],
    display_labels: str = "inside",
    display_legend: bool = False,
    display_values: bool = False,
    enable_hover: bool = True,
    end_angle: float = 360.0,
    font_size: str = "12px",
    font_weight: str = "normal",
    inner_radius: float = 0.1,
    interactions: dict = {"hover": "tooltip"},
    label_color: str = None,
    labels: list = [],
    opacities: list = [],
    preserve_domain: dict = {},
    radius: float = 180.0,
    scales: dict = {},
    scales_metadata: dict = {"color": {"dimension": "color"}},
    selected: ndarray = None,
    selected_style: dict = {},
    sizes: ndarray = np.array([]),
    sort: bool = False,
    start_angle: float = 0.0,
    stroke: str = None,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unselected_style: dict = {},
    values_format: str = ".1f",
    visible: bool = True,
    x: typing.Union[float, datetime.datetime, str] = traitlets.Undefined,
    y: typing.Union[float, datetime.datetime, str] = traitlets.Undefined,
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_color: typing.Callable[[ndarray], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_display_labels: typing.Callable[[str], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_display_values: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_end_angle: typing.Callable[[float], Any] = None,
    on_font_size: typing.Callable[[str], Any] = None,
    on_font_weight: typing.Callable[[str], Any] = None,
    on_inner_radius: typing.Callable[[float], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_label_color: typing.Callable[[str], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_opacities: typing.Callable[[list], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_radius: typing.Callable[[float], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_sizes: typing.Callable[[ndarray], Any] = None,
    on_sort: typing.Callable[[bool], Any] = None,
    on_start_angle: typing.Callable[[float], Any] = None,
    on_stroke: typing.Callable[[str], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_values_format: typing.Callable[[str], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_x: typing.Callable[[typing.Union[float, datetime.datetime, str]], Any] = None,
    on_y: typing.Callable[[typing.Union[float, datetime.datetime, str]], Any] = None,
) -> Element[bqplot.marks.Pie]:
    """Piechart mark.

    Attributes
    ----------
    colors: list of colors (default: CATEGORY10)
        list of colors for the slices.
    stroke: color (default: 'white')
        stroke color for the marker
    opacities: list of floats (default: [])
        Opacities for the slices of the Pie mark. Defaults to 1 when the list
        is too short, or the element of the list is set to None.
    sort: bool (default: False)
        sort the pie slices by descending sizes
    x: Float (default: 0.5) or Date
        horizontal position of the pie center, in data coordinates or in figure
        coordinates
    y: Float (default: 0.5)
        vertical y position of the pie center, in data coordinates or in figure
        coordinates
    radius: Float
        radius of the pie, in pixels
    inner_radius: Float
        inner radius of the pie, in pixels
    start_angle: Float (default: 0.0)
        start angle of the pie (from top), in degrees
    end_angle: Float (default: 360.0)
        end angle of the pie (from top), in degrees
    display_labels: {'none', 'inside', 'outside'} (default: 'inside')
        label display options
    display_values: bool (default: False)
        if True show values along with labels
    values_format: string (default: '.2f')
        format for displaying values
    label_color: Color or None (default: None)
        color of the labels
    font_size: string (default: '14px')
        label font size in px, em or ex
    font_weight: {'bold', 'normal', 'bolder'} (default: 'normal')
        label font weight

    Data Attributes

    sizes: numpy.ndarray (default: [])
        proportions of the pie slices
    color: numpy.ndarray or None (default: None)
        color of the data points. Defaults to colors when not provided.

    Notes
    -----
    The fields which can be passed to the default tooltip are:
        : the x value associated with the bar/candle
        open: open value for the bar/candle
        high: high value for the bar/candle
        low: low value for the bar/candle
        close: close value for the bar/candle
        index: index of the bar/candle being hovered on


    """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        color=color,
        colors=colors,
        display_labels=display_labels,
        display_legend=display_legend,
        display_values=display_values,
        enable_hover=enable_hover,
        end_angle=end_angle,
        font_size=font_size,
        font_weight=font_weight,
        inner_radius=inner_radius,
        interactions=interactions,
        label_color=label_color,
        labels=labels,
        opacities=opacities,
        preserve_domain=preserve_domain,
        radius=radius,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        sizes=sizes,
        sort=sort,
        start_angle=start_angle,
        stroke=stroke,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unselected_style=unselected_style,
        values_format=values_format,
        visible=visible,
        x=x,
        y=y,
        on_apply_clip=on_apply_clip,
        on_color=on_color,
        on_colors=on_colors,
        on_display_labels=on_display_labels,
        on_display_legend=on_display_legend,
        on_display_values=on_display_values,
        on_enable_hover=on_enable_hover,
        on_end_angle=on_end_angle,
        on_font_size=on_font_size,
        on_font_weight=on_font_weight,
        on_inner_radius=on_inner_radius,
        on_interactions=on_interactions,
        on_label_color=on_label_color,
        on_labels=on_labels,
        on_opacities=on_opacities,
        on_preserve_domain=on_preserve_domain,
        on_radius=on_radius,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_sizes=on_sizes,
        on_sort=on_sort,
        on_start_angle=on_start_angle,
        on_stroke=on_stroke,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unselected_style=on_unselected_style,
        on_values_format=on_values_format,
        on_visible=on_visible,
        on_x=on_x,
        on_y=on_y,
    )
    widget_cls = bqplot.marks.Pie
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Scale(
    allow_padding: bool = True,
    reverse: bool = False,
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.scales.Scale]:
    """The base scale class.

    Scale objects represent a mapping between data (the domain) and a visual
    quantity (The range).

    Attributes
    ----------
    scale_types: dict (class-level attribute)
        A registry of existing scale types.
    domain_class: type (default: Float)
        traitlet type used to validate values in of the domain of the scale.
    reverse: bool (default: False)
        whether the scale should be reversed.
    allow_padding: bool (default: True)
        indicates whether figures are allowed to add data padding to this scale
        or not.
    precedence: int (class-level attribute)
        attribute used to determine which scale takes precedence in cases when
        two or more scales have the same rtype and dtype.


    """

    kwargs: Dict[Any, Any] = dict(allow_padding=allow_padding, reverse=reverse, on_allow_padding=on_allow_padding, on_reverse=on_reverse)
    widget_cls = bqplot.scales.Scale
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Scatter(
    apply_clip: bool = True,
    color: ndarray = None,
    colors: list = ["steelblue"],
    default_size: int = 64,
    default_skew: float = 0.5,
    display_legend: bool = False,
    display_names: bool = True,
    drag_color: str = None,
    drag_size: float = 5.0,
    enable_delete: bool = False,
    enable_hover: bool = True,
    enable_move: bool = False,
    fill: bool = True,
    hovered_point: int = None,
    hovered_style: dict = {},
    interactions: dict = {"hover": "tooltip"},
    label_display_horizontal_offset: float = 0.0,
    label_display_vertical_offset: float = 0.0,
    labels: list = [],
    marker: str = "circle",
    names: ndarray = None,
    names_unique: bool = True,
    opacities: ndarray = np.array([1.0]),
    opacity: ndarray = None,
    preserve_domain: dict = {},
    restrict_x: bool = False,
    restrict_y: bool = False,
    rotation: ndarray = None,
    scales: dict = {},
    scales_metadata: dict = {
        "x": {"orientation": "horizontal", "dimension": "x"},
        "y": {"orientation": "vertical", "dimension": "y"},
        "color": {"dimension": "color"},
        "size": {"dimension": "size"},
        "opacity": {"dimension": "opacity"},
        "rotation": {"dimension": "rotation"},
        "skew": {"dimension": "skew"},
    },
    selected: ndarray = None,
    selected_style: dict = {},
    size: ndarray = None,
    skew: ndarray = None,
    stroke: str = None,
    stroke_width: float = 1.5,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unhovered_style: dict = {},
    unselected_style: dict = {},
    update_on_move: bool = False,
    visible: bool = True,
    x: ndarray = np.array([]),
    y: ndarray = np.array([]),
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_color: typing.Callable[[ndarray], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_default_size: typing.Callable[[int], Any] = None,
    on_default_skew: typing.Callable[[float], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_display_names: typing.Callable[[bool], Any] = None,
    on_drag_color: typing.Callable[[str], Any] = None,
    on_drag_size: typing.Callable[[float], Any] = None,
    on_enable_delete: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_enable_move: typing.Callable[[bool], Any] = None,
    on_fill: typing.Callable[[bool], Any] = None,
    on_hovered_point: typing.Callable[[int], Any] = None,
    on_hovered_style: typing.Callable[[dict], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_label_display_horizontal_offset: typing.Callable[[float], Any] = None,
    on_label_display_vertical_offset: typing.Callable[[float], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_marker: typing.Callable[[str], Any] = None,
    on_names: typing.Callable[[ndarray], Any] = None,
    on_names_unique: typing.Callable[[bool], Any] = None,
    on_opacities: typing.Callable[[ndarray], Any] = None,
    on_opacity: typing.Callable[[ndarray], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_restrict_x: typing.Callable[[bool], Any] = None,
    on_restrict_y: typing.Callable[[bool], Any] = None,
    on_rotation: typing.Callable[[ndarray], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_size: typing.Callable[[ndarray], Any] = None,
    on_skew: typing.Callable[[ndarray], Any] = None,
    on_stroke: typing.Callable[[str], Any] = None,
    on_stroke_width: typing.Callable[[float], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unhovered_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_update_on_move: typing.Callable[[bool], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_x: typing.Callable[[ndarray], Any] = None,
    on_y: typing.Callable[[ndarray], Any] = None,
) -> Element[bqplot.marks.Scatter]:
    """Scatter mark.

    In the case of the Scatter mark, scales for 'x' and 'y' MUST be provided.
    The scales of other data attributes are optional. In the case where another
    data attribute than 'x' or 'y' is provided but the corresponding scale is
    missing, the data attribute is ignored.

    Attributes
    ----------
    icon: string (class-level attribute)
        Font-awesome icon for that mark
    name: string (class-level attribute)
        User-friendly name of the mark
    marker: {'circle', 'cross', 'diamond', 'square', 'triangle-down', 'triangle-up', 'arrow', 'rectangle', 'ellipse'}
        Marker shape
    colors: list of colors (default: ['steelblue'])
        List of colors of the markers. If the list is shorter than the number
        of points, the colors are reused.
    default_colors: Deprecated
        Same as `colors`, deprecated as of version 0.8.4
    fill: Bool (default: True)
        Whether to fill the markers or not
    stroke: Color or None (default: None)
        Stroke color of the marker
    stroke_width: Float (default: 1.5)
        Stroke width of the marker
    opacities: list of floats (default: [1.0])
        Default opacities of the markers. If the list is shorter than
        the number
        of points, the opacities are reused.
    default_skew: float (default: 0.5)
        Default skew of the marker.
        This number is validated to be between 0 and 1.
    default_size: nonnegative int (default: 64)
        Default marker size in pixel.
        If size data is provided with a scale, default_size stands for the
        maximal marker size (i.e. the maximum value for the 'size' scale range)
    drag_size: nonnegative float (default: 5.)
        Ratio of the size of the dragged scatter size to the default
        scatter size.
    names: numpy.ndarray (default: None)
        Labels for the points of the chart
    display_names: bool (default: True)
        Controls whether names are displayed for points in the scatter
    label_display_horizontal_offset: float (default: None)
        Adds an offset, in pixels, to the horizontal positioning of the 'names'
        label above each data point
    label_display_vertical_offset: float (default: None)
        Adds an offset, in pixels, to the vertical positioning of the 'names'
        label above each data point
    enable_move: bool (default: False)
        Controls whether points can be moved by dragging. Refer to restrict_x,
        restrict_y for more options.
    restrict_x: bool (default: False)
        Restricts movement of the point to only along the x axis. This is valid
        only when enable_move is set to True. If both restrict_x and restrict_y
        are set to True, the point cannot be moved.
    restrict_y: bool (default: False)
        Restricts movement of the point to only along the y axis. This is valid
        only when enable_move is set to True. If both restrict_x and restrict_y
        are set to True, the point cannot be moved.


    Data Attributes

    x: numpy.ndarray (default: [])
        abscissas of the data points (1d array)
    y: numpy.ndarray (default: [])
        ordinates of the data points (1d array)
    color: numpy.ndarray or None (default: None)
        color of the data points (1d array). Defaults to default_color when not
        provided or when a value is NaN
    opacity: numpy.ndarray or None (default: None)
        opacity of the data points (1d array). Defaults to default_opacity when
        not provided or when a value is NaN
    size: numpy.ndarray or None (default: None)
        size of the data points. Defaults to default_size when not provided or
        when a value is NaN
    skew: numpy.ndarray or None (default: None)
        skewness of the markers representing the data points. Defaults to
        default_skew when not provided or when a value is NaN
    rotation: numpy.ndarray or None (default: None)
        orientation of the markers representing the data points.
        The rotation scale's range is [0, 180]
        Defaults to 0 when not provided or when a value is NaN.

    Notes
    -----
    The fields which can be passed to the default tooltip are:
        All the data attributes
        index: index of the marker being hovered on
    The following are the events which can trigger interactions:
        click: left click of the mouse
        hover: mouse-over an element
    The following are the interactions which can be linked to the above events:
        tooltip: display tooltip
        add: add new points to the scatter (can only linked to click)


    """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        color=color,
        colors=colors,
        default_size=default_size,
        default_skew=default_skew,
        display_legend=display_legend,
        display_names=display_names,
        drag_color=drag_color,
        drag_size=drag_size,
        enable_delete=enable_delete,
        enable_hover=enable_hover,
        enable_move=enable_move,
        fill=fill,
        hovered_point=hovered_point,
        hovered_style=hovered_style,
        interactions=interactions,
        label_display_horizontal_offset=label_display_horizontal_offset,
        label_display_vertical_offset=label_display_vertical_offset,
        labels=labels,
        marker=marker,
        names=names,
        names_unique=names_unique,
        opacities=opacities,
        opacity=opacity,
        preserve_domain=preserve_domain,
        restrict_x=restrict_x,
        restrict_y=restrict_y,
        rotation=rotation,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        size=size,
        skew=skew,
        stroke=stroke,
        stroke_width=stroke_width,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unhovered_style=unhovered_style,
        unselected_style=unselected_style,
        update_on_move=update_on_move,
        visible=visible,
        x=x,
        y=y,
        on_apply_clip=on_apply_clip,
        on_color=on_color,
        on_colors=on_colors,
        on_default_size=on_default_size,
        on_default_skew=on_default_skew,
        on_display_legend=on_display_legend,
        on_display_names=on_display_names,
        on_drag_color=on_drag_color,
        on_drag_size=on_drag_size,
        on_enable_delete=on_enable_delete,
        on_enable_hover=on_enable_hover,
        on_enable_move=on_enable_move,
        on_fill=on_fill,
        on_hovered_point=on_hovered_point,
        on_hovered_style=on_hovered_style,
        on_interactions=on_interactions,
        on_label_display_horizontal_offset=on_label_display_horizontal_offset,
        on_label_display_vertical_offset=on_label_display_vertical_offset,
        on_labels=on_labels,
        on_marker=on_marker,
        on_names=on_names,
        on_names_unique=on_names_unique,
        on_opacities=on_opacities,
        on_opacity=on_opacity,
        on_preserve_domain=on_preserve_domain,
        on_restrict_x=on_restrict_x,
        on_restrict_y=on_restrict_y,
        on_rotation=on_rotation,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_size=on_size,
        on_skew=on_skew,
        on_stroke=on_stroke,
        on_stroke_width=on_stroke_width,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unhovered_style=on_unhovered_style,
        on_unselected_style=on_unselected_style,
        on_update_on_move=on_update_on_move,
        on_visible=on_visible,
        on_x=on_x,
        on_y=on_y,
    )
    widget_cls = bqplot.marks.Scatter
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def ScatterGL(
    apply_clip: bool = True,
    color: ndarray = None,
    colors: list = ["steelblue"],
    default_size: int = 64,
    default_skew: float = 0.5,
    display_legend: bool = False,
    display_names: bool = True,
    drag_color: str = None,
    drag_size: float = 5.0,
    enable_delete: bool = False,
    enable_hover: bool = True,
    enable_move: bool = False,
    fill: bool = True,
    hovered_point: int = None,
    hovered_style: dict = {},
    interactions: dict = {"hover": "tooltip"},
    label_display_horizontal_offset: float = 0.0,
    label_display_vertical_offset: float = 0.0,
    labels: list = [],
    marker: str = "circle",
    names: ndarray = None,
    names_unique: bool = True,
    opacities: ndarray = np.array([1.0]),
    opacity: ndarray = None,
    preserve_domain: dict = {},
    restrict_x: bool = False,
    restrict_y: bool = False,
    rotation: ndarray = None,
    scales: dict = {},
    scales_metadata: dict = {
        "x": {"orientation": "horizontal", "dimension": "x"},
        "y": {"orientation": "vertical", "dimension": "y"},
        "color": {"dimension": "color"},
        "size": {"dimension": "size"},
        "opacity": {"dimension": "opacity"},
        "rotation": {"dimension": "rotation"},
        "skew": {"dimension": "skew"},
    },
    selected: ndarray = None,
    selected_style: dict = {},
    size: ndarray = None,
    skew: ndarray = None,
    stroke: str = None,
    stroke_width: float = 1.5,
    tooltip: Element[ipywidgets.widgets.domwidget.DOMWidget] = None,
    tooltip_location: str = "mouse",
    tooltip_style: dict = {"opacity": 0.9},
    unhovered_style: dict = {},
    unselected_style: dict = {},
    update_on_move: bool = False,
    visible: bool = True,
    x: ndarray = np.array([]),
    y: ndarray = np.array([]),
    on_apply_clip: typing.Callable[[bool], Any] = None,
    on_color: typing.Callable[[ndarray], Any] = None,
    on_colors: typing.Callable[[list], Any] = None,
    on_default_size: typing.Callable[[int], Any] = None,
    on_default_skew: typing.Callable[[float], Any] = None,
    on_display_legend: typing.Callable[[bool], Any] = None,
    on_display_names: typing.Callable[[bool], Any] = None,
    on_drag_color: typing.Callable[[str], Any] = None,
    on_drag_size: typing.Callable[[float], Any] = None,
    on_enable_delete: typing.Callable[[bool], Any] = None,
    on_enable_hover: typing.Callable[[bool], Any] = None,
    on_enable_move: typing.Callable[[bool], Any] = None,
    on_fill: typing.Callable[[bool], Any] = None,
    on_hovered_point: typing.Callable[[int], Any] = None,
    on_hovered_style: typing.Callable[[dict], Any] = None,
    on_interactions: typing.Callable[[dict], Any] = None,
    on_label_display_horizontal_offset: typing.Callable[[float], Any] = None,
    on_label_display_vertical_offset: typing.Callable[[float], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_marker: typing.Callable[[str], Any] = None,
    on_names: typing.Callable[[ndarray], Any] = None,
    on_names_unique: typing.Callable[[bool], Any] = None,
    on_opacities: typing.Callable[[ndarray], Any] = None,
    on_opacity: typing.Callable[[ndarray], Any] = None,
    on_preserve_domain: typing.Callable[[dict], Any] = None,
    on_restrict_x: typing.Callable[[bool], Any] = None,
    on_restrict_y: typing.Callable[[bool], Any] = None,
    on_rotation: typing.Callable[[ndarray], Any] = None,
    on_scales: typing.Callable[[dict], Any] = None,
    on_scales_metadata: typing.Callable[[dict], Any] = None,
    on_selected: typing.Callable[[ndarray], Any] = None,
    on_selected_style: typing.Callable[[dict], Any] = None,
    on_size: typing.Callable[[ndarray], Any] = None,
    on_skew: typing.Callable[[ndarray], Any] = None,
    on_stroke: typing.Callable[[str], Any] = None,
    on_stroke_width: typing.Callable[[float], Any] = None,
    on_tooltip: typing.Callable[[Element[ipywidgets.widgets.domwidget.DOMWidget]], Any] = None,
    on_tooltip_location: typing.Callable[[str], Any] = None,
    on_tooltip_style: typing.Callable[[dict], Any] = None,
    on_unhovered_style: typing.Callable[[dict], Any] = None,
    on_unselected_style: typing.Callable[[dict], Any] = None,
    on_update_on_move: typing.Callable[[bool], Any] = None,
    on_visible: typing.Callable[[bool], Any] = None,
    on_x: typing.Callable[[ndarray], Any] = None,
    on_y: typing.Callable[[ndarray], Any] = None,
) -> Element[bqplot.marks.ScatterGL]:
    """ """

    kwargs: Dict[Any, Any] = dict(
        apply_clip=apply_clip,
        color=color,
        colors=colors,
        default_size=default_size,
        default_skew=default_skew,
        display_legend=display_legend,
        display_names=display_names,
        drag_color=drag_color,
        drag_size=drag_size,
        enable_delete=enable_delete,
        enable_hover=enable_hover,
        enable_move=enable_move,
        fill=fill,
        hovered_point=hovered_point,
        hovered_style=hovered_style,
        interactions=interactions,
        label_display_horizontal_offset=label_display_horizontal_offset,
        label_display_vertical_offset=label_display_vertical_offset,
        labels=labels,
        marker=marker,
        names=names,
        names_unique=names_unique,
        opacities=opacities,
        opacity=opacity,
        preserve_domain=preserve_domain,
        restrict_x=restrict_x,
        restrict_y=restrict_y,
        rotation=rotation,
        scales=scales,
        scales_metadata=scales_metadata,
        selected=selected,
        selected_style=selected_style,
        size=size,
        skew=skew,
        stroke=stroke,
        stroke_width=stroke_width,
        tooltip=tooltip,
        tooltip_location=tooltip_location,
        tooltip_style=tooltip_style,
        unhovered_style=unhovered_style,
        unselected_style=unselected_style,
        update_on_move=update_on_move,
        visible=visible,
        x=x,
        y=y,
        on_apply_clip=on_apply_clip,
        on_color=on_color,
        on_colors=on_colors,
        on_default_size=on_default_size,
        on_default_skew=on_default_skew,
        on_display_legend=on_display_legend,
        on_display_names=on_display_names,
        on_drag_color=on_drag_color,
        on_drag_size=on_drag_size,
        on_enable_delete=on_enable_delete,
        on_enable_hover=on_enable_hover,
        on_enable_move=on_enable_move,
        on_fill=on_fill,
        on_hovered_point=on_hovered_point,
        on_hovered_style=on_hovered_style,
        on_interactions=on_interactions,
        on_label_display_horizontal_offset=on_label_display_horizontal_offset,
        on_label_display_vertical_offset=on_label_display_vertical_offset,
        on_labels=on_labels,
        on_marker=on_marker,
        on_names=on_names,
        on_names_unique=on_names_unique,
        on_opacities=on_opacities,
        on_opacity=on_opacity,
        on_preserve_domain=on_preserve_domain,
        on_restrict_x=on_restrict_x,
        on_restrict_y=on_restrict_y,
        on_rotation=on_rotation,
        on_scales=on_scales,
        on_scales_metadata=on_scales_metadata,
        on_selected=on_selected,
        on_selected_style=on_selected_style,
        on_size=on_size,
        on_skew=on_skew,
        on_stroke=on_stroke,
        on_stroke_width=on_stroke_width,
        on_tooltip=on_tooltip,
        on_tooltip_location=on_tooltip_location,
        on_tooltip_style=on_tooltip_style,
        on_unhovered_style=on_unhovered_style,
        on_unselected_style=on_unselected_style,
        on_update_on_move=on_update_on_move,
        on_visible=on_visible,
        on_x=on_x,
        on_y=on_y,
    )
    widget_cls = bqplot.marks.ScatterGL
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Stereographic(
    allow_padding: bool = True,
    center: tuple = (0, 60),
    clip_angle: float = 179.9999,
    precision: float = 0.1,
    reverse: bool = False,
    rotate: tuple = (96, 0),
    scale_factor: float = 145.0,
    on_allow_padding: typing.Callable[[bool], Any] = None,
    on_center: typing.Callable[[tuple], Any] = None,
    on_clip_angle: typing.Callable[[float], Any] = None,
    on_precision: typing.Callable[[float], Any] = None,
    on_reverse: typing.Callable[[bool], Any] = None,
    on_rotate: typing.Callable[[tuple], Any] = None,
    on_scale_factor: typing.Callable[[float], Any] = None,
) -> Element[bqplot.scales.Stereographic]:
    """A perspective projection that uses a bijective and smooth map at every
    point except the projection point.

    The projection is not an equal-area projection but it is conformal.

    Attributes
    ----------
    scale_factor: float (default: 250)
        Specifies the scale value for the projection
    rotate: tuple (default: (96, 0))
        Degree of rotation in each axis.
    center: tuple (default: (0, 60))
        Specifies the longitude and latitude where the map is centered.
    precision: float (default: 0.1)
        Specifies the threshold for the projections adaptive resampling to the
        specified value in pixels.
    clip_angle: float (default: 90.)
        Specifies the clipping circle radius to the specified angle in degrees.


    """

    kwargs: Dict[Any, Any] = dict(
        allow_padding=allow_padding,
        center=center,
        clip_angle=clip_angle,
        precision=precision,
        reverse=reverse,
        rotate=rotate,
        scale_factor=scale_factor,
        on_allow_padding=on_allow_padding,
        on_center=on_center,
        on_clip_angle=on_clip_angle,
        on_precision=on_precision,
        on_reverse=on_reverse,
        on_rotate=on_rotate,
        on_scale_factor=on_scale_factor,
    )
    widget_cls = bqplot.scales.Stereographic
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Toolbar(
    figure: Element[bqplot.figure.Figure] = None,
    layout: Union[Dict[str, Any], Element[ipywidgets.widgets.widget_layout.Layout]] = {},
    on_figure: typing.Callable[[Element[bqplot.figure.Figure]], Any] = None,
    on_layout: typing.Callable[[Union[Dict[str, Any], Element[ipywidgets.widgets.widget_layout.Layout]]], Any] = None,
) -> Element[bqplot.toolbar.Toolbar]:
    """Default toolbar for bqplot figures.

    The default toolbar provides three buttons:

    - A *Panzoom* toggle button which enables panning and zooming the figure.
    - A *Save* button to save the figure as a png image.
    - A *Reset* button, which resets the figure position to its original
      state.

    When the *Panzoom* button is toggled to True for the first time, a new
    instance of ``PanZoom`` widget is created.
    The created ``PanZoom`` widget uses the scales of all the marks that are on
    the figure at this point.
    When the *PanZoom* widget is toggled to False, the figure retrieves its
    previous interaction.
    When the *Reset* button is pressed, the ``PanZoom`` widget is deleted and
    the figure scales reset to their initial state. We are back to the case
    where the PanZoom widget has never been set.

    If new marks are added to the figure after the panzoom button is toggled,
    and these use new scales, those scales will not be panned or zoomed,
    unless the reset button is clicked.

    Attributes
    ----------
    figure: instance of Figure
        The figure to which the toolbar will apply.


    """
    if isinstance(layout, dict):
        layout = Layout(**layout)
    kwargs: Dict[Any, Any] = dict(figure=figure, layout=layout, on_figure=on_figure, on_layout=on_layout)
    widget_cls = bqplot.toolbar.Toolbar
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)


###
def Tooltip(
    fields: list = [],
    formats: list = [],
    labels: list = [],
    layout: Union[Dict[str, Any], Element[ipywidgets.widgets.widget_layout.Layout]] = {},
    show_labels: bool = True,
    on_fields: typing.Callable[[list], Any] = None,
    on_formats: typing.Callable[[list], Any] = None,
    on_labels: typing.Callable[[list], Any] = None,
    on_layout: typing.Callable[[Union[Dict[str, Any], Element[ipywidgets.widgets.widget_layout.Layout]]], Any] = None,
    on_show_labels: typing.Callable[[bool], Any] = None,
) -> Element[bqplot.default_tooltip.Tooltip]:
    """Default tooltip widget for marks.

    Attributes
    ----------
    fields: list (default: [])
        list of names of fields to be displayed in the tooltip
        All the attributes  of the mark are accessible in the tooltip
    formats: list (default: [])
        list of formats to be applied to each of the fields.
        if no format is specified for a field, the value is displayed as it is
    labels: list (default: [])
        list of labels to be displayed in the table instead of the fields. If
        the length of labels is less than the length of fields, then the field
        names are displayed for those fields for which label is missing.
    show_labels: bool (default: True)
        Boolean attribute to enable and disable display of the
        label /field name
        as the first column along with the value


    """
    if isinstance(layout, dict):
        layout = Layout(**layout)
    kwargs: Dict[Any, Any] = dict(
        fields=fields,
        formats=formats,
        labels=labels,
        layout=layout,
        show_labels=show_labels,
        on_fields=on_fields,
        on_formats=on_formats,
        on_labels=on_labels,
        on_layout=on_layout,
        on_show_labels=on_show_labels,
    )
    widget_cls = bqplot.default_tooltip.Tooltip
    comp = react.core.ComponentWidget(widget=widget_cls)
    return react.core.Element(comp, **kwargs)
